import getColorValue from '@collab-ui/utils/lib/getColorValue';
import find from 'lodash-es/find';
import reject from 'lodash-es/reject';
import { uniqueId as uniqueId$1 } from 'lodash';
import { BrowserModule } from '@angular/platform-browser';
import isObject from 'lodash-es/isObject';
import { SelectionModel } from '@angular/cdk/collections';
import uniqueId from 'lodash-es/uniqueId';
import { take, switchMap, takeUntil, startWith } from 'rxjs/operators';
import { ENTER, SPACE, hasModifierKey, A, DOWN_ARROW, END, HOME, UP_ARROW } from '@angular/cdk/keycodes';
import { Overlay, OverlayModule, OverlayConfig } from '@angular/cdk/overlay';
import { ComponentPortal, PortalModule, PortalInjector, TemplatePortal } from '@angular/cdk/portal';
import { ActiveDescendantKeyManager, A11yModule } from '@angular/cdk/a11y';
import { NG_VALUE_ACCESSOR, FormsModule, ReactiveFormsModule, NgControl } from '@angular/forms';
import { CommonModule, DOCUMENT } from '@angular/common';
import { Component, EventEmitter, HostBinding, Input, Output, ViewEncapsulation, NgModule, CUSTOM_ELEMENTS_SCHEMA, HostListener, ViewChild, ElementRef, Injectable, Injector, forwardRef, ChangeDetectorRef, Optional, Self, ContentChildren, NgZone, ViewContainerRef, Inject } from '@angular/core';
import { Subject, defer, merge, BehaviorSubject } from 'rxjs';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// const getColorValue = require('@collab-ui/utils/lib/getColorValue');
/** @type {?} */
const iconNames = require('@collab-ui/icons/data/iconNames.json');
class IconComponent {
    constructor() {
        this.click = new EventEmitter();
        this.getIconNameClass = (/**
         * @return {?}
         */
        () => {
            /** @type {?} */
            const defaultSize = 16;
            /** @type {?} */
            const sizeFromName = Number(this.name.split('_')[1]);
            /** @type {?} */
            let nameWithSize = this.name;
            if (sizeFromName && this.fontSize) {
                nameWithSize = `${this.name.split('_')[0]}_${this.fontSize}`;
            }
            if (!sizeFromName) {
                nameWithSize = `${this.name}_${this.size || defaultSize}`;
            }
            /** @type {?} */
            const lookupIconName = nameWithSize.replace(/^(icon-)/, '');
            /** @type {?} */
            const iconNameClass = `icon-${lookupIconName}`;
            // validate Icon name with Collab Toolkit
            if (((/** @type {?} */ (iconNames))).indexOf(lookupIconName) < 0) {
                this.consoleHandler('name-error', nameWithSize);
            }
            return iconNameClass;
        });
        this.getColor = (/**
         * @return {?}
         */
        () => {
            if (!this.color) {
                return 'inherit';
            }
            if (this.color.startsWith('#')) {
                this.consoleHandler('color-warn', this.color);
                return this.color;
            }
            return getColorValue(this.color);
        });
        this.consoleHandler = (/**
         * @param {?} message
         * @param {?} data
         * @return {?}
         */
        (message, data) => {
            switch (message) {
                case 'color-warn':
                    console.warn(`[@collab-ui/angular] Icon: ${data} may not exist in the design system,` +
                        ` please use a color name from http://collab-ui.cisco.com/styles/colors`);
                    break;
                case 'name-error':
                    console.warn(`[@collab-ui/angular] Icon: Icon ${data} does not exist in the design system.` +
                        ` Visit https://icons.collab-ui.com for a list of available icons or to request a new icon.`);
                    break;
            }
        });
    }
    /**
     * @return {?}
     */
    get _class() {
        return !this.isClickable ? this.classes : '';
    }
    /**
     * @return {?}
     */
    get _color() {
        return !this.isClickable ? this.getColor() : '';
    }
    /**
     * @return {?}
     */
    get _ariaLabelledBy() {
        return !this.isClickable ? this.ariaLabelledBy : '';
    }
    /**
     * @return {?}
     */
    get _ariaLabel() {
        return !this.isClickable ? this.ariaLabel : null;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        // input validations
        if (!this.name) {
            throw new Error('Attribute "name" is required');
        }
        if (this.type && this.type !== 'white' && this.type !== '') {
            throw new Error('Attribute "type" if defined is either "" or "white"');
        }
        this.isClickable = this.click.observers.length > 0;
        this.classes =
            'cui-icon icon' +
                `${(this.name && ` ${this.getIconNameClass()}`) || ''}` +
                `${(this.className && ` ${this.className}`) || ''}` +
                ``;
        this.buttonClasses =
            'cui-button cui-button--icon' +
                `${(this.type && ` cui-button--icon-${this.type}`) || ''}` +
                `${(this.buttonClassName && ` ${this.buttonClassName}`) || ''}` +
                ``;
        /** @type {?} */
        const iconCount = uniqueId();
        this.titleId = `icon-title-${iconCount}`;
        this.descId = this.description ? `icon-desc-${iconCount}` : undefined;
    }
    /**
     * @return {?}
     */
    get ariaLabelledBy() {
        const { ariaLabel, title, description, titleId, descId } = this;
        if (!ariaLabel) {
            if (title && description) {
                return `${titleId} ${descId}`;
            }
            if (title) {
                return `${titleId}`;
            }
            if (description) {
                return `${descId}`;
            }
        }
        return null;
    }
}
IconComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-icon',
                template: `
    <button
      *ngIf="isClickable"
      class="{{ buttonClasses }}"
      [attr.aria-labelledby]="ariaLabelledBy"
      [attr.aria-label]="ariaLabel"
    >
      <i class="{{ classes }}" [style.color]="getColor()"></i>
    </button>
  `,
                encapsulation: ViewEncapsulation.None
            }] }
];
/** @nocollapse */
IconComponent.ctorParameters = () => [];
IconComponent.propDecorators = {
    ariaLabel: [{ type: Input }],
    buttonClassName: [{ type: Input }],
    color: [{ type: Input }],
    className: [{ type: Input }],
    description: [{ type: Input }],
    name: [{ type: Input }],
    size: [{ type: Input }],
    fontSize: [{ type: Input }],
    title: [{ type: Input }],
    type: [{ type: Input }],
    click: [{ type: Output }],
    _class: [{ type: HostBinding, args: ['class',] }],
    _color: [{ type: HostBinding, args: ['style.color',] }],
    _ariaLabelledBy: [{ type: HostBinding, args: ['attr.aria-labelledby',] }],
    _ariaLabel: [{ type: HostBinding, args: ['attr.aria-label',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class IconModule {
}
IconModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                declarations: [IconComponent],
                exports: [IconComponent],
                schemas: [CUSTOM_ELEMENTS_SCHEMA]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AlertBannerComponent {
    constructor() {
        /**
         * Sets the visibility of AlertBanner's close button | false
         */
        this.closable = false;
        /**
         * Sets the AlertBanner type | 'info'
         */
        this.type = 'info';
        /**
         * Event emitted when the user presses on the AlertBanner's close button or hits the esc key
         */
        this.hide = new EventEmitter();
    }
    /**
     * @param {?} _
     * @return {?}
     */
    handleKeydown(_) {
        this.hide.emit();
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (!this.hide.observers.length) {
            throw new Error('Observer for "hide" is required');
        }
    }
    /**
     * @return {?}
     */
    onClick() {
        this.hide.emit();
    }
}
AlertBannerComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-alert-banner',
                template: `
    <div [ngClass]="['cui-alert-banner', 'cui-alert-banner--' + type]">
      <div class="cui-alert-banner__text">
        <ng-content></ng-content>
      </div>
      <div
        *ngIf="closable"
        class="cui-alert-banner__close"
        (click)="onClick()"
        (keydown)="handleKeydown($event)"
        tabindex="0"
        role="button"
      >
        <cui-icon name="cancel_16"></cui-icon>
      </div>
    </div>
  `
            }] }
];
/** @nocollapse */
AlertBannerComponent.ctorParameters = () => [];
AlertBannerComponent.propDecorators = {
    closable: [{ type: Input }],
    type: [{ type: HostBinding, args: ['attr.type',] }, { type: Input }],
    hide: [{ type: Output }],
    handleKeydown: [{ type: HostListener, args: ['document:keydown.escape', ['$event'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AlertBannerModule {
}
AlertBannerModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, IconModule],
                declarations: [AlertBannerComponent],
                exports: [AlertBannerComponent],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ButtonComponent {
    /**
     * @param {?} el
     */
    constructor(el) {
        this.el = el;
        /**
         * \@option Active Sets button to active | false
         */
        this.active = false;
        /** @option AriaLabel Text to display for blindness accessibility features | '' */
        /** @option AriaLabelledBy ID to reference for blindness accessibility feature | '' */
        /**
         * \@option Creates a circle shaped button | false
         */
        this.circle = false;
        /**
         * \@option Optional css class string | ''
         */
        this.class = '';
        /**
         * \@option Sets optional button color | ''
         */
        this.color = '';
        /**
         * \@option Sets the attribute disabled to Button | false
         */
        this.disabled = false;
        /**
         * \@option Sets expand css styling to widen the Button | false
         */
        this.expand = false;
        /**
         * \@option Sets href for an anchor tagged button | ''
         */
        this.href = '';
        /**
         * \@option Activates the loading animation and sets the button to disabled | false
         */
        this.loading = false;
        /**
         * \@option Boolean to remove Button's default style | false
         */
        this.removeStyle = false;
        /**
         * \@option Size value | '36'
         */
        this.size = '36';
        /**
         * \@option Html button type | 'button'
         */
        this.type = 'button';
        this.getColor = (/**
         * @return {?}
         */
        () => {
            if (this.removeStyle) {
                return false;
            }
            return this.color === 'none' ? 'color-none' : this.color;
        });
        this.getSize = (/**
         * @return {?}
         */
        () => {
            if (this.removeStyle) {
                return false;
            }
            /** @type {?} */
            const validButtonSize = this.checkButtonSize();
            if (!this.circle && !validButtonSize) {
                console.warn(`[@collab-ui/angular] Button: selected size is not supported
         for non-circular button. Size will default to 36`);
                return '36';
            }
            else {
                return this.size === 'none' ? 'size-none' : this.size;
            }
        });
        this.checkButtonSize = (/**
         * @return {?}
         */
        () => ['none', '28', '36', '40', '52', 28, 36, 40, 52].includes(this.size));
    }
    /**
     * @return {?}
     */
    get activeOption() {
        return this.active || null;
    }
    /**
     * @return {?}
     */
    get disabledOption() {
        return this.disabled || null;
    }
    /**
     * @return {?}
     */
    get hrefOption() {
        return this.href || null;
    }
    /**
     * @return {?}
     */
    get typeOption() {
        return this.type || 'button';
    }
    /**
     * @return {?}
     */
    get className() {
        return ('cui-button' +
            `${(this.circle && ` cui-button--circle`) || ''}` +
            `${(this.removeStyle && ' cui-button--none') || ''}` +
            `${(this.getSize() && ` cui-button--${this.getSize()}`) || ''}` +
            `${(this.getColor() && ` cui-button--${this.getColor()}`) || ''}` +
            `${(this.expand && ` cui-button--expand`) || ''}` +
            `${(this.class && ` ${this.class}`) || ''}` +
            `${(this.disabled && ` cui-button--disabled`) || ''}` +
            `${(this.active && !this.disabled && ` active`) || ''}` +
            ``);
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.disabled = this.disabled || this.loading || null;
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        this.initWidth = `${this.element.nativeElement.offsetWidth + 0.001}px`;
        if (this.type &&
            this.type !== 'button' &&
            this.type !== 'reset' &&
            this.type !== 'submit') {
            throw new Error('cui-button: Button type must be one of the following: button, reset, submit');
        }
    }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        setTimeout((/**
         * @return {?}
         */
        () => this.setAriaLabel()), 100);
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (changes.loading && changes.loading.currentValue) {
            this.width = this.initWidth;
        }
        else {
            this.width = '';
        }
    }
    /**
     * @return {?}
     */
    get opacity() {
        return this.loading ? 0 : 1;
    }
    /**
     * @private
     * @return {?}
     */
    setAriaLabel() {
        /** @type {?} */
        let ariaLabel = this.getAttr('aria-label');
        if (!ariaLabel) {
            if (this.element.nativeElement.children.length > 0) {
                throw new Error('cui-button: content is not a string, you must add an "ariaLabel" for accessibility.');
            }
            else {
                ariaLabel = this.element.nativeElement.innerText;
            }
        }
    }
    /**
     * @private
     * @param {?} attr
     * @return {?}
     */
    getAttr(attr) {
        return this.el.nativeElement.getAttribute(attr);
    }
}
ButtonComponent.decorators = [
    { type: Component, args: [{
                selector: 'button[cui-button], a[cui-button], input[cui-button]',
                exportAs: 'cuiButton',
                template: `
    <div #divBody>
      <cui-loading *ngIf="loading"></cui-loading>
      <span [ngStyle]="{ opacity: opacity }">
        <ng-content></ng-content>
      </span>
    </div>
  `
            }] }
];
/** @nocollapse */
ButtonComponent.ctorParameters = () => [
    { type: ElementRef }
];
ButtonComponent.propDecorators = {
    active: [{ type: Input }],
    circle: [{ type: Input }],
    class: [{ type: Input }],
    color: [{ type: Input }],
    disabled: [{ type: Input }],
    expand: [{ type: Input }],
    href: [{ type: Input }],
    loading: [{ type: Input }],
    removeStyle: [{ type: Input }],
    size: [{ type: Input }],
    type: [{ type: Input }],
    element: [{ type: ViewChild, args: ['divBody',] }],
    loaderChild: [{ type: ViewChild, args: ['loader',] }],
    width: [{ type: HostBinding, args: ['attr.width',] }],
    alt: [{ type: HostBinding, args: ['attr.alt',] }],
    activeOption: [{ type: HostBinding, args: ['attr.active',] }],
    disabledOption: [{ type: HostBinding, args: ['attr.disabled',] }],
    hrefOption: [{ type: HostBinding, args: ['attr.href',] }],
    typeOption: [{ type: HostBinding, args: ['attr.type',] }],
    className: [{ type: HostBinding, args: ['class',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class LoadingComponent {
    constructor() {
        /**
         * \@option Prop to make the Loading animation small | false
         */
        this.small = false;
    }
    /**
     * @return {?}
     */
    get className() {
        return `cui-loading ${this.small ? 'cui-loading--small' : ''}`;
    }
}
LoadingComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-loading, CSLoading',
                template: `
    <span class="cui-loading__icon"></span>
    <span class="cui-loading__icon"></span>
    <span class="cui-loading__icon"></span>
  `,
                encapsulation: ViewEncapsulation.None
            }] }
];
/** @nocollapse */
LoadingComponent.ctorParameters = () => [];
LoadingComponent.propDecorators = {
    small: [{ type: Input }],
    className: [{ type: HostBinding, args: ['class',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class LoadingModule {
    /**
     * @return {?}
     */
    static forRoot() {
        return {
            ngModule: LoadingModule,
            providers: [],
        };
    }
}
LoadingModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                exports: [LoadingComponent],
                declarations: [LoadingComponent],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ButtonModule {
    /**
     * @return {?}
     */
    static forRoot() {
        return {
            ngModule: ButtonModule,
            providers: [],
        };
    }
}
ButtonModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, LoadingModule.forRoot()],
                exports: [ButtonComponent],
                declarations: [ButtonComponent],
                schemas: [CUSTOM_ELEMENTS_SCHEMA]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AvatarComponent {
    /**
     * @param {?} elementRef
     */
    constructor(elementRef) {
        this.elementRef = elementRef;
        this.ariaLabel = '';
        /**
         * \@option Image alt tag | ''
         */
        this.alt = '';
        /**
         * \@option Set Avatar background color | ''
         */
        this.backgroundColor = '';
        /**
         * \@option Optional css class string for button | ''
         */
        this.buttonClassName = '';
        /**
         * \@option Set Avatar text color | ''
         */
        this.color = '';
        /**
         * \@option Set existance of a failureBadge on the Avatar | false
         */
        this.failureBadge = false;
        /**
         * \@option Set existance of a notification on the Avatar | false
         */
        this.hasNotification = false;
        /**
         * \@option Set the visibility of Avatar's default tooltip | false
         */
        this.hideDefaultTooltip = false;
        /**
         * \@option Set if Avatar's content is decrypting | false
         */
        this.isDecrypting = false;
        /**
         * \@option Set existance of Avatar's Overview | false
         */
        this.isOverview = false;
        this.click = new EventEmitter();
        this._size = 'medium';
        this._type = '';
        this.clickable = false;
        this.isImageLoaded = false;
        this.isImageErrored = false;
    }
    /**
     * \@option Set the size of the Avatar from one of the preconfigured options | 'medium'
     * @return {?}
     */
    get size() {
        return this._size;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set size(value) {
        if (!this.clickable) {
            this.elementRef.nativeElement.classList.remove(`cui-avatar--${this._size}`);
            this.elementRef.nativeElement.classList.add(`cui-avatar--${value}`);
        }
        this._size = value;
    }
    /**
     * \@option Optional Avatar color theme | null
     * @return {?}
     */
    get theme() {
        return this._theme;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set theme(value) {
        if (!this.clickable) {
            this.elementRef.nativeElement.classList.remove(`cui-avatar--${this._theme}`);
            this.elementRef.nativeElement.classList.add(`cui-avatar--${value}`);
        }
        this._theme = value;
    }
    /**
     * \@option optional Avatar type | ''
     * @return {?}
     */
    get type() {
        return this._type;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set type(value) {
        if (!this.clickable) {
            this.elementRef.nativeElement.classList.remove(`cui-avatar--${this._type}`);
            this.elementRef.nativeElement.classList.add(`cui-avatar--${value}`);
        }
        this._type = value;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        /** @type {?} */
        const img = this.image;
        if (img && img.nativeElement.complete) {
            this.handleImgLoaded();
        }
        this.clickable = this.click.observers.length > 0;
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (changes.src && changes.src.previousValue !== changes.src.currentValue) {
            this.handleImgChange();
        }
    }
    /**
     * @return {?}
     */
    handleImgChange() {
        this.isImageLoaded = false;
        this.isImageErrored = false;
    }
    /**
     * @return {?}
     */
    handleImgError() {
        this.isImageErrored = true;
    }
    /**
     * @return {?}
     */
    handleImgLoaded() {
        this.isImageLoaded = true;
    }
    /**
     * @return {?}
     */
    getInitials() {
        if (!this.title || !this.title.replace(/\s/g, '').length) {
            return '';
        }
        /** @type {?} */
        const letters = [];
        /** @type {?} */
        const words = this.title.trim().split(/ +/);
        /** @type {?} */
        const repeatTimes = Math.min((this.type === 'group' && 1) || 2, words.length);
        for (let i = 0; i < repeatTimes; i++) {
            letters.push(String.fromCodePoint(words[i].codePointAt(0)));
        }
        return letters.join('');
    }
}
AvatarComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-avatar',
                template: `
    <button
      cui-button
      *ngIf="clickable; else avatarTpl"
      [attr.aria-label]="ariaLabel"
      [class]="buttonClassName"
      [circle]="true"
      [removeStyle]="true"
    >
      <ng-container *ngTemplateOutlet="avatarTpl"></ng-container>
    </button>

    <ng-template #avatarTpl>
      <div
        *ngIf="clickable; else overlayTpl"
        class="cui-avatar"
        [ngClass]="[
          'cui-avatar--clickable',
          (type && 'cui-avatar--' + type) || '',
          (size && 'cui-avatar--' + size) || '',
          (theme && 'cui-avatar--' + theme) || '',
          (isDecrypting && 'cui-decrypting') || '',
          className || ''
        ]"
        [title]="!hideDefaultTooltip ? title : ''"
      >
        <ng-container *ngTemplateOutlet="overlayTpl"></ng-container>
      </div>
    </ng-template>

    <ng-template #overlayTpl>
      <ng-container *ngTemplateOutlet="childrenTpl"></ng-container>
      <cui-loading *ngIf="type === 'typing'"></cui-loading>
      <span *ngIf="failureBadge" class="cui-avatar__failure-badge"></span>
      <span
        *ngIf="hasNotification"
        class="cui-avatar__notification-badge"
      ></span>
    </ng-template>

    <ng-template #childrenTpl>
      <span
        *ngIf="type === 'self'; else imageTpl"
        class="cui-avatar__self"
        [ngStyle]="{
          'background-color': backgroundColor,
          color: color
        }"
      >
        <cui-icon
          [name]="
            size === 40 || size === 'medium'
              ? 'chat-active_18'
              : 'chat-active_16'
          "
          color="blue-50"
        ></cui-icon>
      </span>
    </ng-template>

    <ng-template #imageTpl>
      <ng-container *ngIf="src && !isImageErrored; else iconTpl">
        <ng-container *ngIf="title && !isImageLoaded">
          <ng-container *ngTemplateOutlet="letterTpl"></ng-container>
        </ng-container>
        <img
          #image
          [alt]="alt"
          class="cui-avatar__img"
          [ngClass]="{ 'cui-avatar__img--hidden': !isImageLoaded }"
          draggable="false"
          (error)="handleImgError()"
          (load)="handleImgLoaded()"
          [src]="src"
        />
      </ng-container>
    </ng-template>

    <ng-template #iconTpl>
      <span
        *ngIf="icon; else letterTpl"
        class="cui-avatar__icon"
        [ngClass]="{ 'cui-avatar__icon--overview': isOverview }"
        [ngStyle]="{
          'background-color': backgroundColor,
          color: color
        }"
      >
        <cui-icon [name]="icon"></cui-icon>
      </span>
    </ng-template>

    <ng-template #letterTpl>
      <span
        class="cui-avatar__letter"
        [ngClass]="{
          'cui-decrypting': isDecrypting,
          'cui-avatar__img--hidden': isImageLoaded
        }"
        [ngStyle]="{
          'background-color': backgroundColor,
          color: color
        }"
        >{{ getInitials() }}</span
      >
    </ng-template>
  `,
                host: {
                    '[attr.title]': '!hideDefaultTooltip ? title : ""',
                    class: 'cui-avatar cui-avatar--medium',
                    '[class.cui-decrypting]': '!clickable && isDecrypting',
                }
            }] }
];
/** @nocollapse */
AvatarComponent.ctorParameters = () => [
    { type: ElementRef }
];
AvatarComponent.propDecorators = {
    ariaLabel: [{ type: Input }],
    alt: [{ type: Input }],
    backgroundColor: [{ type: Input }],
    buttonClassName: [{ type: Input }],
    className: [{ type: Input }],
    color: [{ type: Input }],
    failureBadge: [{ type: Input }],
    hasNotification: [{ type: Input }],
    hideDefaultTooltip: [{ type: Input }],
    icon: [{ type: Input }],
    isDecrypting: [{ type: Input }],
    isOverview: [{ type: Input }],
    size: [{ type: Input }],
    src: [{ type: Input }],
    theme: [{ type: Input }],
    title: [{ type: Input }],
    type: [{ type: Input }],
    click: [{ type: Output }],
    image: [{ type: ViewChild, args: ['image',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AvatarModule {
}
AvatarModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, ButtonModule, IconModule, LoadingModule],
                declarations: [AvatarComponent],
                exports: [AvatarComponent],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CompositeAvatarComponent {
    constructor() {
        this.className = '';
        this.size = 'medium';
    }
    /**
     * @return {?}
     */
    get _class() {
        return ('cui-composite-avatar' +
            `${(this.size && ` cui-composite-avatar--${this.size}`) || ''}` +
            `${(this.className && ` ${this.className}`) || ''}`);
    }
}
CompositeAvatarComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-composite-avatar',
                template: `
    <ng-content></ng-content>
  `
            }] }
];
/** @nocollapse */
CompositeAvatarComponent.ctorParameters = () => [];
CompositeAvatarComponent.propDecorators = {
    className: [{ type: Input }],
    size: [{ type: Input }],
    _class: [{ type: HostBinding, args: ['class',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CompositeAvatarModule {
}
CompositeAvatarModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, AvatarModule],
                declarations: [CompositeAvatarComponent],
                exports: [CompositeAvatarComponent],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// Used internally by the alert meeting service.
class AlertMeetingComponent {
    constructor() {
        this.attendees = [];
        this.closeAriaLabel = 'close';
        this.message = '';
        this.remindAriaLabel = 'snooze';
        this.status = '';
        this.title = '';
        this.mouseclick = new EventEmitter();
        this.hide = new EventEmitter();
        this.snooze = new EventEmitter();
    }
    /**
     * @return {?}
     */
    get contentClasses() {
        return [
            'cui-alert__content',
            this.snoozeButtonVisible ? '' : 'cui-alert__content--wide'
        ];
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (!this.avatar && this.attendees.length === 0) {
            throw new Error('MeetingAlert needs at least one attendee to render an avatar.');
        }
    }
    /**
     * @return {?}
     */
    onClick() {
        this.mouseclick.emit(this.key);
    }
    /**
     * @return {?}
     */
    onSnooze() {
        this.snooze.emit(this.key);
    }
    /**
     * @return {?}
     */
    onHide() {
        this.hide.emit(this.key);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onKeydown(event) {
        if (event.code === 'Space' || event.code === 'Enter') {
            this.onClick();
            event.preventDefault();
        }
    }
}
AlertMeetingComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-alert-meeting',
                template: `
    <ng-container *ngIf="avatar; else avatarTemplate" [ngTemplateOutlet]="avatar"></ng-container>
    <div [ngClass]="contentClasses">
      <div class="cui-alert__title" [attr.title]="title">{{ title }}</div>
      <div class="cui-alert__status">{{ status }}</div>
      <div class="cui-alert__message" [attr.title]="message">{{ message }}</div>
    </div>
    <div class="cui-alert__button" *ngIf="snoozeButtonVisible">
      <button
        cui-button
        [attr.aria-label]="remindAriaLabel"
        [circle]="true"
        size="44"
        tabindex="0"
        (click)="onSnooze()"
      >
        <cui-icon name="alarm_16"></cui-icon>
      </button>
    </div>
    <div class="cui-alert__button">
      <button
        cui-button
        [attr.aria-label]="closeAriaLabel"
        [circle]="true"
        size="44"
        tabindex="0"
        (click)="onHide()"
      >
        <cui-icon name="cancel_16"></cui-icon>
      </button>
    </div>

    <ng-template #avatarTemplate>
      <cui-composite-avatar *ngIf="attendees.length >= 2">
        <cui-avatar
          [title]="attendees[0].title"
          [alt]="attendees[0].alt"
          [src]="attendees[0].src"
        ></cui-avatar>
        <cui-avatar
          [title]="attendees[1].title"
          [alt]="attendees[1].alt"
          [src]="attendees[1].src"
        ></cui-avatar>
      </cui-composite-avatar>
      <cui-avatar
        *ngIf="attendees.length === 1"
        [title]="attendees[0].title"
        [alt]="attendees[0].alt"
        [src]="attendees[0].src"
      ></cui-avatar>
    </ng-template>
  `,
                host: {
                    'class': 'cui-alert cui-alert--meeting',
                    '[attr.role]': 'clickable ? "button" : null',
                    '[attr.tabindex]': 'clickable ? "0" : null',
                    '(click)': 'clickable ? onClick() : null',
                    '(keydown)': 'clickable ? onKeydown($event) : null',
                }
            }] }
];
AlertMeetingComponent.propDecorators = {
    key: [{ type: Input }],
    attendees: [{ type: Input }],
    avatar: [{ type: Input }],
    clickable: [{ type: Input }],
    closeAriaLabel: [{ type: Input }],
    message: [{ type: Input }],
    remindAriaLabel: [{ type: Input }],
    snoozeButtonVisible: [{ type: Input }],
    status: [{ type: Input }],
    title: [{ type: Input }],
    mouseclick: [{ type: Output }],
    hide: [{ type: Output }],
    snooze: [{ type: Output }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// Used internally by the alert meeting service.
class AlertMeetingContainerComponent {
    constructor() {
        this._onDestroy = new Subject();
        this.alertList = [];
    }
    /**
     * @param {?} alert
     * @return {?}
     */
    addAlert(alert) {
        alert.key = uniqueId('alert_meeting_');
        this.alertList.push(alert);
        return alert.key;
    }
    /**
     * @param {?} key
     * @return {?}
     */
    removeAlert(key) {
        this.alertList = reject(this.alertList, { key });
        if (!this.alertList.length) {
            this.destroy();
        }
    }
    /**
     * @param {?} key
     * @return {?}
     */
    onClick(key) {
        /** @type {?} */
        const alert = find(this.alertList, { key });
        if (alert && alert.onClick) {
            alert.onClick();
        }
        this.removeAlert(key);
    }
    /**
     * @param {?} key
     * @return {?}
     */
    onHide(key) {
        /** @type {?} */
        const alert = find(this.alertList, { key });
        if (alert && alert.onHide) {
            alert.onHide();
        }
        this.removeAlert(key);
    }
    /**
     * @param {?} key
     * @return {?}
     */
    onSnooze(key) {
        /** @type {?} */
        const alert = find(this.alertList, { key });
        if (alert && alert.onSnooze) {
            alert.onSnooze();
        }
        this.removeAlert(key);
    }
    /**
     * @return {?}
     */
    destroy() {
        this._onDestroy.next();
        this._onDestroy.complete();
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.destroy();
    }
}
AlertMeetingContainerComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-alert-meeting-container',
                template: `
    <cui-alert-meeting *ngFor="let alert of alertList"
      [key]="alert.key"
      [attendees]="alert.attendees"
      [avatar]="alert.avatar"
      [clickable]="!!alert.onClick"
      [closeAriaLabel]="alert.closeAriaLabel"
      [message]="alert.message"
      [remindAriaLabel]="alert.remindAriaLabel"
      [snoozeButtonVisible]="!!alert.onSnooze"
      [status]="alert.status"
      [title]="alert.title"
      (mouseclick)="onClick($event)"
      (hide)="onHide($event)"
      (snooze)="onSnooze($event)"
    ></cui-alert-meeting>
  `,
                host: {
                    'class': 'cui-alert__container cui-alert__container--bottom-right'
                }
            }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// Configuration used when opening an alert.
class AlertMeetingConfig {
    constructor() {
        /**
         * Optional attendee array. If more than one attendee, a Composite Avatar with only the first 2 attendees is created.
         */
        this.attendees = [];
        /**
         * Optional aria label for the close button | 'close'
         */
        this.closeAriaLabel = 'close';
        /**
         * Optional AlertMeeting Message | ''
         */
        this.message = '';
        /**
         * Optional aria label for snooze buton | 'snooze'
         */
        this.remindAriaLabel = 'snooze';
        /**
         * Optional AlertMeeting status | ''
         */
        this.status = '';
        /**
         * Optional AlertMeeting title | ''
         */
        this.title = '';
        /**
         * Optional callback function invoked on click of alert | null
         */
        this.onClick = null;
        /**
         * Optional handler invoked when the user presses on the Alert's close button or hit's the esc key | null
         */
        this.onHide = null;
        /**
         * Optional callback function invoked when the snooze button is clicked | null
         */
        this.onSnooze = null;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AlertMeetingService {
    /**
     * @param {?} _overlay
     */
    constructor(_overlay) {
        this._overlay = _overlay;
        this._overlayRef = null;
        this._alertContainerRef = null;
    }
    /**
     * @param {?} config
     * @return {?}
     */
    show(config) {
        if (!this._overlayRef) {
            this._overlayRef = this._overlay.create();
        }
        /** @type {?} */
        const container = this._attachAlertContainer(this._overlayRef);
        /** @type {?} */
        const _config = Object.assign({}, new AlertMeetingConfig(), config);
        return container.addAlert(_config);
    }
    /**
     * @param {?} key
     * @return {?}
     */
    hide(key) {
        if (this._alertContainerRef) {
            this._alertContainerRef.instance.removeAlert(key);
        }
    }
    /**
     * @private
     * @param {?} overlayRef
     * @return {?}
     */
    _attachAlertContainer(overlayRef) {
        if (this._alertContainerRef) {
            return this._alertContainerRef.instance;
        }
        /** @type {?} */
        const containerPortal = new ComponentPortal(AlertMeetingContainerComponent);
        this._alertContainerRef = overlayRef.attach(containerPortal);
        /** @type {?} */
        const onDestroySub = this._alertContainerRef.instance._onDestroy.subscribe((/**
         * @return {?}
         */
        () => {
            onDestroySub.unsubscribe();
            this._dispose();
        }));
        return this._alertContainerRef.instance;
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this._alertContainerRef) {
            this._alertContainerRef.instance.destroy();
        }
    }
    /**
     * @private
     * @return {?}
     */
    _dispose() {
        this._alertContainerRef = null;
        this._overlayRef.dispose();
        this._overlayRef = null;
    }
}
AlertMeetingService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
AlertMeetingService.ctorParameters = () => [
    { type: Overlay }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AlertMeetingModule {
}
AlertMeetingModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    OverlayModule,
                    PortalModule,
                    CommonModule,
                    AvatarModule,
                    ButtonModule,
                    CompositeAvatarModule,
                    IconModule,
                ],
                declarations: [AlertMeetingComponent, AlertMeetingContainerComponent],
                exports: [AlertMeetingContainerComponent],
                entryComponents: [AlertMeetingContainerComponent],
                providers: [AlertMeetingService]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// Used internally by the alert service.
class AlertComponent {
    constructor() {
        this.dismissed = new EventEmitter();
    }
    /**
     * @return {?}
     */
    get className() {
        return `cui-alert cui-alert--${this.type}`;
    }
    /**
     * @return {?}
     */
    dismiss() {
        this.dismissed.emit(this.key);
    }
}
AlertComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-alert',
                template: `
    <div class="cui-alert__icon"></div>
    <div class="cui-alert__content">
      <div class="cui-alert__title">{{ title }}</div>
      <div class="cui-alert__message">{{ message }}</div>
    </div>
    <div class="cui-alert__button" *ngIf="closable">
      <button
        cui-button
        #closeButton
        [attr.aria-label]="ariaLabel"
        [circle]="true"
        size="44"
        (click)="dismiss()"
      >
        <cui-icon name="cancel_16"></cui-icon>
      </button>
    </div>
  `
            }] }
];
/** @nocollapse */
AlertComponent.ctorParameters = () => [];
AlertComponent.propDecorators = {
    key: [{ type: Input }],
    type: [{ type: Input }],
    ariaLabel: [{ type: Input }],
    closable: [{ type: Input }],
    message: [{ type: Input }],
    title: [{ type: Input }],
    className: [{ type: HostBinding, args: ['class',] }],
    dismissed: [{ type: Output }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Configuration used when opening an alert.
 */
class AlertConfig {
    constructor() {
        this.ariaLabel = 'Close';
        /**
         * To show/hide Close button of the Alert | true
         */
        this.closable = true;
        /**
         * Optional Alert message | ''
         */
        this.message = '';
        /**
         * Handler invoked when the user presses on the Alert's close button | null
         */
        this.onHide = null;
        /**
         * Display newest alert messages first | true
         */
        this.orderNewest = true;
        /**
         * Define alert's position with css class name: 'bottom-right'
         */
        this.position = 'bottom-right';
        /**
         * Optional Alert title | ''
         */
        this.title = '';
        /**
         * Sets the type of the Alert | 'info'
         */
        this.type = 'info';
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Used internally by the alert service.
 */
class AlertContainerComponent {
    /**
     * @param {?} alertConfig
     */
    constructor(alertConfig) {
        this.alertConfig = alertConfig;
        this.role = 'alert';
        this._onDestroy = new Subject();
        this.alertList = [];
    }
    /**
     * @return {?}
     */
    get className() {
        return `cui-alert__container cui-alert__container--${this.alertConfig.position}`;
    }
    /**
     * @param {?} alert
     * @return {?}
     */
    addAlert(alert) {
        alert.key = uniqueId('alert_');
        this.alertList = this.alertConfig.orderNewest
            ? [alert, ...this.alertList]
            : [...this.alertList, alert];
        return alert.key;
    }
    /**
     * @param {?} key
     * @return {?}
     */
    removeAlert(key) {
        /** @type {?} */
        const alert = find(this.alertList, { key: key });
        if (alert && alert.onHide) {
            alert.onHide();
        }
        this.alertList = reject(this.alertList, { key: key });
        if (!this.alertList.length) {
            this.destroy();
        }
    }
    /**
     * @return {?}
     */
    destroy() {
        this._onDestroy.next();
        this._onDestroy.complete();
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.destroy();
    }
}
AlertContainerComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-alert-container',
                template: `
    <ng-container *ngFor="let alert of alertList">
      <cui-alert
        [key]="alert.key"
        [ariaLabel]="alert.ariaLabel"
        [type]="alert.type"
        [title]="alert.title"
        [message]="alert.message"
        [closable]="alert.closable"
        (dismissed)="removeAlert($event)"
      ></cui-alert>
    </ng-container>
  `
            }] }
];
/** @nocollapse */
AlertContainerComponent.ctorParameters = () => [
    { type: AlertConfig }
];
AlertContainerComponent.propDecorators = {
    role: [{ type: HostBinding, args: ['attr.role',] }],
    className: [{ type: HostBinding, args: ['class',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AlertService {
    /**
     * @param {?} _overlay
     * @param {?} _injector
     */
    constructor(_overlay, _injector) {
        this._overlay = _overlay;
        this._injector = _injector;
        this._overlayRef = null;
        this._alertContainerRef = null;
    }
    /**
     * @param {?} title
     * @param {?} message
     * @param {?=} config
     * @return {?}
     */
    info(title, message, config) {
        return this._open('info', title, message, config);
    }
    /**
     * @param {?} title
     * @param {?} message
     * @param {?=} config
     * @return {?}
     */
    success(title, message, config) {
        return this._open('success', title, message, config);
    }
    /**
     * @param {?} title
     * @param {?} message
     * @param {?=} config
     * @return {?}
     */
    warning(title, message, config) {
        return this._open('warning', title, message, config);
    }
    /**
     * @param {?} title
     * @param {?} message
     * @param {?=} config
     * @return {?}
     */
    error(title, message, config) {
        return this._open('error', title, message, config);
    }
    /**
     * @param {?} key
     * @return {?}
     */
    hide(key) {
        if (this._alertContainerRef) {
            this._alertContainerRef.instance.removeAlert(key);
        }
    }
    /**
     * @private
     * @param {?} type
     * @param {?} title
     * @param {?} message
     * @param {?=} config
     * @return {?}
     */
    _open(type, title, message, config) {
        /** @type {?} */
        const _config = Object.assign({}, new AlertConfig(), config);
        _config.type = type;
        _config.title = title;
        _config.message = message;
        if (!this._overlayRef) {
            this._overlayRef = this._overlay.create();
        }
        /** @type {?} */
        const container = this._attachAlertContainer(this._overlayRef, _config);
        return container.addAlert(_config);
    }
    /**
     * @private
     * @param {?} overlayRef
     * @param {?} config
     * @return {?}
     */
    _attachAlertContainer(overlayRef, config) {
        if (this._alertContainerRef) {
            return this._alertContainerRef.instance;
        }
        /** @type {?} */
        const injector = new PortalInjector(this._injector, new WeakMap([[AlertConfig, config]]));
        /** @type {?} */
        const containerPortal = new ComponentPortal(AlertContainerComponent, null, injector);
        this._alertContainerRef = overlayRef.attach(containerPortal);
        /** @type {?} */
        const onDestroySub = this._alertContainerRef.instance._onDestroy.subscribe((/**
         * @return {?}
         */
        () => {
            onDestroySub.unsubscribe();
            this._dispose();
        }));
        return this._alertContainerRef.instance;
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this._alertContainerRef) {
            this._alertContainerRef.instance.destroy();
        }
    }
    /**
     * @private
     * @return {?}
     */
    _dispose() {
        this._alertContainerRef = null;
        this._overlayRef.dispose();
        this._overlayRef = null;
    }
}
AlertService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
AlertService.ctorParameters = () => [
    { type: Overlay },
    { type: Injector }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AlertModule {
}
AlertModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    OverlayModule,
                    PortalModule,
                    CommonModule,
                    ButtonModule,
                    IconModule,
                ],
                declarations: [AlertComponent, AlertContainerComponent],
                exports: [AlertContainerComponent],
                entryComponents: [AlertContainerComponent],
                providers: [AlertService],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class BadgeComponent {
    /**
     * @param {?} elementRef
     */
    constructor(elementRef) {
        this.elementRef = elementRef;
        /**
         * \@option Sets the optional rounded class | false
         */
        this.rounded = false;
        this._color = null;
    }
    /**
     * \@option Sets optional button color | null
     * @param {?} color
     * @return {?}
     */
    set color(color) {
        if (this._color) {
            this.elementRef.nativeElement.classList.remove(`cui-badge--${this._color}`);
        }
        this.elementRef.nativeElement.classList.add(`cui-badge--${color}`);
        this._color = color;
    }
}
BadgeComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-badge',
                template: `
    <ng-content></ng-content>
  `,
                host: {
                    class: 'cui-badge',
                    '[class.cui-badge--round]': 'rounded',
                }
            }] }
];
/** @nocollapse */
BadgeComponent.ctorParameters = () => [
    { type: ElementRef }
];
BadgeComponent.propDecorators = {
    rounded: [{ type: Input }],
    color: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class BadgeModule {
}
BadgeModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                declarations: [BadgeComponent],
                exports: [BadgeComponent],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// tslint:disable:no-use-before-declare
/** @type {?} */
const CUSTOM_CHECKBOX_CONTROL_VALUE_ACCESSOR = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef((/**
     * @return {?}
     */
    () => CheckboxComponent)),
    multi: true,
};
// tslint:enable:no-use-before-declare
class CheckboxComponent {
    /**
     * @param {?} cdr
     */
    constructor(cdr) {
        this.cdr = cdr;
        /**
         * \@option String value that corresponds with Checkbox  | ''
         */
        this.value = '';
        /**
         * \@option index of the checkbox in tab order
         */
        this.tabIndex = 0;
        /**
         * \@option Optional css class string | ''
         */
        this.class = '';
        /**
         * \@option Sets the disabled attribute of the Input | false
         */
        this.disabled = false;
        /**
         * \@option Optional indeterminate capabilities of checkbox | false
         */
        this.indeterminate = false;
        /**
         * \@option Input label text | ''
         */
        this.label = '';
        /**
         * \@option Set the level of nested checkboxes | 0
         */
        this.nestedLevel = 0;
        /**
         * \@option Optional required setting for Checkbox input | false
         */
        this.required = false;
        /**
         * \@option Unique HTML ID. Used for tying label to HTML input | ''
         */
        this.htmlId = '';
        /**
         * \@option sets value of the Checkbox input element | false
         */
        this.selectedItem = false;
        /**
         * \@option Sets the attribute name to the Checkbox input element | ''
         */
        this.name = '';
        /**
         * \@prop optional emitter to invoke an checkStatusChange handler when checkbox toggles
         */
        this.checkStatusChange = new EventEmitter();
        this.checked = false;
        this.onListChange = (/**
         * @return {?}
         */
        () => { });
        this.onListTouched = (/**
         * @return {?}
         */
        () => { });
    }
    /**
     * @return {?}
     */
    get checkStatus() {
        return this.checked;
    }
    /**
     * @param {?} status
     * @return {?}
     */
    set checkStatus(status) {
        if (status !== undefined && !this.disabled) {
            this.checked = status;
            this.checkStatusChange.emit(this.checked);
            this.updateList();
        }
    }
    /**
     * @param {?} list
     * @return {?}
     */
    writeValue(list) {
        this.list = list;
        this.checked = this.isChecked();
        this.cdr.markForCheck();
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onListChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onListTouched = fn;
    }
    /**
     * @param {?} e
     * @return {?}
     */
    onClick(e) {
        event.preventDefault();
        if (this.checkStatusChange.observers.length > 0) {
            return;
        }
        if (this.disabled) {
            return;
        }
        this.checked = !this.checked;
        this.updateList();
    }
    /**
     * @return {?}
     */
    updateList() {
        if (this.indeterminate) {
            return;
        }
        if (this.checked) {
            this.addCheck();
        }
        else {
            this.uncheck();
        }
        this.onListChange(this.list);
        if (this.formControl) {
            this.formControl.setValue(this.list);
        }
        this.checkStatusChange.emit(this.checked);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    handleChange(event) {
        this.checked = event.target.checked;
        this.updateList();
    }
    /**
     * @return {?}
     */
    isChecked() {
        return this.list && this.list.includes(this.value);
    }
    /**
     * @return {?}
     */
    uncheck() {
        this.list = this.list.filter((/**
         * @param {?} check
         * @return {?}
         */
        check => check !== this.value));
    }
    /**
     * @return {?}
     */
    addCheck() {
        if (this.list) {
            this.list = [...this.list, this.value];
        }
        else {
            this.list = [this.value];
        }
    }
    /**
     * @return {?}
     */
    get wrapperClasses() {
        return {
            ['cui-input--nested-' + this.nestedLevel]: this.nestedLevel,
            [this.class]: this.class,
        };
    }
    /**
     * @return {?}
     */
    get inputClasses() {
        return {
            ['indeterminate']: this.indeterminate,
        };
    }
}
CheckboxComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-checkbox',
                template: `
    <div class="cui-input-group cui-checkbox" [ngClass]="wrapperClasses">
      <input
        class="cui-input cui-checkbox__input"
        [ngClass]="inputClasses"
        type="checkbox"
        (change)="handleChange($event)"
        [value]="value"
        [checked]="checked"
        [name]="name"
        [attr.id]="htmlId"
        [attr.tabindex]="tabIndex"
        [disabled]="disabled"
        [required]="required"
      />

      <label
        *ngIf="label"
        class="cui-checkbox__label"
        [attr.for]="htmlId"
        (click)="onClick($event)"
      >
        <span>{{ label }}</span>
      </label>

      <ng-content></ng-content>
    </div>
  `,
                providers: [CUSTOM_CHECKBOX_CONTROL_VALUE_ACCESSOR]
            }] }
];
/** @nocollapse */
CheckboxComponent.ctorParameters = () => [
    { type: ChangeDetectorRef }
];
CheckboxComponent.propDecorators = {
    value: [{ type: Input }],
    tabIndex: [{ type: Input }],
    formControl: [{ type: Input }],
    class: [{ type: Input }],
    disabled: [{ type: Input }],
    indeterminate: [{ type: Input }],
    label: [{ type: Input }],
    nestedLevel: [{ type: Input }],
    required: [{ type: Input }],
    htmlId: [{ type: Input }],
    selectedItem: [{ type: Input }],
    name: [{ type: Input }],
    checkStatus: [{ type: Input }],
    checkStatusChange: [{ type: Output }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class LabelComponent {
    constructor() { }
    /**
     * @return {?}
     */
    ngOnInit() { }
    /**
     * @return {?}
     */
    get labelClasses() {
        return {
            ['cui-label--' + this.theme]: this.theme,
            [this.className]: this.className,
        };
    }
}
LabelComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-label',
                template: "<label [ngClass]=\"labelClasses\" [attr.for]=\"htmlFor\">\n  <span>{{ label }}</span>\n</label>\n"
            }] }
];
/** @nocollapse */
LabelComponent.ctorParameters = () => [];
LabelComponent.propDecorators = {
    className: [{ type: Input }],
    theme: [{ type: Input }],
    label: [{ type: Input }],
    htmlFor: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class LabelModule {
}
LabelModule.decorators = [
    { type: NgModule, args: [{
                declarations: [LabelComponent],
                imports: [CommonModule],
                exports: [LabelComponent],
                schemas: [CUSTOM_ELEMENTS_SCHEMA],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class InputHelperComponent {
    constructor() { }
    /**
     * @return {?}
     */
    ngOnInit() { }
    /**
     * @return {?}
     */
    get inputHelperClasses() {
        return {
            [this.className]: this.className,
        };
    }
}
InputHelperComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-input-helper',
                template: `
    <p [ngClass]="inputHelperClasses" class="cui-input__help-text">
      {{ message }}
    </p>
  `
            }] }
];
/** @nocollapse */
InputHelperComponent.ctorParameters = () => [];
InputHelperComponent.propDecorators = {
    message: [{ type: Input }],
    className: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class InputHelperModule {
}
InputHelperModule.decorators = [
    { type: NgModule, args: [{
                declarations: [InputHelperComponent],
                imports: [CommonModule],
                exports: [InputHelperComponent],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CheckboxModule {
}
CheckboxModule.decorators = [
    { type: NgModule, args: [{
                declarations: [CheckboxComponent],
                imports: [
                    CommonModule,
                    FormsModule,
                    ReactiveFormsModule,
                    BrowserModule,
                    LabelModule,
                    InputHelperModule,
                ],
                exports: [CheckboxComponent],
                schemas: [CUSTOM_ELEMENTS_SCHEMA]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const cb = (/**
 * @return {?}
 */
() => { });
class InputComponent {
    /**
     * @param {?} control
     */
    constructor(control) {
        this.control = control;
        this.innerValue = '';
        this.onTouchedCallback = cb;
        this.onChangeCallback = cb;
        /**
         * \@option Optional css class string | ''
         */
        this.class = '';
        /**
         * \@option optional button to clear input text
         */
        this.clear = false;
        /**
         * \@option Sets the disabled attribute of the Input | false
         */
        this.disabled = false;
        /**
         * \@option Unique HTML ID used for tying label to HTML input for automated testing
         */
        this.htmlId = '';
        /**
         * \@option Input css class name string
         */
        this.inputClass = '';
        /**
         * \@option Help Text to appear under the input | ''
         */
        this.inputHelpText = '';
        /**
         * \@option Overall input group size | ''
         */
        this.inputSize = '';
        /**
         * \@option Input label text | ''
         */
        this.label = '';
        /**
         * \@option Placeholder text to display when Input is empty | ''
         */
        this.placeholder = '';
        /**
         * \@option Determines if Input can be edited | false
         */
        this.readOnly = false;
        /**
         * \@option Secondary Input label | ''
         */
        this.secondaryLabel = '';
        /**
         * \@option Input type | 'text'
         */
        this.type = 'text';
        /**
         * \@option Sets the attribute name to the input element | ''
         */
        this.name = '';
        /**
         * \@option Optional error messages object with angular validators | {}
         */
        this.errorObj = {};
        /**
         * \@option function when clicked outside of input
         */
        this.handleBlur = new EventEmitter();
        /**
         * \@option function when input is focused
         */
        this.handleFocus = new EventEmitter();
        /**
         * \@option function when key down on input
         */
        this.handleKeyDown = new EventEmitter();
        /**
         * \@option function when mouse down on input
         */
        this.handleMouseDown = new EventEmitter();
        this.determineErrorType = (/**
         * @param {?} array
         * @return {?}
         */
        array => {
            return array.reduce((/**
             * @param {?} agg
             * @param {?} e
             * @return {?}
             */
            (agg, e) => {
                return agg === 'error' ? agg : e.type || '';
            }), '');
        });
        this.filterErrorsByType = (/**
         * @param {?} array
         * @param {?} value
         * @return {?}
         */
        (array, value) => {
            return array.reduce((/**
             * @param {?} agg
             * @param {?} e
             * @return {?}
             */
            (agg, e) => (e.type === value ? agg.concat(e.error) : agg)), []);
        });
        if (this.control) {
            this.control.valueAccessor = this;
        }
    }
    /**
     * @return {?}
     */
    get value() {
        return this.innerValue;
    }
    /**
     * @param {?} v
     * @return {?}
     */
    set value(v) {
        if (v !== this.innerValue) {
            this.innerValue = v;
            this.onChangeCallback(v);
        }
    }
    /**
     * @return {?}
     */
    get hasRequiredError() {
        return this.control.hasError('required');
    }
    /**
     * @return {?}
     */
    get errorValid() {
        // if (this.control.pristine) {
        //   return '';
        // }
        for (const error in this.errorObj) {
            if (this.control.errors[error]) {
                return this.errorObj[error];
            }
        }
        return '';
    }
    /**
     * @return {?}
     */
    get wrapperClasses() {
        return {
            [this.inputSize + ' columns']: this.inputSize,
            'read-only': this.readOnly,
            disabled: this.disabled,
            error: Object.keys(this.errorObj).length > 0 &&
                this.control.invalid &&
                this.control.dirty,
            ['cui-input-group--' + this.theme]: this.theme,
            [this.errorType]: this.errorType,
            [this.class]: this.class,
        };
    }
    /**
     * @return {?}
     */
    get inputClasses() {
        return {
            [this.inputClass]: this.inputClass,
            'read-only': this.readOnly,
            disabled: this.disabled,
            dirty: this.value,
        };
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onBlur(event) {
        this.handleBlur.emit();
        event.stopPropagation();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onFocus(event) {
        if (this.disabled) {
            event.stopPropagation();
            return;
        }
        this.handleFocus.emit();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onKeyDown(event) {
        this.handleKeyDown.emit();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onMouseDown(event) {
        if (this.disabled) {
            event.stopPropagation();
            return;
        }
        this.handleMouseDown.emit();
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        if (value !== this.innerValue) {
            this.innerValue = value;
        }
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onChangeCallback = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onTouchedCallback = fn;
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (changes.errorArr) {
            /** @type {?} */
            const changedArr = changes.errorArr.currentValue;
            if (changedArr.length > 0) {
                this.errorType = this.determineErrorType(changedArr);
                this.errors =
                    this.errorType && this.filterErrorsByType(changedArr, this.errorType);
            }
        }
    }
    /**
     * @return {?}
     */
    handleClear() {
        this.value = '';
    }
}
InputComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-input',
                template: `
    <div class="cui-input-group" [ngClass]="wrapperClasses">
      <cui-label
        *ngIf="label"
        [label]="label"
        className="cui-label"
        htmlFor="htmlId"
      ></cui-label>

      <ng-container *ngIf="clear; else normal">
        <div class="cui-input__icon-container">
          <input
            *ngIf="!secondaryLabel"
            class="cui-input"
            [ngClass]="inputClasses"
            [(ngModel)]="value"
            [placeholder]="placeholder"
            [readonly]="readOnly"
            [attr.disabled]="disabled ? '' : null"
            [type]="type"
            (blur)="onBlur($event)"
            (focus)="onFocus($event)"
            (keydown)="onKeyDown($event)"
            (mousedown)="onMouseDown($event)"
            [name]="name"
            [attr.id]="htmlId"
          />

          <button
            *ngIf="clear && value !== ''"
            class="cui-button cui-button--36 cui-button--icon"
            type="button"
            aria-label="clear input"
            (click)="handleClear()"
          >
            <span class="cui-button__children" style="opacity: 1;">
              <i
                class="cui-icon icon icon-clear-active_16 cui-input__icon-clear"
              ></i>
            </span>
          </button>
        </div>
      </ng-container>

      <ng-template #normal>
        <input
          class="cui-input"
          [ngClass]="inputClasses"
          [(ngModel)]="value"
          [type]="type"
          [placeholder]="placeholder"
          [readonly]="readOnly"
          [attr.disabled]="disabled ? '' : null"
          (blur)="onBlur($event)"
          (focus)="onFocus($event)"
          (keydown)="onKeyDown($event)"
          (mousedown)="onMouseDown($event)"
          [name]="name"
          [attr.id]="htmlId"
        />
      </ng-template>

      <!-- Secondary Label -->
      <div class="cui-label__secondary-label-container" *ngIf="secondaryLabel">
        <input
          class="cui-input"
          [ngClass]="inputClasses"
          [(ngModel)]="value"
          [type]="type"
          [placeholder]="placeholder"
          [readonly]="readOnly"
          [attr.disabled]="disabled ? '' : null"
          (blur)="onBlur($event)"
          (focus)="onFocus($event)"
          (keydown)="onKeyDown($event)"
          (mousedown)="onMouseDown($event)"
          [name]="name"
          [attr.id]="htmlId"
        />

        <label class="cui-label__secondary-label" [attr.for]="htmlId">
          <span>{{ secondaryLabel }}</span>
        </label>
      </div>

      <!-- Error Input -->
      <ng-container *ngIf="errors">
        <cui-input-error *ngFor="let error of errors" [error]="error">
        </cui-input-error>
      </ng-container>

      <!-- Angular Control Error Validation Message  -->
      <div class="cui-input__messages" role="alert" *ngIf="control.invalid">
        <div class="message">{{ errorValid }}</div>
      </div>

      <!-- Helper Text -->
      <cui-input-helper *ngIf="inputHelpText" [message]="inputHelpText">
      </cui-input-helper>
    </div>
  `
            }] }
];
/** @nocollapse */
InputComponent.ctorParameters = () => [
    { type: NgControl, decorators: [{ type: Optional }, { type: Self }] }
];
InputComponent.propDecorators = {
    class: [{ type: Input }],
    clear: [{ type: Input }],
    disabled: [{ type: Input }],
    htmlId: [{ type: Input }],
    errorArr: [{ type: Input }],
    inputClass: [{ type: Input }],
    inputHelpText: [{ type: Input }],
    inputSize: [{ type: Input }],
    label: [{ type: Input }],
    placeholder: [{ type: Input }],
    readOnly: [{ type: Input }],
    secondaryLabel: [{ type: Input }],
    theme: [{ type: Input }],
    type: [{ type: Input }],
    name: [{ type: Input }],
    errorObj: [{ type: Input }],
    handleBlur: [{ type: Output }],
    handleFocus: [{ type: Output }],
    handleKeyDown: [{ type: Output }],
    handleMouseDown: [{ type: Output }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class InputErrorComponent {
    constructor() { }
    /**
     * @return {?}
     */
    ngOnInit() { }
}
InputErrorComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-input-error',
                template: `
    <div class="cui-input__messages" role="alert">
      <div class="message">{{ error }}</div>
    </div>
  `
            }] }
];
/** @nocollapse */
InputErrorComponent.ctorParameters = () => [];
InputErrorComponent.propDecorators = {
    error: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class InputErrorModule {
}
InputErrorModule.decorators = [
    { type: NgModule, args: [{
                declarations: [InputErrorComponent],
                imports: [CommonModule],
                exports: [InputErrorComponent],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class InputModule {
}
InputModule.decorators = [
    { type: NgModule, args: [{
                declarations: [InputComponent],
                imports: [
                    CommonModule,
                    BrowserModule,
                    FormsModule,
                    ReactiveFormsModule,
                    LabelModule,
                    InputErrorModule,
                    InputHelperModule,
                    IconModule,
                ],
                exports: [InputComponent],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const cb$1 = (/**
 * @return {?}
 */
() => { });
class EditableTextfieldComponent {
    /**
     * @param {?} control
     * @param {?} elementRef
     */
    constructor(control, elementRef) {
        this.control = control;
        this.elementRef = elementRef;
        this._alignment = null;
        /**
         * \@option Optional css class name for non-edit text div | null
         */
        this.buttonClass = '';
        /**
         * \@option Optional css class string on div wrapping input | ''
         */
        this.wrapperClass = '';
        /**
         * \@option Sets the disabled attribute of the Input | false
         */
        this.disabled = false;
        /**
         * \@option Unique HTML ID used for tying label to HTML input for automated testing
         */
        this.htmlId = '';
        /**
         * \@option Input css class name string
         */
        this.inputClass = '';
        /**
         * \@option Overall input group size | ''
         */
        this.inputSize = '';
        /**
         * \@option Placeholder text to display when Input is empty | ''
         */
        this.placeholder = '';
        /**
         * \@option Determines if Input can be edited | false
         */
        this.readOnly = false;
        /**
         * \@option Input type | 'text'
         */
        this.type = 'text';
        /**
         * \@option Sets the attribute name to the input element | ''
         */
        this.name = '';
        /**
         * \@option Optional function after done editing
         */
        this.handleDoneEditing = new EventEmitter();
        this.innerValue = '';
        this.isEditing = false;
        this.onTouchedCallback = cb$1;
        this.onChangeCallback = cb$1;
        this.handleClick = (/**
         * @return {?}
         */
        () => {
            if (this.disabled) {
                return;
            }
            else {
                this.isEditing = true;
                setTimeout((/**
                 * @return {?}
                 */
                () => this.inputRef.nativeElement.focus()), 100);
            }
        });
        if (this.control) {
            this.control.valueAccessor = this;
        }
    }
    /**
     * \@option Sets optional button alignment | null
     * @param {?} alignment
     * @return {?}
     */
    set alignment(alignment) {
        if (this._alignment) {
            this.elementRef.nativeElement.classList.remove(`cui-editable-textfield--${this._alignment}`);
        }
        this.elementRef.nativeElement.classList.add(`cui-editable-textfield--${alignment}`);
        this._alignment = alignment;
    }
    /**
     * @return {?}
     */
    get value() {
        return this.innerValue;
    }
    /**
     * @param {?} v
     * @return {?}
     */
    set value(v) {
        if (v !== this.innerValue) {
            this.innerValue = v;
            this.onChangeCallback(v);
        }
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        if (value !== this.innerValue) {
            this.innerValue = value;
        }
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onChangeCallback = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onTouchedCallback = fn;
    }
    /**
     * @param {?} e
     * @return {?}
     */
    onKeyDown(e) {
        if (e.keyCode === 27) {
            this.isEditing = false;
        }
        else if (e.keyCode === 13) {
            this.handleEnter(e, e.target.value);
        }
    }
    /**
     * @param {?} e
     * @param {?} value
     * @return {?}
     */
    handleEnter(e, value) {
        this.isEditing = false;
    }
    // keydown on non edit button
    /**
     * @param {?} event
     * @return {?}
     */
    handleKey(event) {
        if (!this.disabled) {
            if ((event.keyCode >= 65 && event.keyCode <= 90) ||
                (event.keyCode >= 48 && event.keyCode <= 57) ||
                event.keyCode === 32) {
                this.isEditing = true;
                setTimeout((/**
                 * @return {?}
                 */
                () => this.inputRef.nativeElement.focus()), 100);
                this.value = this.value.concat(event.key);
            }
        }
    }
    /**
     * @param {?} e
     * @return {?}
     */
    onBlur(e) {
        this.isEditing = false;
        this.handleDoneEditing.emit(this.value);
        e.stopPropagation();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onFocus(event) {
        if (this.disabled) {
            event.stopPropagation();
            return;
        }
        this.isEditing = true;
    }
    /**
     * @return {?}
     */
    get textWrapperClasses() {
        return {
            [this.inputSize + ' columns']: this.inputSize,
            [this.wrapperClass]: this.wrapperClass,
        };
    }
    /**
     * @return {?}
     */
    get buttonClasses() {
        return {
            [this.buttonClass]: this.buttonClass,
        };
    }
    /**
     * @return {?}
     */
    get inputClasses() {
        return {
            [this.inputClass]: this.inputClass,
            'read-only': this.readOnly,
            disabled: this.disabled,
            dirty: this.value,
        };
    }
}
EditableTextfieldComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-editable-textfield',
                template: `
    <div
      *ngIf="isEditing"
      class="cui-input-group cui-editable-textfield__editing"
      [ngClass]="textWrapperClasses"
    >
      <input
        #inputRef
        class="cui-input"
        [ngClass]="inputClasses"
        [attr.id]="htmlId"
        [(ngModel)]="value"
        [placeholder]="placeholder"
        [readonly]="readOnly"
        [attr.disabled]="disabled ? '' : null"
        [type]="type"
        [name]="name"
        (blur)="onBlur($event)"
        (focus)="onFocus($event)"
        (keydown)="onKeyDown($event)"
      />
    </div>

    <!-- Not in Edit Mode -->
    <div
      [hidden]="isEditing"
      class="cui-editable-textfield__button"
      [ngClass]="buttonClasses"
      role="button"
      tabindex="0"
      (click)="handleClick()"
      (keydown)="handleKey($event)"
    >
      {{ value }}
    </div>
  `,
                host: {
                    class: 'cui-editable-textfield',
                }
            }] }
];
/** @nocollapse */
EditableTextfieldComponent.ctorParameters = () => [
    { type: NgControl, decorators: [{ type: Optional }, { type: Self }] },
    { type: ElementRef }
];
EditableTextfieldComponent.propDecorators = {
    alignment: [{ type: Input }],
    buttonClass: [{ type: Input }],
    wrapperClass: [{ type: Input }],
    disabled: [{ type: Input }],
    htmlId: [{ type: Input }],
    inputClass: [{ type: Input }],
    inputSize: [{ type: Input }],
    placeholder: [{ type: Input }],
    readOnly: [{ type: Input }],
    type: [{ type: Input }],
    name: [{ type: Input }],
    handleDoneEditing: [{ type: Output }],
    inputRef: [{ type: ViewChild, args: ['inputRef',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class EditableTextfieldModule {
}
EditableTextfieldModule.decorators = [
    { type: NgModule, args: [{
                declarations: [EditableTextfieldComponent],
                imports: [CommonModule, InputModule, FormsModule, ReactiveFormsModule],
                exports: [EditableTextfieldComponent],
                schemas: [CUSTOM_ELEMENTS_SCHEMA],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class OptionSelectionChange {
    /**
     * @param {?} source
     * @param {?=} isUserInput
     */
    constructor(source, isUserInput = false) {
        this.source = source;
        this.isUserInput = isUserInput;
    }
}
class ListItemComponent {
    /**
     * @param {?} el
     */
    constructor(el) {
        this.el = el;
        this.checkedValues = [''];
        this.isSelectOption = this._hasHostAttributes('cui-select-option');
        /**
         * \@prop Active prop to determine styles | false
         */
        this.active = false;
        /**
         * \@prop selected prop to determine styles | false
         */
        this.selected = false;
        /**
         * \@prop Disabled attribute for ListItem to determine styles | false
         */
        this.disabled = false;
        /**
         * \@prop Sets ListItem to a focus state | false
         */
        this.focus = false;
        /**
         * \@prop Sets ListItem id | null
         */
        this.id = uniqueId$1('cui-list-item-');
        /**
         * \@prop Determines if is part of a multi select | false
         */
        this.isMulti = false;
        /**
         * \@prop Determines if ListItem is clickable | false
         */
        this.isReadOnly = false;
        /**
         * \@prop ListItem text and form value | ''
         */
        this.label = null;
        /**
         * \@prop external link associated input | ''
         */
        this.link = '';
        /**
         * \@prop ListItem ref name | 'navLink'
         */
        this.role = null;
        /**
         * \@prop Controls whether to show separator or not | false
         */
        this.separator = false;
        /**
         * \@prop ListItem Title | ''
         */
        this.title = '';
        /**
         * \@prop ListItem size | ''
         */
        this._type = null;
        this.selectionChange = new EventEmitter();
        this.handleClick = (/**
         * @param {?} event
         * @return {?}
         */
        event => {
            if (this.isReadOnly) {
                event.stopImmediatePropagation();
            }
            else {
                this._selectViaInteraction();
            }
        });
        this.handleKeydown = (/**
         * @param {?} event
         * @return {?}
         */
        event => {
            if ((event.keyCode === ENTER || event.keyCode === SPACE) && !hasModifierKey(event)) {
                this._selectViaInteraction();
                event.preventDefault();
            }
        });
        this._isTypeOptionValid = (/**
         * @return {?}
         */
        () => (['', 'small', 'large', 'xlarge', 'space', 'header', 36, 52, 60].includes(this._type)));
    }
    /**
     * @return {?}
     */
    get myRole() {
        return this.role || this.isSelectOption ? 'option' : 'listItem';
    }
    /**
     * @param {?} type
     * @return {?}
     */
    set type(type) {
        if (this._type) {
            this.el.nativeElement.classList.remove(`cui-list-item--${this._type}`);
        }
        this.el.nativeElement.classList.add(`cui-list-item--${type}`);
        this._type = type;
    }
    /**
     * @return {?}
     */
    get myHref() {
        return (this.link && this._getHostElement().localName === 'a') ? this.link : null;
    }
    /**
     * @return {?}
     */
    get theTitle() {
        return this.title || this.label;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (this._type && !this._isTypeOptionValid()) {
            throw new Error(`cui-list-item: ListItem type option must be one of the following:
        small, large, xlarge, space, header, 36, 52, 60`);
        }
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        if (!this.label) {
            throw new Error('cui-list-item: Label property is required');
        }
    }
    /**
     * @return {?}
     */
    select() {
        if (!this.selected) {
            this.selected = true;
            this._emitSelectionChangeEvent();
        }
    }
    /**
     * Deselects the option.
     * @return {?}
     */
    deselect() {
        if (this.selected) {
            this.selected = false;
            this._emitSelectionChangeEvent();
        }
    }
    // Highlightable Interface methods
    /**
     * @return {?}
     */
    setActiveStyles() {
        this.focus = true;
    }
    /**
     * @return {?}
     */
    setInactiveStyles() {
        this.focus = false;
    }
    /**
     * @return {?}
     */
    getLabel() {
        return this.label;
    }
    /**
     * @return {?}
     */
    _selectViaInteraction() {
        if (!this.disabled) {
            this.selected = !this.selected;
            this.checkStatus = this.selected;
            this._emitSelectionChangeEvent(true);
        }
    }
    /**
     * @param {?=} isUserInput
     * @return {?}
     */
    _emitSelectionChangeEvent(isUserInput = false) {
        return this.selectionChange.emit(new OptionSelectionChange(this, isUserInput));
    }
    /**
     * @return {?}
     */
    _getHostElement() {
        return this.el.nativeElement;
    }
    /**
     * Gets whether the button has one of the given attributes.
     * @param {...?} attributes
     * @return {?}
     */
    _hasHostAttributes(...attributes) {
        return attributes.some((/**
         * @param {?} attribute
         * @return {?}
         */
        attribute => this._getHostElement().hasAttribute(attribute)));
    }
}
ListItemComponent.decorators = [
    { type: Component, args: [{
                selector: 'a[cui-list-item], div[cui-list-item], div[cui-select-option]',
                exportAs: 'cuiListItem',
                template: `
      <ng-container *ngIf="isSelectOption">

        <ng-container *ngIf="!isMulti">
          <cui-list-item-section key="child-0" position='center'>
              <ng-container *ngIf="label; else content">{{label}}</ng-container>
              <ng-template #content>
                  <ng-content></ng-content>
              </ng-template>
          </cui-list-item-section>
          <cui-list-item-section key="child-1" position='right'>
              <cui-icon *ngIf="selected" name="check_20" color="blue"></cui-icon>
          </cui-list-item-section>
        </ng-container>

        <ng-container *ngIf="isMulti">
          <cui-checkbox
            name="{{id}}-checkbox"
            value="{{label}}"
            label="{{label}}"
            [(checkStatus)]="checkStatus"
            htmlId="{{id}}-checkbox">
          </cui-checkbox>
        </ng-container>

      </ng-container>

      <ng-container *ngIf="!isSelectOption">
        <div *ngIf="label; else content">{{label}}</div>
        <ng-template #content>
          <ng-content></ng-content>
        </ng-template>
      </ng-container>
  `,
                host: {
                    'class': 'cui-list-item',
                    '[class.cui-list-item--read-only]': 'isReadOnly',
                    '[class.cui-list-item--separator]': 'separator',
                    '[class.cui-select-option]': 'isSelectOption',
                    '[class.active]': 'active',
                    '[class.focus]': 'focus',
                    '[class.disabled]': 'disabled',
                }
            }] }
];
/** @nocollapse */
ListItemComponent.ctorParameters = () => [
    { type: ElementRef }
];
ListItemComponent.propDecorators = {
    active: [{ type: Input }],
    selected: [{ type: Input }],
    disabled: [{ type: Input }],
    focus: [{ type: Input }],
    id: [{ type: HostBinding, args: ['attr.id',] }, { type: Input }],
    isMulti: [{ type: Input }],
    isReadOnly: [{ type: Input }],
    label: [{ type: Input }],
    link: [{ type: Input }],
    role: [{ type: Input }],
    myRole: [{ type: HostBinding, args: ['attr.role',] }],
    separator: [{ type: Input }],
    title: [{ type: Input }],
    type: [{ type: Input }],
    selectionChange: [{ type: Output }],
    myHref: [{ type: HostBinding, args: ['attr.href',] }],
    theTitle: [{ type: HostBinding, args: ['attr.title',] }],
    handleClick: [{ type: HostListener, args: ['click', ['$event'],] }],
    handleKeydown: [{ type: HostListener, args: ['keydown', ['$event'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ListItemSectionComponent {
    constructor() {
        /**
         * \@option position Determine the ListItemSection's position | 'center'
         */
        this.position = 'center';
        /**
         * \@option class Optional css class name | ''
         */
        this.class = '';
        this.isValidPosition = (/**
         * @return {?}
         */
        () => ['left', 'center', 'right', 'center-align'].includes(this.position));
    }
    /**
     * @return {?}
     */
    get className() {
        return (`cui-list-item__${this.position}` +
            `${(this.class && ` ${this.class}`) || ''}`);
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (this.position && !this.isValidPosition()) {
            throw new Error(`cui-list-item-section: ListItemSection position option
        must be one of the following: left, center, right, center-align`);
        }
    }
}
ListItemSectionComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-list-item-section',
                template: `
    <ng-content></ng-content>
  `
            }] }
];
/** @nocollapse */
ListItemSectionComponent.ctorParameters = () => [];
ListItemSectionComponent.propDecorators = {
    position: [{ type: Input }],
    class: [{ type: Input }],
    className: [{ type: HostBinding, args: ['class',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ListItemSectionModule {
}
ListItemSectionModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                declarations: [ListItemSectionComponent],
                exports: [ListItemSectionComponent],
                schemas: [CUSTOM_ELEMENTS_SCHEMA]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ListItemModule {
}
ListItemModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CheckboxModule,
                    CommonModule,
                    IconModule,
                    ListItemSectionModule
                ],
                declarations: [ListItemComponent],
                exports: [ListItemComponent],
                schemas: [CUSTOM_ELEMENTS_SCHEMA]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ListComponent {
    /**
     * @param {?} cd
     * @param {?} _ngZone
     * @param {?} el
     */
    constructor(cd, _ngZone, el) {
        this.cd = cd;
        this._ngZone = _ngZone;
        this.el = el;
        /**
         * \@option Optional ID value of List | null
         */
        this.id = uniqueId$1('cui-list-');
        /**
         * \@option itemRole prop to set child roles | 'listItem'
         */
        this.itemRole = 'listItem';
        /**
         * \@option Callback function invoked by user selecting an interactive item within list | null
         */
        this.select = new EventEmitter();
        /**
         * \@option Sets the ARIA role for the Nav, in the context of a TabContainer | 'list'
         */
        this.role = 'list';
        /**
         * \@option Sets the orientation of the List | 'vertical'
         */
        this._tabType = 'vertical';
        /**
         * \@option Type sets List size | null
         */
        this.type = null;
        /**
         * \@option Wrap prop type to wrap items to next row
         */
        this.wrap = false;
        /**
         * Combined stream of all of the child options' change events.
         */
        this.optionSelectionChanges = (/** @type {?} */ (defer((/**
         * @return {?}
         */
        () => {
            if (this.listItems) {
                return merge(...this.listItems.map((/**
                 * @param {?} option
                 * @return {?}
                 */
                option => option.selectionChange)));
            }
            return this._ngZone.onStable.asObservable()
                .pipe(take(1), switchMap((/**
             * @return {?}
             */
            () => this.optionSelectionChanges)));
        }))));
        this._isTabTypeOptionValid = (/**
         * @return {?}
         */
        () => (['vertical', 'horizontal'].includes(this._tabType)));
        this._isTypeOptionValid = (/**
         * @return {?}
         */
        () => (['small', 'large', 'space', 'xlarge'].includes(this.type)));
        this.optionSelectionChanges.subscribe((/**
         * @param {?} event
         * @return {?}
         */
        event => {
            this._onSelect(event.source);
        }));
    }
    /**
     * @param {?} tabType
     * @return {?}
     */
    set tabType(tabType) {
        if (this._tabType) {
            this.el.nativeElement.classList.remove(`cui-list--${this._tabType}`);
        }
        this.el.nativeElement.classList.add(`cui-list--${tabType}`);
        this._tabType = tabType;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (this._tabType && !this._isTabTypeOptionValid()) {
            throw new Error(`cui-list: List tabType option must be one of the following:
        vertical, horizontal`);
        }
        if (this.type && !this._isTypeOptionValid()) {
            throw new Error(`cui-list: List type option must be one of the following:
        small, large, space, xlarge`);
        }
    }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        // set up listItems children with props
        this.listItems.forEach((/**
         * @param {?} listItem
         * @return {?}
         */
        listItem => {
            if (this.type) {
                listItem.type = this.type;
            }
            listItem.role = this.itemRole;
            listItem.id = uniqueId$1(`${this.id}__list-item-`);
        }));
        this.cd.detectChanges();
    }
    /**
     * Invoked when an option is clicked.
     * @private
     * @param {?} option
     * @return {?}
     */
    _onSelect(option) {
        if (this.select) {
            return this.select.emit(option);
        }
    }
}
ListComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-list',
                template: `
    <ng-content></ng-content>
  `,
                host: {
                    'class': 'cui-list',
                    '[class.cui-list--wrap]': 'wrap'
                }
            }] }
];
/** @nocollapse */
ListComponent.ctorParameters = () => [
    { type: ChangeDetectorRef },
    { type: NgZone },
    { type: ElementRef }
];
ListComponent.propDecorators = {
    id: [{ type: HostBinding, args: ['id',] }, { type: Input }],
    itemRole: [{ type: Input }],
    select: [{ type: Output }],
    role: [{ type: HostBinding, args: ['attr.role',] }, { type: Input }],
    tabType: [{ type: Input }],
    type: [{ type: Input }],
    wrap: [{ type: Input }],
    listItems: [{ type: ContentChildren, args: [ListItemComponent,] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ListModule {
}
ListModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                declarations: [ListComponent],
                exports: [ListComponent],
                schemas: [CUSTOM_ELEMENTS_SCHEMA]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ListSeparatorComponent {
    /**
     * @return {?}
     */
    get _backgroundColor() {
        return this.backgroundColor;
    }
    /**
     * @return {?}
     */
    get _lineColor() {
        return this.lineColor;
    }
    /**
     * @return {?}
     */
    get _margin() {
        return this.margin;
    }
}
ListSeparatorComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-list-separator',
                template: `
    <span class="cui-list-separator__container">
      <ng-content *ngIf="!text; else textTemplate"></ng-content>
    </span>

    <ng-template #textTemplate>
      <span
        class="cui-list-separator__text"
        [ngStyle]="{ color: textColor, padding: textPadding }"
        >{{ text }}</span
      >
    </ng-template>
  `,
                host: {
                    class: 'cui-list-separator',
                }
            }] }
];
ListSeparatorComponent.propDecorators = {
    backgroundColor: [{ type: Input }],
    lineColor: [{ type: Input }],
    margin: [{ type: Input }],
    text: [{ type: Input }],
    textColor: [{ type: Input }],
    textPadding: [{ type: Input }],
    _backgroundColor: [{ type: HostBinding, args: ['style.background-color',] }],
    _lineColor: [{ type: HostBinding, args: ['style.color',] }],
    _margin: [{ type: HostBinding, args: ['style.margin',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ListSeparatorModule {
}
ListSeparatorModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                declarations: [ListSeparatorComponent],
                exports: [ListSeparatorComponent],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ModalBodyComponent {
    constructor() {
        /**
         * \@option css class names | '' | ''
         */
        this.class = '';
    }
    /**
     * @return {?}
     */
    get className() {
        return 'cui-modal__body' + `${(this.class && ` ${this.class}`) || ''}` + ``;
    }
    /**
     * @return {?}
     */
    ngOnInit() { }
}
ModalBodyComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-modal-body',
                template: `
    <ng-content></ng-content>
  `
            }] }
];
/** @nocollapse */
ModalBodyComponent.ctorParameters = () => [];
ModalBodyComponent.propDecorators = {
    class: [{ type: Input }],
    className: [{ type: HostBinding, args: ['class',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ModalBodyModule {
}
ModalBodyModule.decorators = [
    { type: NgModule, args: [{
                declarations: [ModalBodyComponent],
                imports: [CommonModule],
                exports: [ModalBodyComponent],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ModalFooterComponent {
    constructor() {
        /**
         * \@option css class names | '' | ''
         */
        this.class = '';
    }
    /**
     * @return {?}
     */
    get className() {
        return ('cui-modal__footer' + `${(this.class && ` ${this.class}`) || ''}` + ``);
    }
    /**
     * @return {?}
     */
    ngOnInit() { }
}
ModalFooterComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-modal-footer',
                template: `
    <ng-content></ng-content>
  `
            }] }
];
/** @nocollapse */
ModalFooterComponent.ctorParameters = () => [];
ModalFooterComponent.propDecorators = {
    class: [{ type: Input }],
    className: [{ type: HostBinding, args: ['class',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ModalFooterModule {
}
ModalFooterModule.decorators = [
    { type: NgModule, args: [{
                declarations: [ModalFooterComponent],
                imports: [CommonModule],
                exports: [ModalFooterComponent],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ModalService {
    constructor() {
        this.isModalOpened = new Subject();
        this.isModalOpened$ = this.isModalOpened.asObservable();
    }
    /**
     * @param {?} isOpen
     * @return {?}
     */
    setModalStatus(isOpen) {
        this.isModalOpened.next(isOpen);
    }
}
ModalService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
ModalService.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ModalHeaderComponent {
    /**
     * @param {?} modalService
     */
    constructor(modalService) {
        this.modalService = modalService;
        /**
         * \@option Optional css class string | ''
         */
        this.class = '';
        /**
         * ModalHeader label text | ''
         */
        this.headerLabel = '';
        /**
         * Modal message | ''
         */
        this.message = '';
        /**
         * show/hide close button | true
         */
        this.showCloseButton = true;
        this.hideNative = true;
        this.closeModal = (/**
         * @return {?}
         */
        () => {
            this.modalService.setModalStatus(false);
        });
    }
    /**
     * @return {?}
     */
    get className() {
        return ('cui-modal__header' + `${(this.class && ` ${this.class}`) || ''}` + ``);
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.hideNative = this.headerLabel !== '';
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        // prevent memory leak when component destroyed
        if (this.subscription) {
            this.subscription.unsubscribe();
        }
    }
}
ModalHeaderComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-modal-header',
                template: `
    <ng-content></ng-content>
    <span *ngIf="hideNative" class="cui-modal__title">{{ headerLabel }}</span>
    <span *ngIf="hideNative && message" class="cui-modal__message">{{
      message
    }}</span>
    <button
      *ngIf="showCloseButton"
      cui-button
      aria-label="Close Modal"
      (click)="closeModal()"
      class="cui-close cui-modal__close"
    ></button>
  `,
                providers: []
            }] }
];
/** @nocollapse */
ModalHeaderComponent.ctorParameters = () => [
    { type: ModalService }
];
ModalHeaderComponent.propDecorators = {
    class: [{ type: Input }],
    headerLabel: [{ type: Input }],
    message: [{ type: Input }],
    showCloseButton: [{ type: Input }],
    className: [{ type: HostBinding, args: ['class',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ModalHeaderModule {
}
ModalHeaderModule.decorators = [
    { type: NgModule, args: [{
                declarations: [ModalHeaderComponent],
                imports: [CommonModule],
                exports: [ModalHeaderComponent],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ModalComponent {
    /**
     * @param {?} overlay
     * @param {?} viewContainerRef
     * @param {?} modalService
     */
    constructor(overlay, viewContainerRef, modalService) {
        this.overlay = overlay;
        this.viewContainerRef = viewContainerRef;
        this.modalService = modalService;
        /**
         * \@option Determines the visibility and ability to edit the backdrop of the Modal | true
         */
        this.backdrop = true;
        /**
         * \@option To enable/disable clicking on underlay to exit modal | false
         */
        this.backdropClickExit = false;
        /**
         * \@option Show/hide modal
         */
        this.show = false; //
        //
        /**
         * \@option Size of the modal | 'default'
         */
        this.sizeType = 'default';
        /**
         * \@option Callback function invoked when user clicks on cross button or esc key
         */
        this.whenHide = new EventEmitter();
        this.showModal = (/**
         * @return {?}
         */
        () => {
            this.modalService.setModalStatus(true);
        });
        this.closeModal = (/**
         * @return {?}
         */
        () => {
            this.modalService.setModalStatus(false);
        });
        this.showContent = (/**
         * @return {?}
         */
        () => {
            if (this.overlayRef && !this.overlayRef.hasAttached()) {
                this.overlayRef.attach(this.tp);
            }
        });
        this.dismissContent = (/**
         * @return {?}
         */
        () => {
            if (this.overlayRef && this.overlayRef.hasAttached()) {
                this.overlayRef.detach();
                this.overlayRef.backdropElement.remove();
                this.whenHide.emit();
            }
        });
    }
    /**
     * @return {?}
     */
    get classes() {
        return {
            'cui-modal': true,
            ['cui-modal--' + this.sizeType]: true,
            overlayerFix: this.sizeType === 'full',
            [this.class]: this.class,
        };
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        /** @type {?} */
        const strategy = this.overlay
            .position()
            .global()
            .centerHorizontally()
            .centerVertically();
        /** @type {?} */
        const config = new OverlayConfig({
            hasBackdrop: true,
            backdropClass: this.backdrop ? 'cui-modal__backdrop' : '',
            positionStrategy: strategy,
        });
        this.overlayRef = this.overlay.create(config);
        this.tp = new TemplatePortal(this.modalContentTemplate, this.viewContainerRef);
        if (this.backdropClickExit) {
            this.overlayRef.backdropClick().subscribe((/**
             * @return {?}
             */
            () => {
                this.dismissContent();
            }));
        }
        this.modalService.isModalOpened$.subscribe((/**
         * @param {?} isOpen
         * @return {?}
         */
        isOpen => {
            if (isOpen) {
                this.showContent();
            }
            else {
                this.dismissContent();
            }
        }));
        this.modalService.setModalStatus(this.show);
    }
}
ModalComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-modal',
                template: `
    <ng-template #overlayGlobalTemplate>
      <div [ngClass]="classes">
        <div class="cui-modal__content">
          <div class="cui-modal__flex-container">
            <ng-content></ng-content>
          </div>
        </div>
      </div>
    </ng-template>
  `,
                providers: [ModalService],
                styles: ['.overlayerFix{ top:0; }']
            }] }
];
/** @nocollapse */
ModalComponent.ctorParameters = () => [
    { type: Overlay },
    { type: ViewContainerRef },
    { type: ModalService }
];
ModalComponent.propDecorators = {
    backdrop: [{ type: Input }],
    backdropClickExit: [{ type: Input }],
    class: [{ type: Input }],
    show: [{ type: Input }],
    sizeType: [{ type: Input }],
    whenHide: [{ type: Output }],
    modalContentTemplate: [{ type: ViewChild, args: ['overlayGlobalTemplate',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ModalModule {
}
ModalModule.decorators = [
    { type: NgModule, args: [{
                declarations: [ModalComponent],
                entryComponents: [ModalComponent],
                imports: [CommonModule, OverlayModule],
                exports: [ModalComponent],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// tslint:disable:no-use-before-declare
/** @type {?} */
const CUSTOM_RADIO_VALUE_ACCESSOR = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef((/**
     * @return {?}
     */
    () => RadioComponent)),
    multi: true,
};
// tslint:enable:no-use-before-declare
class RadioComponent {
    /**
     * @param {?} cdr
     */
    constructor(cdr) {
        this.cdr = cdr;
        /**
         * \@option Optional CSS class name | ''
         */
        this.class = '';
        /**
         * \@option Sets the attribute disabled to the Radio | false
         */
        this.disabled = false;
        /**
         * \@option Unique HTML ID used for tying label to HTML input for automated testing
         */
        this.htmlId = '';
        /**
         * \@option Radio label text | ''
         */
        this.label = '';
        /**
         * \@option Radio name for group | ''
         */
        this.name = '';
        /**
         * \@option String value that corresponds with Radio button | ''
         */
        this.value = '';
        /**
         * \@option Set the level of nested radio | 0
         */
        this.nestedLevel = 0;
        /**
         * \@option Callback function invoked when user clicks the Radio button | null
         */
        this.click = new EventEmitter();
        this.onChangeCallback = (/**
         * @return {?}
         */
        () => { });
        this.onTouchedCallback = (/**
         * @return {?}
         */
        () => { });
    }
    /**
     * @return {?}
     */
    get wrapperClasses() {
        return {
            ['cui-input--nested-' + this.nestedLevel]: this.nestedLevel,
            [this.class]: this.class,
        };
    }
    /**
     * @param {?} e
     * @return {?}
     */
    onToggle(e) {
        if (!this.disabled) {
            this.radioViewChild.nativeElement.checked = true;
            this.checked = true;
            this.onChangeCallback(this.value);
            this.click.emit();
        }
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        this.checked = value === this.value;
        if (this.radioViewChild.nativeElement) {
            this.radioViewChild.nativeElement.checked = this.checked;
        }
        this.cdr.markForCheck();
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onTouchedCallback = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onChangeCallback = fn;
    }
}
RadioComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-radio',
                template: `
    <div class="cui-radio-group">
      <div class="cui-input-group cui-radio" [ngClass]="wrapperClasses">
        <input
          class="cui-input cui-radio__input"
          type="radio"
          #radioInput
          (change)="onToggle($event)"
          [attr.id]="htmlId"
          [attr.name]="name"
          [attr.tabindex]="tabIndex"
          [attr.value]="value"
          [checked]="checked"
          [disabled]="disabled"
        />

        <label
          class="cui-radio__label"
          (click)="onToggle($event)"
          [attr.for]="htmlId"
        >
          <span>{{ label }}</span>
        </label>
      </div>

      <ng-content></ng-content>
    </div>
  `,
                providers: [CUSTOM_RADIO_VALUE_ACCESSOR]
            }] }
];
/** @nocollapse */
RadioComponent.ctorParameters = () => [
    { type: ChangeDetectorRef }
];
RadioComponent.propDecorators = {
    class: [{ type: Input }],
    disabled: [{ type: Input }],
    htmlId: [{ type: Input }],
    label: [{ type: Input }],
    name: [{ type: Input }],
    tabIndex: [{ type: Input }],
    value: [{ type: Input }],
    nestedLevel: [{ type: Input }],
    click: [{ type: Output }],
    radioViewChild: [{ type: ViewChild, args: ['radioInput',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class RadioModule {
}
RadioModule.decorators = [
    { type: NgModule, args: [{
                declarations: [RadioComponent],
                imports: [CommonModule, FormsModule, ReactiveFormsModule, BrowserModule],
                exports: [RadioComponent],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const cb$2 = (/**
 * @return {?}
 */
() => { });
class SearchInputComponent {
    /**
     * @param {?} control
     */
    constructor(control) {
        this.control = control;
        /**
         * \@option Optional css class string | ''
         */
        this.class = '';
        /**
         * \@option optional button to clear input text
         */
        this.clear = false;
        /**
         * \@option Sets the disabled attribute of the Input | false
         */
        this.disabled = false;
        /**
         * \@option Unique HTML ID used for tying label to HTML input for automated testing
         */
        this.htmlId = '';
        /**
         * \@option Input css class name string
         */
        this.inputClass = '';
        /**
         * \@option Overall input group size | ''
         */
        this.inputSize = '';
        /**
         * \@option Input label text | ''
         */
        this.label = '';
        /**
         * \@option Placeholder text to display when Input is empty | ''
         */
        this.placeholder = '';
        /**
         * \@option Determines if Input can be edited | false
         */
        this.readOnly = false;
        /**
         * \@option Input type | 'text'
         */
        this.type = 'text';
        /**
         * \@option Sets the attribute name to the input element | ''
         */
        this.name = '';
        /**
         * \@option function when clicked outside of input
         */
        this.handleBlur = new EventEmitter();
        /**
         * \@option function when input is focused
         */
        this.handleFocus = new EventEmitter();
        /**
         * \@option function when key down on input
         */
        this.handleKeyDown = new EventEmitter();
        /**
         * \@option function when mouse down on input
         */
        this.handleMouseDown = new EventEmitter();
        this.innerValue = '';
        this.onTouchedCallback = cb$2;
        this.onChangeCallback = cb$2;
        if (this.control) {
            this.control.valueAccessor = this;
        }
    }
    /**
     * @return {?}
     */
    get value() {
        return this.innerValue;
    }
    /**
     * @param {?} v
     * @return {?}
     */
    set value(v) {
        if (v !== this.innerValue) {
            this.innerValue = v;
            this.onChangeCallback(v);
        }
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        if (value !== this.innerValue) {
            this.innerValue = value;
        }
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onChangeCallback = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onTouchedCallback = fn;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onKeyDown(event) {
        this.handleKeyDown.emit();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onMouseDown(event) {
        if (this.disabled) {
            event.stopPropagation();
            return;
        }
        this.handleMouseDown.emit();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onBlur(event) {
        this.handleBlur.emit();
        event.stopPropagation();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onFocus(event) {
        if (this.disabled) {
            event.stopPropagation();
            return;
        }
        this.handleFocus.emit();
    }
    /**
     * @return {?}
     */
    get wrapperClasses() {
        return {
            [this.inputSize + ' columns']: this.inputSize,
            'cui-search-input--pill': this.type === 'pill',
            'read-only': this.readOnly,
            disabled: this.disabled,
            [this.class]: this.class,
        };
    }
    /**
     * @return {?}
     */
    get inputClasses() {
        return {
            [this.inputClass]: this.inputClass,
            'read-only': this.readOnly,
            disabled: this.disabled,
            dirty: this.value,
        };
    }
    /**
     * @return {?}
     */
    get iconClass() {
        return {
            'icon-search_16 ': this.type === 'pill',
            'icon-search_20': this.type !== 'pill',
        };
    }
    /**
     * @return {?}
     */
    handleClear() {
        this.value = '';
    }
}
SearchInputComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-search-input',
                template: `
    <div class="cui-input-group cui-search-input" [ngClass]="wrapperClasses">
      <cui-label
        *ngIf="label"
        [label]="label"
        className="cui-label"
        htmlFor="htmlId"
      ></cui-label>

      <div class="cui-input__icon-container">
        <input
          class="cui-input"
          [ngClass]="inputClasses"
          [attr.id]="htmlId"
          [(ngModel)]="value"
          [placeholder]="placeholder"
          [readonly]="readOnly"
          [attr.disabled]="disabled ? '' : null"
          [type]="type"
          [name]="name"
          (blur)="onBlur($event)"
          (focus)="onFocus($event)"
          (keydown)="onKeyDown($event)"
          (mousedown)="onMouseDown($event)"
        />

        <i
          class="cui-icon icon cui-search-input__icon"
          [ngClass]="iconClass"
        ></i>

        <button
          *ngIf="clear && value !== ''"
          class="cui-button cui-button--36 cui-button--icon"
          (click)="handleClear()"
          type="button"
          aria-label="clear input"
        >
          <span class="cui-button__children" style="opacity: 1;">
            <i
              class="cui-icon icon icon-clear-active_16 cui-input__icon-clear"
            ></i>
          </span>
        </button>
      </div>
    </div>
  `
            }] }
];
/** @nocollapse */
SearchInputComponent.ctorParameters = () => [
    { type: NgControl, decorators: [{ type: Optional }, { type: Self }] }
];
SearchInputComponent.propDecorators = {
    class: [{ type: Input }],
    clear: [{ type: Input }],
    disabled: [{ type: Input }],
    htmlId: [{ type: Input }],
    inputClass: [{ type: Input }],
    inputSize: [{ type: Input }],
    label: [{ type: Input }],
    placeholder: [{ type: Input }],
    readOnly: [{ type: Input }],
    theme: [{ type: Input }],
    type: [{ type: Input }],
    name: [{ type: Input }],
    handleBlur: [{ type: Output }],
    handleFocus: [{ type: Output }],
    handleKeyDown: [{ type: Output }],
    handleMouseDown: [{ type: Output }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class SearchInputModule {
}
SearchInputModule.decorators = [
    { type: NgModule, args: [{
                declarations: [SearchInputComponent],
                imports: [
                    FormsModule,
                    ReactiveFormsModule,
                    CommonModule,
                    IconModule,
                    InputModule,
                    LabelModule,
                ],
                exports: [SearchInputComponent],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// tslint:disable:no-use-before-declare
/** @type {?} */
const CUI_SLIDER_VALUE_ACCESSOR = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef((/**
     * @return {?}
     */
    () => SliderComponent)),
    multi: true,
};
class SliderComponent {
    /**
     * @param {?} changeDetectorRef
     */
    constructor(changeDetectorRef) {
        this.changeDetectorRef = changeDetectorRef;
        /**
         * \@option Determines if minimum pointer can cross over maximum pointer | false
         */
        this.canCross = false;
        /**
         * \@option Set the attribute disabled to Slider | false
         */
        this.disabled = false;
        /**
         * \@option Set the initial minimum value | 0
         */
        this.min = 0;
        /**
         * \@option Set visual step measurement | 1
         */
        this.step = 1;
        /**
         * \@option Set increment of x-axis labels | 0
         */
        this.tick = 0;
        /**
         * Event emitted when change a pointer position
         */
        this.change = new EventEmitter();
        this._value = 0;
    }
    /**
     * \@option Set either maximum pointer value or a combination of high and low pointer values | 0
     * @return {?}
     */
    get value() {
        return this._value;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set value(value) {
        if (this._value !== value) {
            this._value = value;
            this.sliderLow =
                (isObject(this.value) && ((/** @type {?} */ (this.value))).low) || this.min;
            this.sliderHigh =
                (isObject(this.value) && ((/** @type {?} */ (this.value))).high) ||
                    ((/** @type {?} */ (this.value)));
            this.returnCurrentValues();
            this.changeDetectorRef.markForCheck();
        }
    }
    /**
     * @return {?}
     */
    get sliderLow() {
        return this._sliderLow;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set sliderLow(value) {
        this._sliderLow = value;
    }
    /**
     * @return {?}
     */
    get sliderHigh() {
        return this._sliderHigh;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set sliderHigh(value) {
        this._sliderHigh = value;
    }
    /**
     * @return {?}
     */
    get sliderLowPosition() {
        return ((this.sliderLow - this.min) / (this.max - this.min)) * 100;
    }
    /**
     * @return {?}
     */
    get sliderHighPosition() {
        return ((this.sliderHigh - this.min) / (this.max - this.min)) * 100;
    }
    /**
     * @private
     * @param {?} _
     * @return {?}
     */
    _change(_) { }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (!this.max) {
            throw new Error('Initial maximum(max) value is required');
        }
        this.isObject = isObject;
        this.scale = [this.min, this.max];
        this.getScale(this.min, this.max, this.tick);
        this.getSelectionWidth();
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        this.setScalePos(this.scale, this.min, this.max);
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        if (value !== null) {
            this.value = value;
        }
    }
    /**
     * @param {?} disabled
     * @return {?}
     */
    setDisabledState(disabled) {
        this.disabled = disabled;
        this.changeDetectorRef.markForCheck();
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this._change = fn;
    }
    /**
     * @param {?} _
     * @return {?}
     */
    registerOnTouched(_) { }
    /**
     * @param {?} scale
     * @param {?} min
     * @param {?} max
     * @return {?}
     */
    setScalePos(scale, min, max) {
        this.ticksContainer.nativeElement
            .querySelectorAll('span')
            .forEach((/**
         * @param {?} child
         * @param {?} idx
         * @return {?}
         */
        (child, idx) => {
            /** @type {?} */
            const leftValue = `${((scale[idx] - min) / max) * 100}%`;
            child.style.left = `calc(${leftValue} - ${child.offsetWidth / 2}px)`;
        }));
    }
    /**
     * @param {?=} low
     * @param {?=} high
     * @param {?=} tick
     * @return {?}
     */
    getScale(low = 0, high, tick) {
        if (this.tick > 0) {
            /** @type {?} */
            let value = high;
            /** @type {?} */
            const ticksArray = [high];
            while (value > 0 && value - tick >= low) {
                value -= tick;
                ticksArray.unshift(Math.abs(Math.round(value / tick) * tick));
            }
            this.scale = ticksArray;
        }
    }
    /**
     * @return {?}
     */
    getSliderWidth() {
        return this.sliderBar.nativeElement.getBoundingClientRect().width;
    }
    /**
     * @return {?}
     */
    getStepWidth() {
        /** @type {?} */
        const maxValue = this.max - this.min;
        /** @type {?} */
        const maxWidth = this.getSliderWidth();
        return (this.step / maxValue) * maxWidth;
    }
    /**
     * @param {?} position
     * @return {?}
     */
    getSteps(position) {
        if (position.isKeyBoard) {
            return 1;
        }
        /** @type {?} */
        const diff = position.direction === 1
            ? position.to - position.from
            : position.from - position.to;
        if (diff < 0) {
            return 0;
        }
        /** @type {?} */
        const steps = diff / this.getStepWidth();
        return steps - Math.floor(steps) >= 0.5
            ? Math.ceil(steps)
            : Math.floor(steps);
    }
    /**
     * @param {?} pointerKey
     * @param {?} direction
     * @return {?}
     */
    getLimit(pointerKey, direction) {
        if (pointerKey === 'sliderLow') {
            return this.canCross
                ? direction === 1
                    ? this.max
                    : this.min
                : direction === 1
                    ? this.sliderHigh
                    : this.min;
        }
        else if (pointerKey === 'sliderHigh') {
            return this.canCross
                ? direction === 1
                    ? this.max
                    : this.min
                : direction === 1
                    ? this.max
                    : this.sliderLow;
        }
    }
    /**
     * @return {?}
     */
    returnCurrentValues() {
        this.getSelectionWidth();
        /** @type {?} */
        const value = isObject(this.value)
            ? {
                low: Math.min(this.sliderHigh, this.sliderLow),
                high: Math.max(this.sliderHigh, this.sliderLow),
            }
            : this.sliderHigh;
        this._change(value);
        this.change.emit(value);
    }
    /**
     * @param {?} key
     * @param {?} pixelMove
     * @param {?} limit
     * @return {?}
     */
    moveForward(key, pixelMove, limit) {
        /** @type {?} */
        const newPosition = this[key] + pixelMove <= limit ? this[key] + pixelMove : limit;
        this[key] = newPosition;
        this.returnCurrentValues();
    }
    /**
     * @param {?} key
     * @param {?} pixelMove
     * @param {?} limit
     * @return {?}
     */
    moveBack(key, pixelMove, limit) {
        /** @type {?} */
        const newPosition = this[key] - pixelMove >= limit ? this[key] - pixelMove : limit;
        this[key] = newPosition;
        this.returnCurrentValues();
    }
    /**
     * @param {?} key
     * @param {?} position
     * @return {?}
     */
    onSliderMove(key, position) {
        if (this.disabled) {
            return;
        }
        /** @type {?} */
        const limit = this.getLimit(key, position.direction);
        /** @type {?} */
        const pixelMove = this.getSteps(position) * this.step;
        position.direction === 1
            ? this.moveForward(key, pixelMove, limit)
            : this.moveBack(key, pixelMove, limit);
    }
    /**
     * @return {?}
     */
    getSelectionWidth() {
        /** @type {?} */
        const baseValue = Number.isInteger(this.sliderLow)
            ? this.sliderLow
            : this.min;
        this.selectionLeft =
            ((Math.min(baseValue, this.sliderHigh) - this.min) / this.max) * 100;
        this.selectionWidth =
            (Math.abs(this.sliderHigh - baseValue) / this.max) * 100;
    }
}
SliderComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-slider',
                template: `
    <span #sliderBar class="cui-slider__bar"></span>
    <span
      class="cui-slider__selection"
      [ngStyle]="{ 'left.%': selectionLeft, 'width.%': selectionWidth }"
    ></span>
    <cui-slider-pointer
      *ngIf="isObject(value)"
      [position]="sliderLowPosition"
      (move)="onSliderMove('sliderLow', $event)"
    ></cui-slider-pointer>
    <cui-slider-pointer
      [position]="sliderHighPosition"
      (move)="onSliderMove('sliderHigh', $event)"
    ></cui-slider-pointer>
    <div #ticksContainer>
      <span *ngFor="let tickValue of scale" class="cui-slider__hashlabel">{{
        (translateFn && translateFn(tickValue)) || tickValue
      }}</span>
    </div>
  `,
                host: {
                    class: 'cui-slider',
                    '[class.cui-slider--disabled]': 'disabled',
                },
                providers: [CUI_SLIDER_VALUE_ACCESSOR]
            }] }
];
/** @nocollapse */
SliderComponent.ctorParameters = () => [
    { type: ChangeDetectorRef }
];
SliderComponent.propDecorators = {
    canCross: [{ type: Input }],
    disabled: [{ type: Input }],
    max: [{ type: Input }],
    min: [{ type: Input }],
    step: [{ type: Input }],
    tick: [{ type: Input }],
    translateFn: [{ type: Input }],
    value: [{ type: Input }],
    change: [{ type: Output }],
    sliderBar: [{ type: ViewChild, args: ['sliderBar',] }],
    ticksContainer: [{ type: ViewChild, args: ['ticksContainer',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class SliderPointerComponent {
    /**
     * @param {?} elementRef
     * @param {?} _document
     */
    constructor(elementRef, _document) {
        this.elementRef = elementRef;
        /**
         * \@option Set Slider Pointer's position | 0
         */
        this.position = 0;
        /**
         * Event emitted when user moves the Slider Pointer
         */
        this.move = new EventEmitter();
        this.onMousemove = (/**
         * @param {?} event
         * @return {?}
         */
        (event) => {
            /** @type {?} */
            const xPos = event.clientX;
            /** @type {?} */
            const direction = this.getDirections(xPos);
            this.move.emit({
                from: this.elementRef.nativeElement.getBoundingClientRect().left,
                to: xPos,
                direction: direction,
                isKeyBoard: false,
            });
        });
        this.onMouseup = (/**
         * @param {?} _
         * @return {?}
         */
        (_) => {
            this._document.removeEventListener('mousemove', this.onMousemove);
            this._document.removeEventListener('mouseup', this.onMouseup);
        });
        this._document = _document;
    }
    /**
     * @return {?}
     */
    get left() {
        return this.position;
    }
    /**
     * @param {?} currentPos
     * @return {?}
     */
    getDirections(currentPos) {
        if (currentPos > this._previousPosition) {
            return 1;
        }
        return -1;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onKeydown(event) {
        switch (event.code) {
            case 'ArrowUp':
            case 'ArrowRight':
                this.move.emit({
                    direction: 1,
                    isKeyBoard: true,
                });
                event.preventDefault();
                break;
            case 'ArrowDown':
            case 'ArrowLeft':
                this.move.emit({
                    direction: -1,
                    isKeyBoard: true,
                });
                event.preventDefault();
                break;
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onMousedown(event) {
        this._document.addEventListener('mousemove', this.onMousemove);
        this._document.addEventListener('mouseup', this.onMouseup);
        this._previousPosition = event.clientX;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onTouchstart(event) {
        this._previousPosition = event.touches[0].clientX;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onTouchmove(event) {
        /** @type {?} */
        const xPos = event.touches[0].clientX;
        /** @type {?} */
        const direction = this.getDirections(xPos);
        this.move.emit({
            from: this.elementRef.nativeElement.getBoundingClientRect().left,
            to: xPos,
            direction: direction,
            isKeyBoard: false,
        });
    }
}
SliderPointerComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-slider-pointer',
                template: ``,
                host: {
                    class: 'cui-slider__pointer',
                    role: 'button',
                    tabindex: '0',
                    '(keydown)': 'onKeydown($event)',
                    '(mousedown)': 'onMousedown($event)',
                    '(touchstart)': 'onTouchstart($event)',
                    '(touchmove)': 'onTouchmove($event)',
                }
            }] }
];
/** @nocollapse */
SliderPointerComponent.ctorParameters = () => [
    { type: ElementRef },
    { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] }] }
];
SliderPointerComponent.propDecorators = {
    position: [{ type: Input }],
    move: [{ type: Output }],
    left: [{ type: HostBinding, args: ['style.left.%',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class SliderModule {
}
SliderModule.decorators = [
    { type: NgModule, args: [{
                declarations: [SliderComponent, SliderPointerComponent],
                imports: [CommonModule],
                exports: [SliderComponent],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Change event object that is emitted when the select value has changed.
 */
class SelectChange {
    /**
     * @param {?} source
     * @param {?} value
     */
    constructor(source, value) {
        this.source = source;
        this.value = value;
    }
}
class SelectComponent {
    /**
     * @param {?} _ngZone
     */
    constructor(_ngZone) {
        this._ngZone = _ngZone;
        this._destroy = new Subject();
        this.overlayOpen = false;
        this.anchorWidth = null;
        /**
         * \@prop Optional CSS button class name | ''
         */
        this.buttonClass = '';
        /**
         * \@prop Set the default selected option | ''
         */
        this.defaultValue = null;
        /**
         * \@prop Disable the Select Component | false
         */
        this.disabled = false;
        /**
         * \@prop Set ID for Select Component | null
         */
        this.id = uniqueId('cui-select-');
        /**
         * \@prop Optional prop to know if user is able to select multiple options | false
         */
        this.isMulti = false;
        this.select = new EventEmitter();
        /**
         * Event emitted when the selected value has been changed by the user.
         */
        this.selectionChange = new EventEmitter();
        /**
         * facilitates the two-way binding for the `ngModel` input.
         */
        this.ngModelChange = new EventEmitter();
        /**
         * Combined stream of all of the child options' change events.
         */
        this.optionSelectionChanges = (/** @type {?} */ (defer((/**
         * @return {?}
         */
        () => {
            if (this.selectOptions) {
                return merge(...this.selectOptions.map((/**
                 * @param {?} option
                 * @return {?}
                 */
                option => option.selectionChange)));
            }
            return this._ngZone.onStable
                .asObservable()
                .pipe(take(1), switchMap((/**
             * @return {?}
             */
            () => this.optionSelectionChanges)));
        }))));
        /**
         * `View -> model callback called when value changes`
         */
        this._onChange = (/**
         * @return {?}
         */
        () => { });
        /**
         * `View -> model callback called when select has been touched`
         */
        this._onTouched = (/**
         * @return {?}
         */
        () => { });
        /**
         * Comparison function to specify which option is displayed. Defaults to object equality.
         */
        this._compareWith = (/**
         * @param {?} o1
         * @param {?} o2
         * @return {?}
         */
        (o1, o2) => o1 === o2);
        /**
         * Handles keyboard events when the selected is open.
         */
        this._handleOpenKeydown = (/**
         * @param {?} event
         * @return {?}
         */
        (event) => {
            if (event.defaultPrevented || !this.overlayOpen) {
                return;
            }
            /** @type {?} */
            const keyCode = event.keyCode;
            /** @type {?} */
            const isArrowKey = keyCode === DOWN_ARROW || keyCode === UP_ARROW;
            /** @type {?} */
            const manager = this._keyManager;
            if (keyCode === HOME || keyCode === END) {
                event.preventDefault();
                keyCode === HOME ? manager.setFirstItemActive() : manager.setLastItemActive();
            }
            else if (isArrowKey && event.altKey) {
                // Close the select on ALT + arrow key to match the native <select>
                event.preventDefault();
                this.close();
            }
            else if ((keyCode === ENTER || keyCode === SPACE) && manager.activeItem &&
                !hasModifierKey(event)) {
                if (!this.isMulti) {
                    this.close();
                }
                event.preventDefault();
                manager.activeItem._selectViaInteraction();
            }
            else if (this.isMulti && keyCode === A && event.ctrlKey) {
                event.preventDefault();
                /** @type {?} */
                const hasDeselectedOptions = this.selectOptions.some((/**
                 * @param {?} opt
                 * @return {?}
                 */
                opt => !opt.disabled && !opt.selected));
                this.selectOptions.forEach((/**
                 * @param {?} option
                 * @return {?}
                 */
                option => {
                    if (!option.disabled) {
                        hasDeselectedOptions ? option.select() : option.deselect();
                    }
                }));
            }
            else {
                /** @type {?} */
                const previouslyFocusedIndex = manager.activeItemIndex;
                manager.onKeydown(event);
                if (this.isMulti && isArrowKey && event.shiftKey && manager.activeItem &&
                    manager.activeItemIndex !== previouslyFocusedIndex) {
                    manager.activeItem._selectViaInteraction();
                }
            }
        });
        /**
         * Toggles the overlay panel open or closed.
         */
        this.toggle = (/**
         * @return {?}
         */
        () => {
            this.overlayOpen ? this.close() : this.open();
            this.focus();
        });
        this.open = (/**
         * @return {?}
         */
        () => {
            if (this.disabled || !this.selectOptions || !this.selectOptions.length || this.overlayOpen) {
                return;
            }
            this.overlayOpen = true;
        });
        this.close = (/**
         * @return {?}
         */
        () => {
            if (this.overlayOpen) {
                this.overlayOpen = false;
            }
        });
        /**
         * Focuses the origin button of the select component.
         */
        this.focus = (/**
         * @return {?}
         */
        () => {
            this.originButton.el.nativeElement.focus();
        });
        this.currentValue = (/**
         * @return {?}
         */
        () => {
            if (!this._selectionModel) {
                return;
            }
            /** @type {?} */
            const selected = this._selectionModel.selected;
            if (!this.isMulti && selected.length) {
                return selected[0].label;
            }
            if (selected.length === 1) {
                return `${selected.length} Item Selected`;
            }
            else if (selected.length) {
                return `${selected.length} Items Selected`;
            }
        });
        /**
         * Sets the select's ngModelValue. Part of the ControlValueAccessor interface
         */
        this.writeValue = (/**
         * @param {?} value
         * @return {?}
         */
        (value) => {
            if (this.selectOptions) {
                this._setSelectionByValue(value);
            }
        });
        this.registerOnChange = (/**
         * @param {?} fn
         * @return {?}
         */
        (fn) => {
            this._onChange = fn;
        });
        this.registerOnTouched = (/**
         * @param {?} fn
         * @return {?}
         */
        (fn) => {
            this._onTouched = fn;
        });
        this.setDisabledState = (/**
         * @param {?} isDisabled
         * @return {?}
         */
        (isDisabled) => {
            this.disabled = isDisabled;
        });
        /**
         * Sets the selected option based on a value.
         */
        this._setSelectionByValue = (/**
         * @param {?} value
         * @return {?}
         */
        (value) => {
            if (this.isMulti && value) {
                if (!Array.isArray(value)) {
                    throw Error('Value must be an array in multi-selection mode.');
                }
                this._selectionModel.clear();
                value.forEach((/**
                 * @param {?} currentValue
                 * @return {?}
                 */
                (currentValue) => this._selectValue(currentValue)));
            }
            else {
                this._selectionModel.clear();
                /** @type {?} */
                const matchedOption = this._selectValue(value);
                if (matchedOption) {
                    this._keyManager.setActiveItem(matchedOption);
                }
            }
        });
        this._selectValue = (/**
         * @param {?} value
         * @return {?}
         */
        (value) => {
            /** @type {?} */
            const matchedOption = this.selectOptions.find((/**
             * @param {?} option
             * @return {?}
             */
            (option) => {
                try {
                    return option.label != null && this._compareWith(option.label, value);
                }
                catch (error) {
                    console.warn(error);
                }
                return false;
            }));
            if (matchedOption) {
                this._selectionModel.select(matchedOption);
            }
            return matchedOption;
        });
        this._propagateChanges = (/**
         * @param {?=} fallbackValue
         * @return {?}
         */
        (fallbackValue) => {
            /** @type {?} */
            let valueToEmit = null;
            if (this.isMulti) {
                valueToEmit = ((/** @type {?} */ (this.selected))).map((/**
                 * @param {?} option
                 * @return {?}
                 */
                option => option.label));
            }
            else {
                valueToEmit = this.selected ? ((/** @type {?} */ (this.selected))).label : fallbackValue;
            }
            this._ngModelValue = valueToEmit;
            this.ngModelChange.emit(valueToEmit);
            this._onChange(valueToEmit);
            this.selectionChange.emit(new SelectChange(this, valueToEmit));
        });
        /**
         * Sets up a key manager to listen to keyboard events on the overlay panel.
         */
        this._initKeyManager = (/**
         * @return {?}
         */
        () => {
            this._keyManager = new ActiveDescendantKeyManager(this.selectOptions)
                .withWrap()
                .withTypeAhead();
            this._keyManager.tabOut.pipe(takeUntil(this._destroy)).subscribe((/**
             * @return {?}
             */
            () => {
                // Restore focus to the trigger before closing
                this.focus();
                this.close();
            }));
            this._keyManager.change.pipe(takeUntil(this._destroy)).subscribe((/**
             * @return {?}
             */
            () => {
                if (!this.overlayOpen && !this.isMulti && this._keyManager.activeItem) {
                    this._keyManager.activeItem._selectViaInteraction();
                }
            }));
        });
        /**
         * Drops current option subscriptions and IDs and resets from scratch.
         */
        this._resetOptions = (/**
         * @return {?}
         */
        () => {
            /** @type {?} */
            const changedOrDestroyed = merge(this.selectOptions.changes, this._destroy);
            this.optionSelectionChanges.pipe(takeUntil(changedOrDestroyed)).subscribe((/**
             * @param {?} event
             * @return {?}
             */
            event => {
                this._onSelect(event.source, event.isUserInput);
                if (event.isUserInput && !this.isMulti && this.overlayOpen) {
                    this.close();
                    this.focus();
                }
            }));
        });
        /**
         * Invoked when an option is clicked.
         */
        this._onSelect = (/**
         * @param {?} option
         * @param {?} isUserInput
         * @return {?}
         */
        (option, isUserInput) => {
            /** @type {?} */
            const wasSelected = this._selectionModel.isSelected(option);
            if (option.label == null && !this.isMulti) {
                option.deselect();
                this._selectionModel.clear();
                this._propagateChanges(option.label);
            }
            else {
                option.selected ? this._selectionModel.select(option) : this._selectionModel.deselect(option);
                if (isUserInput) {
                    this._keyManager.setActiveItem(option);
                }
                if (this.isMulti) {
                    if (isUserInput) {
                        // if clicked with mouse, focus on origin button
                        this.focus();
                    }
                }
            }
            if (wasSelected !== this._selectionModel.isSelected(option)) {
                this._propagateChanges();
            }
        });
        this._setAnchorWidth = (/**
         * @param {?} elementAnchor
         * @return {?}
         */
        (elementAnchor) => {
            /** @type {?} */
            const anchor = elementAnchor && elementAnchor.getBoundingClientRect();
            this.anchorWidth = anchor.width;
        });
        this.optionSelectionChanges.subscribe((/**
         * @param {?} event
         * @return {?}
         */
        event => {
            this._onSelect(event.source, event.isUserInput);
        }));
    }
    /** @option Callback function invoked by user selecting an interactive option within list | null */
    /**
     * ngModel of Select Control
     * @return {?}
     */
    get ngModel() { return this._ngModelValue; }
    /**
     * @param {?} newValue
     * @return {?}
     */
    set ngModel(newValue) {
        if (newValue !== this._ngModelValue) {
            this.writeValue(newValue);
            this._ngModelValue = newValue;
        }
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this._selectionModel = new SelectionModel(this.isMulti);
    }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        this._initKeyManager();
        this._selectionModel.changed.pipe(takeUntil(this._destroy)).subscribe((/**
         * @param {?} event
         * @return {?}
         */
        event => {
            event.added.forEach((/**
             * @param {?} option
             * @return {?}
             */
            option => option.select()));
            event.removed.forEach((/**
             * @param {?} option
             * @return {?}
             */
            option => option.deselect()));
        }));
        this.selectOptions.changes.pipe(startWith(null), takeUntil(this._destroy)).subscribe((/**
         * @return {?}
         */
        () => {
            this._resetOptions();
        }));
        // set up children with props
        this.selectOptions.forEach((/**
         * @param {?} option
         * @return {?}
         */
        (option) => {
            option.isMulti = this.isMulti;
            this._resetOptions();
        }));
    }
    /**
     * @return {?}
     */
    ngAfterContentChecked() {
        this._setAnchorWidth(this.originButton.el.nativeElement);
    }
    /**
     * @return {?}
     */
    get buttonClasses() {
        return 'cui-button--input' +
            `${(this.buttonClass && ` ${this.buttonClass}`) || ''}` +
            `${(this.disabled && ` disabled`) || ''}`;
    }
    /**
     * The currently selected option.
     * @return {?}
     */
    get selected() {
        return this.isMulti ? this._selectionModel.selected : this._selectionModel.selected[0];
    }
}
SelectComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-select',
                template: `
    <button cui-button
        cdkOverlayOrigin
        #trigger="cdkOverlayOrigin"
        [attr.name]='id'
        aria-label="select button"
        [id]='id'
        (click)='toggle()'
        [class]="buttonClasses"
        type="button"
        >
          <div class='cui-select__label' id="{{id}}__label">
              {{currentValue() || defaultValue}}
              <cui-icon name="arrow-down_16"></cui-icon>
          </div>
    </button>

    <ng-template
      cdkConnectedOverlay
      [cdkConnectedOverlayOrigin]="trigger"
      [cdkConnectedOverlayOpen]="overlayOpen"
      [cdkConnectedOverlayWidth]="this.anchorWidth"
      [cdkConnectedOverlayOffsetY]="6"
      [cdkConnectedOverlayPanelClass]="'cui-event-overlay__children'"
      [cdkConnectedOverlayHasBackdrop]="true"
      [cdkConnectedOverlayBackdropClass]="'cdk-overlay-transparent-backdrop'"
      (backdropClick)="close()"
      (detach)="close()">
      <cui-list
        #panel
        role='listbox'>
        <ng-content></ng-content>
      </cui-list>
    </ng-template>
  `,
                host: {
                    class: 'cui-input-group cui-select',
                    '[class.disabled]': 'disabled'
                }
            }] }
];
/** @nocollapse */
SelectComponent.ctorParameters = () => [
    { type: NgZone }
];
SelectComponent.propDecorators = {
    buttonClass: [{ type: Input }],
    defaultValue: [{ type: Input }],
    disabled: [{ type: HostBinding, args: ['attr.disabled',] }, { type: Input }],
    id: [{ type: Input }],
    isMulti: [{ type: Input }],
    ngModel: [{ type: Input }],
    select: [{ type: Output }],
    selectionChange: [{ type: Output }],
    ngModelChange: [{ type: Output }],
    originButton: [{ type: ViewChild, args: [ButtonComponent,] }],
    panel: [{ type: ViewChild, args: ['panel',] }],
    selectOptions: [{ type: ContentChildren, args: [ListItemComponent,] }],
    _handleOpenKeydown: [{ type: HostListener, args: ['keydown', ['$event'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class SelectModule {
}
SelectModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    ButtonModule,
                    IconModule,
                    PortalModule,
                    ListModule,
                    ListItemModule,
                    OverlayModule,
                    A11yModule,
                ],
                declarations: [
                    SelectComponent,
                ],
                exports: [SelectComponent],
                schemas: [CUSTOM_ELEMENTS_SCHEMA]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class SpinnerComponent {
    constructor() {
        this.color = 'black';
        this.size = 36;
        this.showCheck = false;
        this.showPercentage = false;
        this.checkPercentage = (/**
         * @param {?} percent
         * @return {?}
         */
        percent => {
            if (!isNaN(percent)) {
                this.isPercentage = true;
                this.percentage = Math.round(percent);
            }
            if (Number(this.size) !== 36 && this.showPercentage) {
                console.warn('Spinner: Percentage will not be shown for sizes smaller than 36');
            }
        });
        this.sizeCheck = (/**
         * @param {?} size
         * @return {?}
         */
        size => {
            /** @type {?} */
            const sizes = [16, 20, 28, 36];
            if (sizes.includes(Number(size))) {
                return true;
            }
            console.warn('Only sizes 16, 20, 28, 36 are acceptable for spinner');
            return false;
        });
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.checkPercentage(this.percentage);
    }
    /**
     * @return {?}
     */
    get defaultClasses() {
        return {
            ['cui-spinner--' + this.size]: this.sizeCheck(this.size),
            ['cui-spinner--36']: Number(this.size) === 36,
            ['cui-spinner--' + this.color]: this.color,
            [this.className]: this.className,
        };
    }
    /**
     * @return {?}
     */
    get percentageClasses() {
        return {
            ['cui-spinner-progress--' + this.size]: this.sizeCheck(this.size),
            ['cui-spinner-progress__percentage-' + this.percentage]: this.percentage,
            ['cui-spinner-progress--' + this.color]: this.color,
            [this.className]: this.className,
        };
    }
    /**
     * @return {?}
     */
    get checkMarkClasses() {
        return {
            ['cui-spinner-progress__inset-circle--check']: this.sizeCheck(this.size) && this.showCheck,
            ['cui-spinner-progress__inset-circle--check' + this.percentage]: this.showCheck && Number(this.percentage) === 100,
        };
    }
}
SpinnerComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-spinner',
                template: "<div\n  *ngIf=\"isPercentage; else normal\"\n  [ngClass]=\"percentageClasses\"\n  class=\"cui-spinner-progress\"\n>\n  <div class=\"cui-spinner-progress__circle\">\n    <div class=\"cui-spinner-progress__mask cui-spinner-progress__full\">\n      <div class=\"cui-spinner-progress__fill\"></div>\n    </div>\n    <div class=\"cui-spinner-progress__mask cui-spinner-progress__half\">\n      <div class=\"cui-spinner-progress__fill\"></div>\n      <div class=\"cui-spinner-progress__fill cui-spinner-progress__fix\"></div>\n    </div>\n    <div\n      [ngClass]=\"checkMarkClasses\"\n      class=\"cui-spinner-progress__inset-circle\"\n    >\n      <div\n        *ngIf=\"size === 36 && showPercentage && !showCheck\"\n        class=\"cui-spinner-progress__percentage\"\n      >\n        {{ percentage | number: '1.0-0' }}\n      </div>\n    </div>\n  </div>\n</div>\n\n<ng-template #normal>\n  <i [ngClass]=\"defaultClasses\" class=\"cui-spinner\"></i>\n</ng-template>\n"
            }] }
];
/** @nocollapse */
SpinnerComponent.ctorParameters = () => [];
SpinnerComponent.propDecorators = {
    className: [{ type: Input }],
    color: [{ type: Input }],
    size: [{ type: Input }],
    percentage: [{ type: Input }],
    showCheck: [{ type: Input }],
    showPercentage: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class SpinnerModule {
}
SpinnerModule.decorators = [
    { type: NgModule, args: [{
                declarations: [SpinnerComponent],
                imports: [CommonModule],
                exports: [SpinnerComponent],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TabsService {
    constructor() {
        this.Pane = 0;
        this.Tabs = 0;
        this.currentTabValue = 0;
        this.currentFocusValue = 0;
        this.TabDic = {};
        this.TabDisables = {};
        this.current = new Subject();
        this.current$ = this.current.asObservable();
        this.focusIndex = new Subject();
        this.focusIndex$ = this.focusIndex.asObservable();
        this.tabSize = new Subject();
        this.tabSize$ = this.tabSize.asObservable();
        this.paneSize = new Subject();
        this.paneSize$ = this.paneSize.asObservable();
        this.registerPane = (/**
         * @return {?}
         */
        () => {
            /** @type {?} */
            const index = this.Pane + 1;
            this.paneSize.next(index);
            return index;
        });
        this.registerTab = (/**
         * @return {?}
         */
        () => {
            /** @type {?} */
            const index = this.Tabs + 1;
            this.tabSize.next(index);
            return index;
        });
        this.registerDisable = (/**
         * @param {?} index
         * @param {?} disable
         * @return {?}
         */
        (index, disable) => {
            this.TabDisables[index] = disable;
        });
        this.registerKey = (/**
         * @param {?} key
         * @param {?} value
         * @return {?}
         */
        (key, value) => {
            key = key.toLowerCase();
            if (this.TabDic[key] === undefined) {
                this.TabDic[key] = [];
            }
            this.TabDic[key].push(value);
        });
        this.select = (/**
         * @param {?} index
         * @param {?=} ifFocusSelect
         * @return {?}
         */
        (index, ifFocusSelect) => {
            if (this.TabDisables[index] === false || ifFocusSelect) {
                this.setFocus(index);
                this.current.next(index);
            }
        });
        this.selectFocus = (/**
         * @param {?} index
         * @return {?}
         */
        (index) => {
            if (this.currentFocusValue !== this.currentTabValue) {
                this.current.next(this.currentFocusValue);
            }
        });
        this.setFocus = (/**
         * @param {?} index
         * @return {?}
         */
        (index) => {
            this.focusIndex.next(index);
        });
        this.setFocusPre = (/**
         * @return {?}
         */
        () => {
            this.setFocus(Math.max(0, this.currentFocusValue - 1));
        });
        this.setFocusNext = (/**
         * @return {?}
         */
        () => {
            this.setFocus(Math.min(this.Tabs, this.currentFocusValue + 1));
        });
        this.setFocusLast = (/**
         * @return {?}
         */
        () => {
            this.setFocus(this.Tabs - 1);
        });
        this.setFocusByFirstCharacter = (/**
         * @param {?} key
         * @return {?}
         */
        key => {
            key = key.toLowerCase();
            /** @type {?} */
            const arr = this.TabDic[key];
            /** @type {?} */
            let index;
            /** @type {?} */
            let length;
            /** @type {?} */
            let tabIndex = this.currentFocusValue;
            if (arr !== undefined) {
                length = arr.length;
                index = arr.indexOf(this.currentFocusValue);
                if (index !== -1 && length > 1 && index + 1 < length) {
                    tabIndex = arr[index + 1];
                }
                else {
                    tabIndex = arr[0];
                }
            }
            if (tabIndex !== this.currentFocusValue) {
                this.setFocus(tabIndex);
                return true;
            }
            return false;
        });
        this.current$.subscribe((/**
         * @param {?} index
         * @return {?}
         */
        index => {
            this.currentTabValue = index;
        }));
        this.focusIndex$.subscribe((/**
         * @param {?} index
         * @return {?}
         */
        index => {
            this.currentFocusValue = index;
        }));
        this.tabSize$.subscribe((/**
         * @param {?} n
         * @return {?}
         */
        n => {
            this.Tabs = n;
        }));
        this.paneSize$.subscribe((/**
         * @param {?} n
         * @return {?}
         */
        n => {
            this.Pane = n;
        }));
    }
}
TabsService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
TabsService.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TabsComponent {
    /**
     * @param {?} tabsService
     */
    constructor(tabsService) {
        this.tabsService = tabsService;
        /**
         * \@option Optional CSS class name | ''
         */
        this.className = '';
        /**
         * \@option Set the index of the focused Tab | 0
         */
        this.focus = 0;
        /**
         * \@option Determines if the Tabs are in a justified layout | false
         */
        this.justified = false;
        /**
         * \@option Type of Tabs | 'pills'
         */
        this.tabType = 'pills';
        /**
         * \@option Callback function invoked when user selects a tab | null
         */
        this.whenSelect = new EventEmitter();
        this.tabsService.current$.subscribe((/**
         * @param {?} currentIndex
         * @return {?}
         */
        currentIndex => {
            this.whenSelect.emit(currentIndex);
        }));
    }
    /**
     * @return {?}
     */
    get classes() {
        return ('cui-tab' +
            `${(this.tabType && ` cui-tab--${this.tabType}`) || ''}` +
            `${(this.justified && ' cui-tab--justified') || ''}` +
            `${(this.className && ` ${this.className}`) || ''}` +
            ``);
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.tabsService.select(this.focus, true);
    }
}
TabsComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-tabs',
                template: `
    <ng-content></ng-content>
  `,
                providers: [TabsService]
            }] }
];
/** @nocollapse */
TabsComponent.ctorParameters = () => [
    { type: TabsService }
];
TabsComponent.propDecorators = {
    className: [{ type: Input }],
    focus: [{ type: Input }],
    justified: [{ type: Input }],
    tabType: [{ type: Input }],
    whenSelect: [{ type: Output }],
    classes: [{ type: HostBinding, args: ['class',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TabsModule {
}
TabsModule.decorators = [
    { type: NgModule, args: [{
                declarations: [TabsComponent],
                imports: [CommonModule],
                exports: [TabsComponent],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TabComponent {
    /**
     * @param {?} tabsService
     * @param {?} el
     */
    constructor(tabsService, el) {
        this.tabsService = tabsService;
        this.el = el;
        /**
         * \@option Optional CSS class name
         */
        this.className = '';
        /**
         * \@option Sets the attribute disabled to the Tab | false
         */
        this.disabled = false;
        /**
         * \@option Tab's anchor role type | 'tab'
         */
        this.role = 'tab';
        /**
         * \@option Callback function invoked when user click a tab | null
         */
        this.whenPress = new EventEmitter();
        /**
         * \@option Callback function invoked when user press a key | null
         */
        this.whenKeyDown = new EventEmitter();
        this.attrRole = 'presentation';
        this.tabIndex = tabsService.registerTab();
        this.tabSize = this.tabIndex;
        this.tabsService.current$.subscribe((/**
         * @param {?} index
         * @return {?}
         */
        index => {
            this.ifCurrent = this.tabIndex === index;
        }));
        this.tabsService.focusIndex$.subscribe((/**
         * @param {?} index
         * @return {?}
         */
        index => {
            this.ifFocus = this.tabIndex === index;
            if (this.ifFocus) {
                this.el.nativeElement.getElementsByTagName('A')[0].focus();
            }
        }));
        this.tabsService.tabSize$.subscribe((/**
         * @param {?} n
         * @return {?}
         */
        n => {
            this.tabSize = n;
        }));
    }
    /*
      @HostBinding('attr.tabIndex') get attrTabIndex():number {
        return this.tabIndex;
      };
      @HostBinding('attr.aria-posinset') ariaPosinset = this.tabIndex;
      @HostBinding('attr.aria-setsize') ariaSetsize = this.tabSize;
      @HostBinding('attr.aria-selected') ariaSelected = this.ifCurrent;
      @HostBinding('attr.aria-disabled') ariaDisabled = this.disabled;
      */
    /**
     * @return {?}
     */
    get classes() {
        return ('cui-tab__item' +
            `${(this.className && ` ${this.className}`) || ''}` +
            `${(this.ifCurrent && ' active') || ''}` +
            `${(this.disabled && ' disabled') || ''}` +
            ``);
    }
    /**
     * @return {?}
     */
    select() {
        this.tabsService.select(this.tabIndex);
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.tabsService.registerDisable(this.tabIndex, this.disabled);
        /** @type {?} */
        const innerText = this.el.nativeElement.innerText;
        if (innerText) {
            this.tabsService.registerKey(innerText[0], this.tabIndex);
        }
    }
}
TabComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-tab',
                template: `
    <a
      href="javascript:void(0)"
      (click)="whenPress.emit()"
      (keydown)="whenKeyDown.emit()"
      [attr.role]="role"
      [attr.tabindex]="ifFocus ? 0 : -1"
      [attr.aria-posinset]="tabIndex"
      [attr.aria-setsize]="tabSize"
      [attr.aria-selected]="ifCurrent"
      [attr.aria-disabled]="disabled"
    >
      <ng-content></ng-content>
    </a>
  `
            }] }
];
/** @nocollapse */
TabComponent.ctorParameters = () => [
    { type: TabsService },
    { type: ElementRef }
];
TabComponent.propDecorators = {
    className: [{ type: Input }],
    disabled: [{ type: Input }],
    role: [{ type: Input }],
    whenPress: [{ type: Output }],
    whenKeyDown: [{ type: Output }],
    attrRole: [{ type: HostBinding, args: ['attr.role',] }],
    classes: [{ type: HostBinding, args: ['class',] }],
    select: [{ type: HostListener, args: ['click',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TabModule {
}
TabModule.decorators = [
    { type: NgModule, args: [{
                declarations: [TabComponent],
                imports: [CommonModule],
                exports: [TabComponent],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TabContentComponent {
    constructor() { }
    /**
     * @return {?}
     */
    get className() {
        return 'cui-tab__content';
    }
    /**
     * @return {?}
     */
    ngOnInit() { }
}
TabContentComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-tab-content',
                template: `
    <ng-content></ng-content>
  `
            }] }
];
/** @nocollapse */
TabContentComponent.ctorParameters = () => [];
TabContentComponent.propDecorators = {
    className: [{ type: HostBinding, args: ['class',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TabContentModule {
}
TabContentModule.decorators = [
    { type: NgModule, args: [{
                declarations: [TabContentComponent],
                imports: [CommonModule],
                exports: [TabContentComponent],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TabHeaderComponent {
    constructor() {
        /**
         * \@option Subheader text | ''
         */
        this.subHeading = '';
    }
    /**
     * @return {?}
     */
    ngOnInit() { }
}
TabHeaderComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-tab-header',
                template: `
    {{ heading }}
    <div *ngIf="subHeading">{{ subHeading }}</div>
  `
            }] }
];
/** @nocollapse */
TabHeaderComponent.ctorParameters = () => [];
TabHeaderComponent.propDecorators = {
    heading: [{ type: Input }],
    subHeading: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TabHeaderModule {
}
TabHeaderModule.decorators = [
    { type: NgModule, args: [{
                declarations: [TabHeaderComponent],
                imports: [CommonModule],
                exports: [TabHeaderComponent],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TabListComponent {
    /**
     * @param {?} tabsService
     */
    constructor(tabsService) {
        this.tabsService = tabsService;
        /**
         * \@option Tab's anchor role type | 'tablist'
         */
        this.role = 'tablist';
        this.regIsCharacter = /\S/;
        this.isPrintableCharacter = (/**
         * @param {?} str
         * @return {?}
         */
        str => {
            return str.length === 1 && str.match(this.regIsCharacter);
        });
        this.onPress = (/**
         * @param {?} e
         * @return {?}
         */
        e => {
            /** @type {?} */
            const key = e.keyCode;
            /** @type {?} */
            const tgt = e.target;
            /** @type {?} */
            let flag = false;
            /** @type {?} */
            let clickEvent;
            switch (key) {
                case 32:
                case 13:
                    try {
                        clickEvent = new MouseEvent('click', {
                            view: window,
                            bubbles: true,
                            cancelable: true,
                        });
                    }
                    catch (err) {
                        if (document.createEvent) {
                            clickEvent = document.createEvent('MouseEvents');
                            clickEvent.initEvent('click', true, true);
                        }
                    }
                    tgt.dispatchEvent(clickEvent);
                    flag = true;
                    break;
                case 38:
                case 37:
                    this.tabsService.setFocusPre();
                    flag = true;
                    break;
                case 39:
                case 40:
                    this.tabsService.setFocusNext();
                    flag = true;
                    break;
                case 33:
                case 36:
                    this.tabsService.setFocus(0);
                    flag = true;
                    break;
                case 34:
                case 35:
                    this.tabsService.setFocusLast();
                    flag = true;
                    break;
                default:
                    if (this.isPrintableCharacter(e.key)) {
                        flag = this.tabsService.setFocusByFirstCharacter(e.key);
                    }
                    break;
            }
            if (flag) {
                e.stopPropagation();
                e.preventDefault();
            }
        });
    }
    /**
     * @return {?}
     */
    get className() {
        return 'cui-tab__list';
    }
    /**
     * @return {?}
     */
    get roleName() {
        return this.role;
    }
    /**
     * @param {?} $event
     * @return {?}
     */
    select($event) {
        this.onPress($event);
    }
    /**
     * @return {?}
     */
    ngOnInit() { }
}
TabListComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-tab-list',
                template: `
    <ng-content></ng-content>
  `
            }] }
];
/** @nocollapse */
TabListComponent.ctorParameters = () => [
    { type: TabsService }
];
TabListComponent.propDecorators = {
    role: [{ type: Input }],
    className: [{ type: HostBinding, args: ['class',] }],
    roleName: [{ type: HostBinding, args: ['attr.role',] }],
    select: [{ type: HostListener, args: ['keyup', ['$event'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TabListModule {
}
TabListModule.decorators = [
    { type: NgModule, args: [{
                declarations: [TabListComponent],
                imports: [CommonModule],
                exports: [TabListComponent],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TabPaneComponent {
    /**
     * @param {?} tabsService
     */
    constructor(tabsService) {
        this.tabsService = tabsService;
        this.tabIndex = tabsService.registerPane();
        this.tabsService.current$.subscribe((/**
         * @param {?} currentIndex
         * @return {?}
         */
        currentIndex => {
            this.ifCurrent = this.tabIndex === currentIndex;
        }));
    }
    /**
     * @return {?}
     */
    get className() {
        return 'cui-tab__pane' + `${(this.ifCurrent && ' active') || ''}` + ``;
    }
    /**
     * @return {?}
     */
    ngOnInit() { }
}
TabPaneComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-tab-pane',
                template: `
    <div class="cui-tab__content">
      <ng-content></ng-content>
    </div>
  `
            }] }
];
/** @nocollapse */
TabPaneComponent.ctorParameters = () => [
    { type: TabsService }
];
TabPaneComponent.propDecorators = {
    className: [{ type: HostBinding, args: ['class',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TabPaneModule {
}
TabPaneModule.decorators = [
    { type: NgModule, args: [{
                declarations: [TabPaneComponent],
                imports: [CommonModule],
                exports: [TabPaneComponent],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// tslint:disable:no-use-before-declare
/** @type {?} */
const CUSTOM_TOGGLE_SWITCH_VALUE_ACCESSOR = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef((/**
     * @return {?}
     */
    () => ToggleSwitchComponent)),
    multi: true,
};
// tslint:enable:no-use-before-declare
class ToggleSwitchComponent {
    /**
     * @param {?} cdr
     */
    constructor(cdr) {
        this.cdr = cdr;
        /**
         * \@option Optional CSS class name | ''
         */
        this.class = '';
        /**
         * \@option Sets the attribute disabled to the ToggleSwitch | false
         */
        this.disabled = false;
        /**
         * \@option Unique HTML ID used for tying label to HTML input for automated testing
         */
        this.htmlId = '';
        /**
         * \@option ToggleSwitch label text | ''
         */
        this.label = '';
        /**
         * \@option ToggleSwitch name for group | ''
         */
        this.name = '';
        /**
         * \@option String value that corresponds with ToggleSwitch button | ''
         */
        this.value = '';
        /**
         * \@option Callback function invoked when user clicks the ToggleSwitch button | null
         */
        this.change = new EventEmitter();
        // pass in boolean to ngModel for filled.
        this.checked = false;
        this.onSwitchChange = (/**
         * @return {?}
         */
        () => { });
        this.onSwitchTouch = (/**
         * @return {?}
         */
        () => { });
    }
    /**
     * @param {?} checked
     * @return {?}
     */
    writeValue(checked) {
        this.checked = checked;
        this.cdr.markForCheck();
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onSwitchTouch = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onSwitchChange = fn;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onSwitch(event) {
        if (!this.disabled) {
            /** @type {?} */
            const isChecked = ((/** @type {?} */ (event.target))).checked;
            this.updateSwitch(event, isChecked);
        }
    }
    /**
     * @param {?} event
     * @param {?} bool
     * @return {?}
     */
    updateSwitch(event, bool) {
        this.checked = bool;
        this.onSwitchChange(this.checked);
        this.change.emit({ switchEvent: event, checked: this.checked });
    }
}
ToggleSwitchComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-toggle-switch',
                template: `
    <input
      class="cui-input cui-toggle-switch__input"
      type="checkbox"
      (change)="onSwitch($event)"
      [attr.id]="htmlId"
      [attr.name]="name"
      [attr.tabindex]="tabIndex"
      [checked]="checked"
      [disabled]="disabled"
    />
    <label class="cui-toggle-switch__label" [attr.for]="htmlId">
      <span class="cui-toggle-switch__label__container"></span>
      <span class="cui-toggle-switch__label__text">{{ label }}</span>
    </label>
  `,
                host: {
                    class: 'cui-input-group cui-toggle-switch',
                },
                providers: [CUSTOM_TOGGLE_SWITCH_VALUE_ACCESSOR]
            }] }
];
/** @nocollapse */
ToggleSwitchComponent.ctorParameters = () => [
    { type: ChangeDetectorRef }
];
ToggleSwitchComponent.propDecorators = {
    class: [{ type: Input }],
    disabled: [{ type: Input }],
    htmlId: [{ type: Input }],
    label: [{ type: Input }],
    name: [{ type: Input }],
    tabIndex: [{ type: Input }],
    value: [{ type: Input }],
    change: [{ type: Output }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ToggleSwitchModule {
}
ToggleSwitchModule.decorators = [
    { type: NgModule, args: [{
                declarations: [ToggleSwitchComponent],
                imports: [CommonModule, FormsModule, ReactiveFormsModule],
                exports: [ToggleSwitchComponent],
                schemas: [CUSTOM_ELEMENTS_SCHEMA],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TopbarComponent {
}
TopbarComponent.decorators = [
    { type: Component, args: [{
                selector: 'header[cui-top-bar]',
                template: `
    <div class="cui-top-bar__container">
      <div class="cui-top-bar__brand">
        <ng-content select="brand"></ng-content>
      </div>
      <ng-content></ng-content>
    </div>
  `,
                host: {
                    'attr.role': 'navigation',
                    class: 'cui-top-bar',
                    '[class.cui-top-bar--fixed]': 'fixed',
                    '[class.cui-top-bar--blue]': 'color === "blue"',
                    '[class.cui-top-bar--dark]': 'color === "dark"',
                    '[class.cui-top-bar--light]': 'color === "light"',
                }
            }] }
];
TopbarComponent.propDecorators = {
    color: [{ type: Input }],
    fixed: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TopbarBrandComponent {
    constructor() {
        /**
         * \@option Icon class name | 'icon-cisco-logo'
         */
        this.icon = 'icon-cisco-logo';
    }
}
TopbarBrandComponent.decorators = [
    { type: Component, args: [{
                selector: '[cui-top-bar-brand]',
                exportAs: 'cuiTopbarBrand',
                template: `
    <div class="cui-brand__logo">
      <img *ngIf="image" src="{{ image }}" alt="{{ alt }}" />
      <i *ngIf="!image" [ngClass]="['icon', icon]"></i>
    </div>
    <div class="cui-brand__title">{{ title }}</div>
  `,
                host: {
                    class: 'cui-brand',
                }
            }] }
];
TopbarBrandComponent.propDecorators = {
    alt: [{ type: Input }],
    icon: [{ type: Input }],
    image: [{ type: Input }],
    title: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TopbarMobileComponent {
    constructor() {
        /**
         * \@option Aria Label for close Button | 'Close Menu'
         */
        this.closeMenuAriaLabel = 'Close Menu';
        /**
         * \@option Aria Label for open Button | 'Open Menu
         */
        this.openMenuAriaLabel = 'Open Menu';
        /**
         * \@option Set mobile menu to close on any click | true
         */
        this.shouldCloseOnClick = true;
        this.isMobileOpen = false;
    }
    /**
     * @param {?} e
     * @return {?}
     */
    handleKeydown(e) {
        if (e.code === 'Space' || e.code === 'Enter') {
            this.handleClose();
            e.preventDefault();
        }
    }
    /**
     * @return {?}
     */
    handleClose() {
        this.isMobileOpen = false;
    }
    /**
     * @return {?}
     */
    handleOpen() {
        this.isMobileOpen = true;
    }
}
TopbarMobileComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-top-bar-mobile',
                template: `
    <cui-icon
      *ngIf="!isMobileOpen"
      name="list-menu_20"
      buttonClassName="cui-top-bar__mobile-menu-button"
      [attr.aria-pressed]="isMobileOpen"
      [ariaLabel]="openMenuAriaLabel"
      (click)="handleOpen()"
    ></cui-icon>
    <div
      [ngClass]="[
        'cui-top-bar__mobile',
        'cui-tb-mobile',
        (isMobileOpen && 'open') || ''
      ]"
      (click)="shouldCloseOnClick ? handleClose() : null"
      (keydown)="handleKeydown($event)"
      role="menu"
      tabIndex="0"
    >
      <cui-icon
        name="cancel_20"
        buttonClassName="cui-tb-mobile__close"
        [attr.aria-pressed]="isMobileOpen"
        [ariaLabel]="closeMenuAriaLabel"
        (click)="handleClose()"
      ></cui-icon>
      <span (click)="handleClose()" (keydown)="handleKeydown($event)">
        <div class="cui-top-bar__brand">
          <ng-content select="brand"></ng-content>
        </div>
      </span>
      <cui-list-separator backgroundColor="transparent"></cui-list-separator>
      <nav
        class="cui-tb-mobile__nav"
        (click)="!shouldCloseOnClick ? handleClose() : null"
      >
        <ng-content></ng-content>
      </nav>
    </div>
    <div
      [ngClass]="['cui-tb-mobile__mask', (isMobileOpen && 'open') || '']"
      (click)="handleClose()"
      role="none"
    ></div>
  `
            }] }
];
TopbarMobileComponent.propDecorators = {
    closeMenuAriaLabel: [{ type: Input }],
    openMenuAriaLabel: [{ type: Input }],
    shouldCloseOnClick: [{ type: Input }],
    handleKeydown: [{ type: HostListener, args: ['document:keydown', ['$event'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TopbarNavComponent {
}
TopbarNavComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-top-bar-nav',
                template: `
    <cui-list tabType="horizontal">
      <ng-content></ng-content>
    </cui-list>
  `,
                host: {
                    class: 'cui-top-bar__nav',
                }
            }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TopbarRightComponent {
}
TopbarRightComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-top-bar-right',
                template: `
    <ng-content></ng-content>
  `,
                host: {
                    class: 'cui-top-bar__right',
                }
            }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TopbarModule {
}
TopbarModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    IconModule,
                    ListModule,
                    ListItemModule,
                    ListSeparatorModule,
                ],
                declarations: [
                    TopbarComponent,
                    TopbarBrandComponent,
                    TopbarMobileComponent,
                    TopbarNavComponent,
                    TopbarRightComponent,
                ],
                exports: [
                    TopbarComponent,
                    TopbarBrandComponent,
                    TopbarMobileComponent,
                    TopbarNavComponent,
                    TopbarRightComponent,
                ],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AccordionService {
    constructor() {
        this.focus = new BehaviorSubject(0);
        this.focus$ = this.focus.asObservable();
    }
    /**
     * @param {?} idx
     * @return {?}
     */
    changeFocus(idx) {
        this.focus.next(idx);
    }
}
AccordionService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
AccordionService.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AccordionTabComponent {
    /**
     * @param {?} accordion
     * @param {?} el
     * @param {?} accordionService
     * @param {?} cdr
     */
    constructor(accordion, el, accordionService, cdr) {
        this.accordion = accordion;
        this.el = el;
        this.accordionService = accordionService;
        this.cdr = cdr;
        /**
         * \@prop Set Accordion tab header string | false
         */
        this.header = '';
        /**
         * \@prop Set accordion tab to be expanded | false
         */
        this.isExpanded = false;
        /**
         * \@prop Set the attribute disabled to the accordion tab | false
         */
        this.disabled = false;
        /**
         * \@prop Optional accordion tab css class string | ''
         */
        this.tabClass = '';
        /**
         * \@prop Optional underline under Accordion menu item | false
         */
        this.showSeparator = false;
        /**
         * \@prop Optional accordion header css class string | ''
         */
        this.headerClass = '';
        /**
         * \@prop Handler to be called when the user taps the Accordion Tab | null
         */
        this.contentClick = new EventEmitter();
        /**
         * \@prop Handler to be called when the user presses a key | null
         */
        this.handleKeyDown = new EventEmitter();
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (this.height && this.height !== 56) {
            throw new Error(`cui-list: Accordion Header height option must be either the default or 56`);
        }
        this.accordionService.focus$.subscribe((/**
         * @param {?} focus
         * @return {?}
         */
        focus => this.focus = focus));
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        this.index = this.findTabIndex();
        this.cdr.detectChanges();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    handleContentClick(event) {
        this.contentClick.emit({ originalEvent: event, tab: this.accordion.tabs[this.index] });
        event.stopPropagation();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    toggle(event) {
        if (this.disabled) {
            return false;
        }
        this.index = this.findTabIndex();
        this.accordionService.changeFocus(this.index);
        if (this.isExpanded) {
            this.isExpanded = false;
        }
        else {
            if (!this.accordion.multipleVisible) {
                for (let i = 0; i < this.accordion.tabs.length; i++) {
                    this.accordion.tabs[i].isExpanded = false;
                }
            }
            this.isExpanded = true;
        }
        event.preventDefault();
    }
    /**
     * @return {?}
     */
    findTabIndex() {
        /** @type {?} */
        let index = -1;
        this.accordion.tabs.map((/**
         * @param {?} tab
         * @param {?} idx
         * @return {?}
         */
        (tab, idx) => {
            if (tab === this) {
                index = idx;
            }
        }));
        return index;
    }
    /**
     * @param {?} currentIndex
     * @param {?} change
     * @param {?} lastIdx
     * @return {?}
     */
    getNewIndex(currentIndex, change, lastIdx) {
        /** @type {?} */
        const getPossibleIndex = (/**
         * @return {?}
         */
        () => {
            if (currentIndex + change < 0) {
                return lastIdx;
            }
            else if (currentIndex + change > lastIdx) {
                return 0;
            }
            return currentIndex + change;
        });
        /** @type {?} */
        const possibleIndex = getPossibleIndex();
        if (this.accordion.tabs[possibleIndex].disabled) {
            return this.getNewIndex(possibleIndex, change, lastIdx);
        }
        return possibleIndex;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onKeyDown(event) {
        /** @type {?} */
        let newIndex;
        this.handleKeyDown.emit(event.code);
        switch (event.code) {
            case 'Space':
            case 'Enter':
                this.toggle(event);
                break;
            case 'ArrowUp':
            case 'ArrowLeft':
                newIndex = this.getNewIndex(this.index, -1, this.accordion.tabs.length - 1);
                this.accordionService.changeFocus(newIndex);
                if (this.accordion.tabs[this.focus]) {
                    this.accordion.tabs[this.focus].headerRef.nativeElement.focus();
                }
                break;
            case 'Tab':
            case 'ArrowRight':
            case 'ArrowDown':
                newIndex = this.getNewIndex(this.index, 1, this.accordion.tabs.length - 1);
                this.accordionService.changeFocus(newIndex);
                if (this.accordion.tabs[this.focus]) {
                    this.accordion.tabs[this.focus].headerRef.nativeElement.focus();
                }
                break;
            case 'PageUp':
            case 'Home':
                this.accordionService.changeFocus(0);
                this.accordion.tabs[0].headerRef.nativeElement.focus();
                break;
            case 'PageDown':
            case 'End':
                this.accordionService.changeFocus(this.accordion.tabs.length - 1);
                this.accordion.tabs[this.accordion.tabs.length - 1].headerRef.nativeElement.focus();
                break;
            default:
                break;
        }
        event.preventDefault();
    }
    /**
     * @return {?}
     */
    get tabWrapperClasses() {
        return {
            ['cui-accordion__group--disabled']: this.disabled,
            'cui-accordion__group--active': this.isExpanded,
            [this.tabClass]: this.tabClass
        };
    }
    /**
     * @return {?}
     */
    get headerClasses() {
        return {
            ['cui-accordion__header--' + this.height]: this.height === 56,
            'cui-accordion__header--separator': this.showSeparator,
            [this.headerClass]: this.headerClass
        };
    }
}
AccordionTabComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-accordion-tab',
                template: `
    <div
      [attr.aria-expanded]="isExpanded"
      class="cui-accordion__group"
      [ngClass]="tabWrapperClasses"
    >
      <div
        #headerRef
        role="button"
        class="cui-accordion__header"
        [ngClass]="headerClasses"
        (click) = "toggle($event)"
        (keydown)="onKeyDown($event)"
        [attr.tabindex]="!disabled ? 0 : -1"
      >
        <div>{{header}}</div>
        <span class="cui-accordion__header-icon"></span>
      </div>

      <div class="cui-accordion__content">
        <div
          class="cui-list-item"
          id="cui-list-item-12"
          role="listItem"
          (click) = "handleContentClick($event)"
        >
          <ng-content></ng-content>
        </div>
      </div>
    </div>
  `
            }] }
];
/** @nocollapse */
AccordionTabComponent.ctorParameters = () => [
    { type: AccordionComponent, decorators: [{ type: Inject, args: [forwardRef((/**
                     * @return {?}
                     */
                    () => AccordionComponent)),] }] },
    { type: ElementRef },
    { type: AccordionService },
    { type: ChangeDetectorRef }
];
AccordionTabComponent.propDecorators = {
    header: [{ type: Input }],
    isExpanded: [{ type: Input }],
    disabled: [{ type: Input }],
    tabClass: [{ type: Input }],
    height: [{ type: Input }],
    showSeparator: [{ type: Input }],
    headerClass: [{ type: Input }],
    headerRef: [{ type: ViewChild, args: ['headerRef',] }],
    contentRef: [{ type: ViewChild, args: ['contentRef',] }],
    contentClick: [{ type: Output }],
    handleKeyDown: [{ type: Output }]
};
// Accordion Below
class AccordionComponent {
    constructor() {
        /**
         * \@prop Set to allow expansion of multiple AccordionGroups | false
         */
        this.multipleVisible = false;
        /**
         * \@prop Optional css class string | ''
         */
        this.class = '';
        /**
         * \@prop Specifies if AccordionHeader automatically gets focus when page loads | false
         */
        this.focus = false;
        this.tabs = [];
    }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        this.tabs = this.tabList.toArray();
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        if (this.focus) {
            this.tabs[0].headerRef.nativeElement.focus();
        }
    }
    /**
     * @return {?}
     */
    get wrapperClasses() {
        return {
            [this.class]: this.class,
        };
    }
}
AccordionComponent.decorators = [
    { type: Component, args: [{
                selector: 'cui-accordion',
                template: `
    <div
      class="cui-accordion"
      [ngClass]="wrapperClasses"
    >
      <ng-content></ng-content>
    </div>
  `
            }] }
];
/** @nocollapse */
AccordionComponent.ctorParameters = () => [];
AccordionComponent.propDecorators = {
    multipleVisible: [{ type: Input }],
    class: [{ type: Input }],
    focus: [{ type: Input }],
    tabList: [{ type: ContentChildren, args: [AccordionTabComponent,] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AccordionModule {
}
AccordionModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule
                ],
                declarations: [AccordionComponent, AccordionTabComponent],
                exports: [AccordionComponent, AccordionTabComponent],
                schemas: [CUSTOM_ELEMENTS_SCHEMA],
                providers: [AccordionService],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { AlertBannerModule, AlertBannerComponent, AlertMeetingModule, AlertMeetingService, AlertMeetingContainerComponent, AlertMeetingConfig, AlertMeetingComponent, AlertModule, AlertService, AlertContainerComponent, AlertConfig, AlertComponent, AvatarModule, AvatarComponent, BadgeModule, BadgeComponent, ButtonModule, ButtonComponent, CheckboxModule, CheckboxComponent, CompositeAvatarModule, CompositeAvatarComponent, EditableTextfieldModule, EditableTextfieldComponent, IconModule, IconComponent, InputErrorModule, InputErrorComponent, InputHelperModule, InputHelperComponent, InputModule, InputComponent, LabelModule, LabelComponent, ListModule, ListComponent, ListItemModule, OptionSelectionChange, ListItemComponent, ListItemSectionModule, ListItemSectionComponent, ListSeparatorModule, ListSeparatorComponent, LoadingModule, LoadingComponent, ModalBodyModule, ModalBodyComponent, ModalFooterModule, ModalFooterComponent, ModalHeaderModule, ModalHeaderComponent, ModalModule, ModalComponent, RadioModule, RadioComponent, SearchInputModule, SearchInputComponent, SliderModule, SliderComponent, SliderPointerComponent, SelectModule, SelectChange, SelectComponent, SpinnerModule, SpinnerComponent, TabsModule, TabsComponent, TabModule, TabComponent, TabContentModule, TabContentComponent, TabHeaderModule, TabHeaderComponent, TabListModule, TabListComponent, TabPaneModule, TabPaneComponent, ToggleSwitchModule, ToggleSwitchComponent, TopbarModule, TopbarComponent, TopbarBrandComponent, TopbarMobileComponent, TopbarNavComponent, TopbarRightComponent, AccordionModule, AccordionTabComponent, AccordionComponent };

//# sourceMappingURL=collab-ui-angular.js.map