/** @component input */
import { OnChanges, SimpleChanges, EventEmitter } from '@angular/core';
import { ControlValueAccessor, NgControl } from '@angular/forms';
export declare class InputComponent implements ControlValueAccessor, OnChanges {
    control: NgControl;
    value: any;
    readonly hasRequiredError: boolean;
    readonly errorValid: any;
    constructor(control: NgControl);
    readonly wrapperClasses: {
        [x: string]: any;
        [x: number]: any;
        'read-only': boolean;
        disabled: boolean;
        error: boolean;
    };
    readonly inputClasses: {
        [x: string]: any;
        'read-only': boolean;
        disabled: boolean;
        dirty: any;
    };
    private innerValue;
    private onTouchedCallback;
    private onChangeCallback;
    /** @option Optional css class string | ''  */
    class: string;
    /** @option optional button to clear input text */
    clear: boolean;
    /** @option Sets the disabled attribute of the Input | false */
    disabled: boolean;
    /** @option Unique HTML ID used for tying label to HTML input for automated testing */
    htmlId: string;
    /** @option Array of objects with error type and error message */
    errorArr: any[];
    /** @option Input css class name string */
    inputClass: string;
    /** @option Help Text to appear under the input | '' */
    inputHelpText: string;
    /** @option Overall input group size | '' */
    inputSize: string;
    /** @option Input label text | '' */
    label: string;
    /** @option Placeholder text to display when Input is empty | '' */
    placeholder: string;
    /** @option Determines if Input can be edited | false */
    readOnly: boolean;
    /** @option Secondary Input label | ''  */
    secondaryLabel: string;
    /** @option Input color theme | '' */
    theme: string;
    /** @option Input type | 'text' */
    type: string;
    /** @option Sets the attribute name to the input element | '' */
    name: string;
    /** @option Optional error messages object with angular validators | {} */
    errorObj: object;
    /** @option function when clicked outside of input */
    handleBlur: EventEmitter<any>;
    /** @option function when input is focused */
    handleFocus: EventEmitter<any>;
    /** @option function when key down on input */
    handleKeyDown: EventEmitter<any>;
    /** @option function when mouse down on input */
    handleMouseDown: EventEmitter<any>;
    errorType: any;
    errors: any;
    onBlur(event: any): void;
    onFocus(event: any): void;
    onKeyDown(event: any): void;
    onMouseDown(event: any): void;
    writeValue(value: any): void;
    registerOnChange(fn: any): void;
    registerOnTouched(fn: any): void;
    private determineErrorType;
    private filterErrorsByType;
    ngOnChanges(changes: SimpleChanges): void;
    handleClear(): void;
}
/**
* @component input
* @section default
* @angular
*

<div>
  <cui-input
    [(ngModel)]="dataModel"
    inputSize="small-5"
    label="Default Input"
  >
  </cui-input>
</div>
*/
/**
* @component input
* @section error
* @angular
*
<div>
  <cui-input
    inputSize="small-5"
    label="Error (Error) Input"
    [errorArr]="[{error: 'This is where the error message would be.', type: 'error'}]"
  >
  </cui-input>
</div>
*/
/**
* @component input
* @section warning
* @angular
*
<div>
  <cui-input
    inputSize="small-5"
    label="Error (Warning) Input"
    [errorArr]="[{error: 'This is where the success message would be.', type: 'warning'}]"
  >
  </cui-input>
</div>
*/
/**
* @component input
* @section success
* @angular
*
<div>
  <cui-input
    inputSize="small-5"
    label="Error (Success) Input"
    [errorArr]="[{error: 'This is where the success message would be.', type: 'success'}]"
  >
  </cui-input>
</div>
*/
/**
* @component input
* @section disabled
* @angular
*
<div>
  <cui-input
    inputSize="small-5"
    [disabled]="true"
    label="Disabled Input"
  >
  </cui-input>
</div>
*/
/**
* @component input
* @section read-only
* @angular
*
<div>
  <cui-input
    inputSize="small-5"
    [readOnly]="true"
    label="Read Only Input"
    [(ngModel)]="dataModel"
  >
  </cui-input>
</div>
*/
/**
* @component input
* @section help-text
* @angular
*
<div>
  <cui-input
    inputSize="small-5"
    inputHelpText="Help Text"
    label="Help Text Input"
  >
  </cui-input>
</div>
*/
/**
* @component input
* @section secondary-label
* @angular
*
<div>
  <cui-input
    inputSize="small-5"
    secondaryLabel="Secondary Label"
  >
  </cui-input>
</div>
*/
