import getColorValue from '@collab-ui/utils/lib/getColorValue';
import find from 'lodash-es/find';
import reject from 'lodash-es/reject';
import { uniqueId as uniqueId$1 } from 'lodash';
import { BrowserModule } from '@angular/platform-browser';
import isObject from 'lodash-es/isObject';
import { __assign, __spread } from 'tslib';
import { SelectionModel } from '@angular/cdk/collections';
import uniqueId from 'lodash-es/uniqueId';
import { take, switchMap, takeUntil, startWith } from 'rxjs/operators';
import { ENTER, SPACE, hasModifierKey, A, DOWN_ARROW, END, HOME, UP_ARROW } from '@angular/cdk/keycodes';
import { Overlay, OverlayModule, OverlayConfig } from '@angular/cdk/overlay';
import { ComponentPortal, PortalModule, PortalInjector, TemplatePortal } from '@angular/cdk/portal';
import { ActiveDescendantKeyManager, A11yModule } from '@angular/cdk/a11y';
import { NG_VALUE_ACCESSOR, FormsModule, ReactiveFormsModule, NgControl } from '@angular/forms';
import { CommonModule, DOCUMENT } from '@angular/common';
import { Component, EventEmitter, HostBinding, Input, Output, ViewEncapsulation, NgModule, CUSTOM_ELEMENTS_SCHEMA, HostListener, ViewChild, ElementRef, Injectable, Injector, forwardRef, ChangeDetectorRef, Optional, Self, ContentChildren, NgZone, ViewContainerRef, Inject } from '@angular/core';
import { Subject, defer, merge, BehaviorSubject } from 'rxjs';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// const getColorValue = require('@collab-ui/utils/lib/getColorValue');
/** @type {?} */
var iconNames = require('@collab-ui/icons/data/iconNames.json');
var IconComponent = /** @class */ (function () {
    function IconComponent() {
        var _this = this;
        this.click = new EventEmitter();
        this.getIconNameClass = (/**
         * @return {?}
         */
        function () {
            /** @type {?} */
            var defaultSize = 16;
            /** @type {?} */
            var sizeFromName = Number(_this.name.split('_')[1]);
            /** @type {?} */
            var nameWithSize = _this.name;
            if (sizeFromName && _this.fontSize) {
                nameWithSize = _this.name.split('_')[0] + "_" + _this.fontSize;
            }
            if (!sizeFromName) {
                nameWithSize = _this.name + "_" + (_this.size || defaultSize);
            }
            /** @type {?} */
            var lookupIconName = nameWithSize.replace(/^(icon-)/, '');
            /** @type {?} */
            var iconNameClass = "icon-" + lookupIconName;
            // validate Icon name with Collab Toolkit
            if (((/** @type {?} */ (iconNames))).indexOf(lookupIconName) < 0) {
                _this.consoleHandler('name-error', nameWithSize);
            }
            return iconNameClass;
        });
        this.getColor = (/**
         * @return {?}
         */
        function () {
            if (!_this.color) {
                return 'inherit';
            }
            if (_this.color.startsWith('#')) {
                _this.consoleHandler('color-warn', _this.color);
                return _this.color;
            }
            return getColorValue(_this.color);
        });
        this.consoleHandler = (/**
         * @param {?} message
         * @param {?} data
         * @return {?}
         */
        function (message, data) {
            switch (message) {
                case 'color-warn':
                    console.warn("[@collab-ui/angular] Icon: " + data + " may not exist in the design system," +
                        " please use a color name from http://collab-ui.cisco.com/styles/colors");
                    break;
                case 'name-error':
                    console.warn("[@collab-ui/angular] Icon: Icon " + data + " does not exist in the design system." +
                        " Visit https://icons.collab-ui.com for a list of available icons or to request a new icon.");
                    break;
            }
        });
    }
    Object.defineProperty(IconComponent.prototype, "_class", {
        get: /**
         * @return {?}
         */
        function () {
            return !this.isClickable ? this.classes : '';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IconComponent.prototype, "_color", {
        get: /**
         * @return {?}
         */
        function () {
            return !this.isClickable ? this.getColor() : '';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IconComponent.prototype, "_ariaLabelledBy", {
        get: /**
         * @return {?}
         */
        function () {
            return !this.isClickable ? this.ariaLabelledBy : '';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IconComponent.prototype, "_ariaLabel", {
        get: /**
         * @return {?}
         */
        function () {
            return !this.isClickable ? this.ariaLabel : null;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    IconComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        // input validations
        if (!this.name) {
            throw new Error('Attribute "name" is required');
        }
        if (this.type && this.type !== 'white' && this.type !== '') {
            throw new Error('Attribute "type" if defined is either "" or "white"');
        }
        this.isClickable = this.click.observers.length > 0;
        this.classes =
            'cui-icon icon' +
                ("" + ((this.name && " " + this.getIconNameClass()) || '')) +
                ("" + ((this.className && " " + this.className) || '')) +
                "";
        this.buttonClasses =
            'cui-button cui-button--icon' +
                ("" + ((this.type && " cui-button--icon-" + this.type) || '')) +
                ("" + ((this.buttonClassName && " " + this.buttonClassName) || '')) +
                "";
        /** @type {?} */
        var iconCount = uniqueId();
        this.titleId = "icon-title-" + iconCount;
        this.descId = this.description ? "icon-desc-" + iconCount : undefined;
    };
    Object.defineProperty(IconComponent.prototype, "ariaLabelledBy", {
        get: /**
         * @return {?}
         */
        function () {
            var _a = this, ariaLabel = _a.ariaLabel, title = _a.title, description = _a.description, titleId = _a.titleId, descId = _a.descId;
            if (!ariaLabel) {
                if (title && description) {
                    return titleId + " " + descId;
                }
                if (title) {
                    return "" + titleId;
                }
                if (description) {
                    return "" + descId;
                }
            }
            return null;
        },
        enumerable: true,
        configurable: true
    });
    IconComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-icon',
                    template: "\n    <button\n      *ngIf=\"isClickable\"\n      class=\"{{ buttonClasses }}\"\n      [attr.aria-labelledby]=\"ariaLabelledBy\"\n      [attr.aria-label]=\"ariaLabel\"\n    >\n      <i class=\"{{ classes }}\" [style.color]=\"getColor()\"></i>\n    </button>\n  ",
                    encapsulation: ViewEncapsulation.None
                }] }
    ];
    /** @nocollapse */
    IconComponent.ctorParameters = function () { return []; };
    IconComponent.propDecorators = {
        ariaLabel: [{ type: Input }],
        buttonClassName: [{ type: Input }],
        color: [{ type: Input }],
        className: [{ type: Input }],
        description: [{ type: Input }],
        name: [{ type: Input }],
        size: [{ type: Input }],
        fontSize: [{ type: Input }],
        title: [{ type: Input }],
        type: [{ type: Input }],
        click: [{ type: Output }],
        _class: [{ type: HostBinding, args: ['class',] }],
        _color: [{ type: HostBinding, args: ['style.color',] }],
        _ariaLabelledBy: [{ type: HostBinding, args: ['attr.aria-labelledby',] }],
        _ariaLabel: [{ type: HostBinding, args: ['attr.aria-label',] }]
    };
    return IconComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var IconModule = /** @class */ (function () {
    function IconModule() {
    }
    IconModule.decorators = [
        { type: NgModule, args: [{
                    imports: [CommonModule],
                    declarations: [IconComponent],
                    exports: [IconComponent],
                    schemas: [CUSTOM_ELEMENTS_SCHEMA]
                },] }
    ];
    return IconModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var AlertBannerComponent = /** @class */ (function () {
    function AlertBannerComponent() {
        /**
         * Sets the visibility of AlertBanner's close button | false
         */
        this.closable = false;
        /**
         * Sets the AlertBanner type | 'info'
         */
        this.type = 'info';
        /**
         * Event emitted when the user presses on the AlertBanner's close button or hits the esc key
         */
        this.hide = new EventEmitter();
    }
    /**
     * @param {?} _
     * @return {?}
     */
    AlertBannerComponent.prototype.handleKeydown = /**
     * @param {?} _
     * @return {?}
     */
    function (_) {
        this.hide.emit();
    };
    /**
     * @return {?}
     */
    AlertBannerComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        if (!this.hide.observers.length) {
            throw new Error('Observer for "hide" is required');
        }
    };
    /**
     * @return {?}
     */
    AlertBannerComponent.prototype.onClick = /**
     * @return {?}
     */
    function () {
        this.hide.emit();
    };
    AlertBannerComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-alert-banner',
                    template: "\n    <div [ngClass]=\"['cui-alert-banner', 'cui-alert-banner--' + type]\">\n      <div class=\"cui-alert-banner__text\">\n        <ng-content></ng-content>\n      </div>\n      <div\n        *ngIf=\"closable\"\n        class=\"cui-alert-banner__close\"\n        (click)=\"onClick()\"\n        (keydown)=\"handleKeydown($event)\"\n        tabindex=\"0\"\n        role=\"button\"\n      >\n        <cui-icon name=\"cancel_16\"></cui-icon>\n      </div>\n    </div>\n  "
                }] }
    ];
    /** @nocollapse */
    AlertBannerComponent.ctorParameters = function () { return []; };
    AlertBannerComponent.propDecorators = {
        closable: [{ type: Input }],
        type: [{ type: HostBinding, args: ['attr.type',] }, { type: Input }],
        hide: [{ type: Output }],
        handleKeydown: [{ type: HostListener, args: ['document:keydown.escape', ['$event'],] }]
    };
    return AlertBannerComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var AlertBannerModule = /** @class */ (function () {
    function AlertBannerModule() {
    }
    AlertBannerModule.decorators = [
        { type: NgModule, args: [{
                    imports: [CommonModule, IconModule],
                    declarations: [AlertBannerComponent],
                    exports: [AlertBannerComponent],
                },] }
    ];
    return AlertBannerModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ButtonComponent = /** @class */ (function () {
    function ButtonComponent(el) {
        var _this = this;
        this.el = el;
        /**
         * \@option Active Sets button to active | false
         */
        this.active = false;
        /** @option AriaLabel Text to display for blindness accessibility features | '' */
        /** @option AriaLabelledBy ID to reference for blindness accessibility feature | '' */
        /**
         * \@option Creates a circle shaped button | false
         */
        this.circle = false;
        /**
         * \@option Optional css class string | ''
         */
        this.class = '';
        /**
         * \@option Sets optional button color | ''
         */
        this.color = '';
        /**
         * \@option Sets the attribute disabled to Button | false
         */
        this.disabled = false;
        /**
         * \@option Sets expand css styling to widen the Button | false
         */
        this.expand = false;
        /**
         * \@option Sets href for an anchor tagged button | ''
         */
        this.href = '';
        /**
         * \@option Activates the loading animation and sets the button to disabled | false
         */
        this.loading = false;
        /**
         * \@option Boolean to remove Button's default style | false
         */
        this.removeStyle = false;
        /**
         * \@option Size value | '36'
         */
        this.size = '36';
        /**
         * \@option Html button type | 'button'
         */
        this.type = 'button';
        this.getColor = (/**
         * @return {?}
         */
        function () {
            if (_this.removeStyle) {
                return false;
            }
            return _this.color === 'none' ? 'color-none' : _this.color;
        });
        this.getSize = (/**
         * @return {?}
         */
        function () {
            if (_this.removeStyle) {
                return false;
            }
            /** @type {?} */
            var validButtonSize = _this.checkButtonSize();
            if (!_this.circle && !validButtonSize) {
                console.warn("[@collab-ui/angular] Button: selected size is not supported\n         for non-circular button. Size will default to 36");
                return '36';
            }
            else {
                return _this.size === 'none' ? 'size-none' : _this.size;
            }
        });
        this.checkButtonSize = (/**
         * @return {?}
         */
        function () {
            return ['none', '28', '36', '40', '52', 28, 36, 40, 52].includes(_this.size);
        });
    }
    Object.defineProperty(ButtonComponent.prototype, "activeOption", {
        get: /**
         * @return {?}
         */
        function () {
            return this.active || null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ButtonComponent.prototype, "disabledOption", {
        get: /**
         * @return {?}
         */
        function () {
            return this.disabled || null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ButtonComponent.prototype, "hrefOption", {
        get: /**
         * @return {?}
         */
        function () {
            return this.href || null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ButtonComponent.prototype, "typeOption", {
        get: /**
         * @return {?}
         */
        function () {
            return this.type || 'button';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ButtonComponent.prototype, "className", {
        get: /**
         * @return {?}
         */
        function () {
            return ('cui-button' +
                ("" + ((this.circle && " cui-button--circle") || '')) +
                ("" + ((this.removeStyle && ' cui-button--none') || '')) +
                ("" + ((this.getSize() && " cui-button--" + this.getSize()) || '')) +
                ("" + ((this.getColor() && " cui-button--" + this.getColor()) || '')) +
                ("" + ((this.expand && " cui-button--expand") || '')) +
                ("" + ((this.class && " " + this.class) || '')) +
                ("" + ((this.disabled && " cui-button--disabled") || '')) +
                ("" + ((this.active && !this.disabled && " active") || '')) +
                "");
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    ButtonComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        this.disabled = this.disabled || this.loading || null;
    };
    /**
     * @return {?}
     */
    ButtonComponent.prototype.ngAfterViewInit = /**
     * @return {?}
     */
    function () {
        this.initWidth = this.element.nativeElement.offsetWidth + 0.001 + "px";
        if (this.type &&
            this.type !== 'button' &&
            this.type !== 'reset' &&
            this.type !== 'submit') {
            throw new Error('cui-button: Button type must be one of the following: button, reset, submit');
        }
    };
    /**
     * @return {?}
     */
    ButtonComponent.prototype.ngAfterContentInit = /**
     * @return {?}
     */
    function () {
        var _this = this;
        setTimeout((/**
         * @return {?}
         */
        function () { return _this.setAriaLabel(); }), 100);
    };
    /**
     * @param {?} changes
     * @return {?}
     */
    ButtonComponent.prototype.ngOnChanges = /**
     * @param {?} changes
     * @return {?}
     */
    function (changes) {
        if (changes.loading && changes.loading.currentValue) {
            this.width = this.initWidth;
        }
        else {
            this.width = '';
        }
    };
    Object.defineProperty(ButtonComponent.prototype, "opacity", {
        get: /**
         * @return {?}
         */
        function () {
            return this.loading ? 0 : 1;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @private
     * @return {?}
     */
    ButtonComponent.prototype.setAriaLabel = /**
     * @private
     * @return {?}
     */
    function () {
        /** @type {?} */
        var ariaLabel = this.getAttr('aria-label');
        if (!ariaLabel) {
            if (this.element.nativeElement.children.length > 0) {
                throw new Error('cui-button: content is not a string, you must add an "ariaLabel" for accessibility.');
            }
            else {
                ariaLabel = this.element.nativeElement.innerText;
            }
        }
    };
    /**
     * @private
     * @param {?} attr
     * @return {?}
     */
    ButtonComponent.prototype.getAttr = /**
     * @private
     * @param {?} attr
     * @return {?}
     */
    function (attr) {
        return this.el.nativeElement.getAttribute(attr);
    };
    ButtonComponent.decorators = [
        { type: Component, args: [{
                    selector: 'button[cui-button], a[cui-button], input[cui-button]',
                    exportAs: 'cuiButton',
                    template: "\n    <div #divBody>\n      <cui-loading *ngIf=\"loading\"></cui-loading>\n      <span [ngStyle]=\"{ opacity: opacity }\">\n        <ng-content></ng-content>\n      </span>\n    </div>\n  "
                }] }
    ];
    /** @nocollapse */
    ButtonComponent.ctorParameters = function () { return [
        { type: ElementRef }
    ]; };
    ButtonComponent.propDecorators = {
        active: [{ type: Input }],
        circle: [{ type: Input }],
        class: [{ type: Input }],
        color: [{ type: Input }],
        disabled: [{ type: Input }],
        expand: [{ type: Input }],
        href: [{ type: Input }],
        loading: [{ type: Input }],
        removeStyle: [{ type: Input }],
        size: [{ type: Input }],
        type: [{ type: Input }],
        element: [{ type: ViewChild, args: ['divBody',] }],
        loaderChild: [{ type: ViewChild, args: ['loader',] }],
        width: [{ type: HostBinding, args: ['attr.width',] }],
        alt: [{ type: HostBinding, args: ['attr.alt',] }],
        activeOption: [{ type: HostBinding, args: ['attr.active',] }],
        disabledOption: [{ type: HostBinding, args: ['attr.disabled',] }],
        hrefOption: [{ type: HostBinding, args: ['attr.href',] }],
        typeOption: [{ type: HostBinding, args: ['attr.type',] }],
        className: [{ type: HostBinding, args: ['class',] }]
    };
    return ButtonComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var LoadingComponent = /** @class */ (function () {
    function LoadingComponent() {
        /**
         * \@option Prop to make the Loading animation small | false
         */
        this.small = false;
    }
    Object.defineProperty(LoadingComponent.prototype, "className", {
        get: /**
         * @return {?}
         */
        function () {
            return "cui-loading " + (this.small ? 'cui-loading--small' : '');
        },
        enumerable: true,
        configurable: true
    });
    LoadingComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-loading, CSLoading',
                    template: "\n    <span class=\"cui-loading__icon\"></span>\n    <span class=\"cui-loading__icon\"></span>\n    <span class=\"cui-loading__icon\"></span>\n  ",
                    encapsulation: ViewEncapsulation.None
                }] }
    ];
    /** @nocollapse */
    LoadingComponent.ctorParameters = function () { return []; };
    LoadingComponent.propDecorators = {
        small: [{ type: Input }],
        className: [{ type: HostBinding, args: ['class',] }]
    };
    return LoadingComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var LoadingModule = /** @class */ (function () {
    function LoadingModule() {
    }
    /**
     * @return {?}
     */
    LoadingModule.forRoot = /**
     * @return {?}
     */
    function () {
        return {
            ngModule: LoadingModule,
            providers: [],
        };
    };
    LoadingModule.decorators = [
        { type: NgModule, args: [{
                    imports: [CommonModule],
                    exports: [LoadingComponent],
                    declarations: [LoadingComponent],
                },] }
    ];
    return LoadingModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ButtonModule = /** @class */ (function () {
    function ButtonModule() {
    }
    /**
     * @return {?}
     */
    ButtonModule.forRoot = /**
     * @return {?}
     */
    function () {
        return {
            ngModule: ButtonModule,
            providers: [],
        };
    };
    ButtonModule.decorators = [
        { type: NgModule, args: [{
                    imports: [CommonModule, LoadingModule.forRoot()],
                    exports: [ButtonComponent],
                    declarations: [ButtonComponent],
                    schemas: [CUSTOM_ELEMENTS_SCHEMA]
                },] }
    ];
    return ButtonModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var AvatarComponent = /** @class */ (function () {
    function AvatarComponent(elementRef) {
        this.elementRef = elementRef;
        this.ariaLabel = '';
        /**
         * \@option Image alt tag | ''
         */
        this.alt = '';
        /**
         * \@option Set Avatar background color | ''
         */
        this.backgroundColor = '';
        /**
         * \@option Optional css class string for button | ''
         */
        this.buttonClassName = '';
        /**
         * \@option Set Avatar text color | ''
         */
        this.color = '';
        /**
         * \@option Set existance of a failureBadge on the Avatar | false
         */
        this.failureBadge = false;
        /**
         * \@option Set existance of a notification on the Avatar | false
         */
        this.hasNotification = false;
        /**
         * \@option Set the visibility of Avatar's default tooltip | false
         */
        this.hideDefaultTooltip = false;
        /**
         * \@option Set if Avatar's content is decrypting | false
         */
        this.isDecrypting = false;
        /**
         * \@option Set existance of Avatar's Overview | false
         */
        this.isOverview = false;
        this.click = new EventEmitter();
        this._size = 'medium';
        this._type = '';
        this.clickable = false;
        this.isImageLoaded = false;
        this.isImageErrored = false;
    }
    Object.defineProperty(AvatarComponent.prototype, "size", {
        /** @option Set the size of the Avatar from one of the preconfigured options | 'medium' */
        get: /**
         * \@option Set the size of the Avatar from one of the preconfigured options | 'medium'
         * @return {?}
         */
        function () {
            return this._size;
        },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) {
            if (!this.clickable) {
                this.elementRef.nativeElement.classList.remove("cui-avatar--" + this._size);
                this.elementRef.nativeElement.classList.add("cui-avatar--" + value);
            }
            this._size = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AvatarComponent.prototype, "theme", {
        /** @option Optional Avatar color theme | null */
        get: /**
         * \@option Optional Avatar color theme | null
         * @return {?}
         */
        function () {
            return this._theme;
        },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) {
            if (!this.clickable) {
                this.elementRef.nativeElement.classList.remove("cui-avatar--" + this._theme);
                this.elementRef.nativeElement.classList.add("cui-avatar--" + value);
            }
            this._theme = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AvatarComponent.prototype, "type", {
        /** @option optional Avatar type | '' */
        get: /**
         * \@option optional Avatar type | ''
         * @return {?}
         */
        function () {
            return this._type;
        },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) {
            if (!this.clickable) {
                this.elementRef.nativeElement.classList.remove("cui-avatar--" + this._type);
                this.elementRef.nativeElement.classList.add("cui-avatar--" + value);
            }
            this._type = value;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    AvatarComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        /** @type {?} */
        var img = this.image;
        if (img && img.nativeElement.complete) {
            this.handleImgLoaded();
        }
        this.clickable = this.click.observers.length > 0;
    };
    /**
     * @param {?} changes
     * @return {?}
     */
    AvatarComponent.prototype.ngOnChanges = /**
     * @param {?} changes
     * @return {?}
     */
    function (changes) {
        if (changes.src && changes.src.previousValue !== changes.src.currentValue) {
            this.handleImgChange();
        }
    };
    /**
     * @return {?}
     */
    AvatarComponent.prototype.handleImgChange = /**
     * @return {?}
     */
    function () {
        this.isImageLoaded = false;
        this.isImageErrored = false;
    };
    /**
     * @return {?}
     */
    AvatarComponent.prototype.handleImgError = /**
     * @return {?}
     */
    function () {
        this.isImageErrored = true;
    };
    /**
     * @return {?}
     */
    AvatarComponent.prototype.handleImgLoaded = /**
     * @return {?}
     */
    function () {
        this.isImageLoaded = true;
    };
    /**
     * @return {?}
     */
    AvatarComponent.prototype.getInitials = /**
     * @return {?}
     */
    function () {
        if (!this.title || !this.title.replace(/\s/g, '').length) {
            return '';
        }
        /** @type {?} */
        var letters = [];
        /** @type {?} */
        var words = this.title.trim().split(/ +/);
        /** @type {?} */
        var repeatTimes = Math.min((this.type === 'group' && 1) || 2, words.length);
        for (var i = 0; i < repeatTimes; i++) {
            letters.push(String.fromCodePoint(words[i].codePointAt(0)));
        }
        return letters.join('');
    };
    AvatarComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-avatar',
                    template: "\n    <button\n      cui-button\n      *ngIf=\"clickable; else avatarTpl\"\n      [attr.aria-label]=\"ariaLabel\"\n      [class]=\"buttonClassName\"\n      [circle]=\"true\"\n      [removeStyle]=\"true\"\n    >\n      <ng-container *ngTemplateOutlet=\"avatarTpl\"></ng-container>\n    </button>\n\n    <ng-template #avatarTpl>\n      <div\n        *ngIf=\"clickable; else overlayTpl\"\n        class=\"cui-avatar\"\n        [ngClass]=\"[\n          'cui-avatar--clickable',\n          (type && 'cui-avatar--' + type) || '',\n          (size && 'cui-avatar--' + size) || '',\n          (theme && 'cui-avatar--' + theme) || '',\n          (isDecrypting && 'cui-decrypting') || '',\n          className || ''\n        ]\"\n        [title]=\"!hideDefaultTooltip ? title : ''\"\n      >\n        <ng-container *ngTemplateOutlet=\"overlayTpl\"></ng-container>\n      </div>\n    </ng-template>\n\n    <ng-template #overlayTpl>\n      <ng-container *ngTemplateOutlet=\"childrenTpl\"></ng-container>\n      <cui-loading *ngIf=\"type === 'typing'\"></cui-loading>\n      <span *ngIf=\"failureBadge\" class=\"cui-avatar__failure-badge\"></span>\n      <span\n        *ngIf=\"hasNotification\"\n        class=\"cui-avatar__notification-badge\"\n      ></span>\n    </ng-template>\n\n    <ng-template #childrenTpl>\n      <span\n        *ngIf=\"type === 'self'; else imageTpl\"\n        class=\"cui-avatar__self\"\n        [ngStyle]=\"{\n          'background-color': backgroundColor,\n          color: color\n        }\"\n      >\n        <cui-icon\n          [name]=\"\n            size === 40 || size === 'medium'\n              ? 'chat-active_18'\n              : 'chat-active_16'\n          \"\n          color=\"blue-50\"\n        ></cui-icon>\n      </span>\n    </ng-template>\n\n    <ng-template #imageTpl>\n      <ng-container *ngIf=\"src && !isImageErrored; else iconTpl\">\n        <ng-container *ngIf=\"title && !isImageLoaded\">\n          <ng-container *ngTemplateOutlet=\"letterTpl\"></ng-container>\n        </ng-container>\n        <img\n          #image\n          [alt]=\"alt\"\n          class=\"cui-avatar__img\"\n          [ngClass]=\"{ 'cui-avatar__img--hidden': !isImageLoaded }\"\n          draggable=\"false\"\n          (error)=\"handleImgError()\"\n          (load)=\"handleImgLoaded()\"\n          [src]=\"src\"\n        />\n      </ng-container>\n    </ng-template>\n\n    <ng-template #iconTpl>\n      <span\n        *ngIf=\"icon; else letterTpl\"\n        class=\"cui-avatar__icon\"\n        [ngClass]=\"{ 'cui-avatar__icon--overview': isOverview }\"\n        [ngStyle]=\"{\n          'background-color': backgroundColor,\n          color: color\n        }\"\n      >\n        <cui-icon [name]=\"icon\"></cui-icon>\n      </span>\n    </ng-template>\n\n    <ng-template #letterTpl>\n      <span\n        class=\"cui-avatar__letter\"\n        [ngClass]=\"{\n          'cui-decrypting': isDecrypting,\n          'cui-avatar__img--hidden': isImageLoaded\n        }\"\n        [ngStyle]=\"{\n          'background-color': backgroundColor,\n          color: color\n        }\"\n        >{{ getInitials() }}</span\n      >\n    </ng-template>\n  ",
                    host: {
                        '[attr.title]': '!hideDefaultTooltip ? title : ""',
                        class: 'cui-avatar cui-avatar--medium',
                        '[class.cui-decrypting]': '!clickable && isDecrypting',
                    }
                }] }
    ];
    /** @nocollapse */
    AvatarComponent.ctorParameters = function () { return [
        { type: ElementRef }
    ]; };
    AvatarComponent.propDecorators = {
        ariaLabel: [{ type: Input }],
        alt: [{ type: Input }],
        backgroundColor: [{ type: Input }],
        buttonClassName: [{ type: Input }],
        className: [{ type: Input }],
        color: [{ type: Input }],
        failureBadge: [{ type: Input }],
        hasNotification: [{ type: Input }],
        hideDefaultTooltip: [{ type: Input }],
        icon: [{ type: Input }],
        isDecrypting: [{ type: Input }],
        isOverview: [{ type: Input }],
        size: [{ type: Input }],
        src: [{ type: Input }],
        theme: [{ type: Input }],
        title: [{ type: Input }],
        type: [{ type: Input }],
        click: [{ type: Output }],
        image: [{ type: ViewChild, args: ['image',] }]
    };
    return AvatarComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var AvatarModule = /** @class */ (function () {
    function AvatarModule() {
    }
    AvatarModule.decorators = [
        { type: NgModule, args: [{
                    imports: [CommonModule, ButtonModule, IconModule, LoadingModule],
                    declarations: [AvatarComponent],
                    exports: [AvatarComponent],
                },] }
    ];
    return AvatarModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var CompositeAvatarComponent = /** @class */ (function () {
    function CompositeAvatarComponent() {
        this.className = '';
        this.size = 'medium';
    }
    Object.defineProperty(CompositeAvatarComponent.prototype, "_class", {
        get: /**
         * @return {?}
         */
        function () {
            return ('cui-composite-avatar' +
                ("" + ((this.size && " cui-composite-avatar--" + this.size) || '')) +
                ("" + ((this.className && " " + this.className) || '')));
        },
        enumerable: true,
        configurable: true
    });
    CompositeAvatarComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-composite-avatar',
                    template: "\n    <ng-content></ng-content>\n  "
                }] }
    ];
    /** @nocollapse */
    CompositeAvatarComponent.ctorParameters = function () { return []; };
    CompositeAvatarComponent.propDecorators = {
        className: [{ type: Input }],
        size: [{ type: Input }],
        _class: [{ type: HostBinding, args: ['class',] }]
    };
    return CompositeAvatarComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var CompositeAvatarModule = /** @class */ (function () {
    function CompositeAvatarModule() {
    }
    CompositeAvatarModule.decorators = [
        { type: NgModule, args: [{
                    imports: [CommonModule, AvatarModule],
                    declarations: [CompositeAvatarComponent],
                    exports: [CompositeAvatarComponent],
                },] }
    ];
    return CompositeAvatarModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// Used internally by the alert meeting service.
var AlertMeetingComponent = /** @class */ (function () {
    function AlertMeetingComponent() {
        this.attendees = [];
        this.closeAriaLabel = 'close';
        this.message = '';
        this.remindAriaLabel = 'snooze';
        this.status = '';
        this.title = '';
        this.mouseclick = new EventEmitter();
        this.hide = new EventEmitter();
        this.snooze = new EventEmitter();
    }
    Object.defineProperty(AlertMeetingComponent.prototype, "contentClasses", {
        get: /**
         * @return {?}
         */
        function () {
            return [
                'cui-alert__content',
                this.snoozeButtonVisible ? '' : 'cui-alert__content--wide'
            ];
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    AlertMeetingComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        if (!this.avatar && this.attendees.length === 0) {
            throw new Error('MeetingAlert needs at least one attendee to render an avatar.');
        }
    };
    /**
     * @return {?}
     */
    AlertMeetingComponent.prototype.onClick = /**
     * @return {?}
     */
    function () {
        this.mouseclick.emit(this.key);
    };
    /**
     * @return {?}
     */
    AlertMeetingComponent.prototype.onSnooze = /**
     * @return {?}
     */
    function () {
        this.snooze.emit(this.key);
    };
    /**
     * @return {?}
     */
    AlertMeetingComponent.prototype.onHide = /**
     * @return {?}
     */
    function () {
        this.hide.emit(this.key);
    };
    /**
     * @param {?} event
     * @return {?}
     */
    AlertMeetingComponent.prototype.onKeydown = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        if (event.code === 'Space' || event.code === 'Enter') {
            this.onClick();
            event.preventDefault();
        }
    };
    AlertMeetingComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-alert-meeting',
                    template: "\n    <ng-container *ngIf=\"avatar; else avatarTemplate\" [ngTemplateOutlet]=\"avatar\"></ng-container>\n    <div [ngClass]=\"contentClasses\">\n      <div class=\"cui-alert__title\" [attr.title]=\"title\">{{ title }}</div>\n      <div class=\"cui-alert__status\">{{ status }}</div>\n      <div class=\"cui-alert__message\" [attr.title]=\"message\">{{ message }}</div>\n    </div>\n    <div class=\"cui-alert__button\" *ngIf=\"snoozeButtonVisible\">\n      <button\n        cui-button\n        [attr.aria-label]=\"remindAriaLabel\"\n        [circle]=\"true\"\n        size=\"44\"\n        tabindex=\"0\"\n        (click)=\"onSnooze()\"\n      >\n        <cui-icon name=\"alarm_16\"></cui-icon>\n      </button>\n    </div>\n    <div class=\"cui-alert__button\">\n      <button\n        cui-button\n        [attr.aria-label]=\"closeAriaLabel\"\n        [circle]=\"true\"\n        size=\"44\"\n        tabindex=\"0\"\n        (click)=\"onHide()\"\n      >\n        <cui-icon name=\"cancel_16\"></cui-icon>\n      </button>\n    </div>\n\n    <ng-template #avatarTemplate>\n      <cui-composite-avatar *ngIf=\"attendees.length >= 2\">\n        <cui-avatar\n          [title]=\"attendees[0].title\"\n          [alt]=\"attendees[0].alt\"\n          [src]=\"attendees[0].src\"\n        ></cui-avatar>\n        <cui-avatar\n          [title]=\"attendees[1].title\"\n          [alt]=\"attendees[1].alt\"\n          [src]=\"attendees[1].src\"\n        ></cui-avatar>\n      </cui-composite-avatar>\n      <cui-avatar\n        *ngIf=\"attendees.length === 1\"\n        [title]=\"attendees[0].title\"\n        [alt]=\"attendees[0].alt\"\n        [src]=\"attendees[0].src\"\n      ></cui-avatar>\n    </ng-template>\n  ",
                    host: {
                        'class': 'cui-alert cui-alert--meeting',
                        '[attr.role]': 'clickable ? "button" : null',
                        '[attr.tabindex]': 'clickable ? "0" : null',
                        '(click)': 'clickable ? onClick() : null',
                        '(keydown)': 'clickable ? onKeydown($event) : null',
                    }
                }] }
    ];
    AlertMeetingComponent.propDecorators = {
        key: [{ type: Input }],
        attendees: [{ type: Input }],
        avatar: [{ type: Input }],
        clickable: [{ type: Input }],
        closeAriaLabel: [{ type: Input }],
        message: [{ type: Input }],
        remindAriaLabel: [{ type: Input }],
        snoozeButtonVisible: [{ type: Input }],
        status: [{ type: Input }],
        title: [{ type: Input }],
        mouseclick: [{ type: Output }],
        hide: [{ type: Output }],
        snooze: [{ type: Output }]
    };
    return AlertMeetingComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// Used internally by the alert meeting service.
var AlertMeetingContainerComponent = /** @class */ (function () {
    function AlertMeetingContainerComponent() {
        this._onDestroy = new Subject();
        this.alertList = [];
    }
    /**
     * @param {?} alert
     * @return {?}
     */
    AlertMeetingContainerComponent.prototype.addAlert = /**
     * @param {?} alert
     * @return {?}
     */
    function (alert) {
        alert.key = uniqueId('alert_meeting_');
        this.alertList.push(alert);
        return alert.key;
    };
    /**
     * @param {?} key
     * @return {?}
     */
    AlertMeetingContainerComponent.prototype.removeAlert = /**
     * @param {?} key
     * @return {?}
     */
    function (key) {
        this.alertList = reject(this.alertList, { key: key });
        if (!this.alertList.length) {
            this.destroy();
        }
    };
    /**
     * @param {?} key
     * @return {?}
     */
    AlertMeetingContainerComponent.prototype.onClick = /**
     * @param {?} key
     * @return {?}
     */
    function (key) {
        /** @type {?} */
        var alert = find(this.alertList, { key: key });
        if (alert && alert.onClick) {
            alert.onClick();
        }
        this.removeAlert(key);
    };
    /**
     * @param {?} key
     * @return {?}
     */
    AlertMeetingContainerComponent.prototype.onHide = /**
     * @param {?} key
     * @return {?}
     */
    function (key) {
        /** @type {?} */
        var alert = find(this.alertList, { key: key });
        if (alert && alert.onHide) {
            alert.onHide();
        }
        this.removeAlert(key);
    };
    /**
     * @param {?} key
     * @return {?}
     */
    AlertMeetingContainerComponent.prototype.onSnooze = /**
     * @param {?} key
     * @return {?}
     */
    function (key) {
        /** @type {?} */
        var alert = find(this.alertList, { key: key });
        if (alert && alert.onSnooze) {
            alert.onSnooze();
        }
        this.removeAlert(key);
    };
    /**
     * @return {?}
     */
    AlertMeetingContainerComponent.prototype.destroy = /**
     * @return {?}
     */
    function () {
        this._onDestroy.next();
        this._onDestroy.complete();
    };
    /**
     * @return {?}
     */
    AlertMeetingContainerComponent.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.destroy();
    };
    AlertMeetingContainerComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-alert-meeting-container',
                    template: "\n    <cui-alert-meeting *ngFor=\"let alert of alertList\"\n      [key]=\"alert.key\"\n      [attendees]=\"alert.attendees\"\n      [avatar]=\"alert.avatar\"\n      [clickable]=\"!!alert.onClick\"\n      [closeAriaLabel]=\"alert.closeAriaLabel\"\n      [message]=\"alert.message\"\n      [remindAriaLabel]=\"alert.remindAriaLabel\"\n      [snoozeButtonVisible]=\"!!alert.onSnooze\"\n      [status]=\"alert.status\"\n      [title]=\"alert.title\"\n      (mouseclick)=\"onClick($event)\"\n      (hide)=\"onHide($event)\"\n      (snooze)=\"onSnooze($event)\"\n    ></cui-alert-meeting>\n  ",
                    host: {
                        'class': 'cui-alert__container cui-alert__container--bottom-right'
                    }
                }] }
    ];
    return AlertMeetingContainerComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// Configuration used when opening an alert.
var  
// Configuration used when opening an alert.
AlertMeetingConfig = /** @class */ (function () {
    function AlertMeetingConfig() {
        /**
         * Optional attendee array. If more than one attendee, a Composite Avatar with only the first 2 attendees is created.
         */
        this.attendees = [];
        /**
         * Optional aria label for the close button | 'close'
         */
        this.closeAriaLabel = 'close';
        /**
         * Optional AlertMeeting Message | ''
         */
        this.message = '';
        /**
         * Optional aria label for snooze buton | 'snooze'
         */
        this.remindAriaLabel = 'snooze';
        /**
         * Optional AlertMeeting status | ''
         */
        this.status = '';
        /**
         * Optional AlertMeeting title | ''
         */
        this.title = '';
        /**
         * Optional callback function invoked on click of alert | null
         */
        this.onClick = null;
        /**
         * Optional handler invoked when the user presses on the Alert's close button or hit's the esc key | null
         */
        this.onHide = null;
        /**
         * Optional callback function invoked when the snooze button is clicked | null
         */
        this.onSnooze = null;
    }
    return AlertMeetingConfig;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var AlertMeetingService = /** @class */ (function () {
    function AlertMeetingService(_overlay) {
        this._overlay = _overlay;
        this._overlayRef = null;
        this._alertContainerRef = null;
    }
    /**
     * @param {?} config
     * @return {?}
     */
    AlertMeetingService.prototype.show = /**
     * @param {?} config
     * @return {?}
     */
    function (config) {
        if (!this._overlayRef) {
            this._overlayRef = this._overlay.create();
        }
        /** @type {?} */
        var container = this._attachAlertContainer(this._overlayRef);
        /** @type {?} */
        var _config = __assign({}, new AlertMeetingConfig(), config);
        return container.addAlert(_config);
    };
    /**
     * @param {?} key
     * @return {?}
     */
    AlertMeetingService.prototype.hide = /**
     * @param {?} key
     * @return {?}
     */
    function (key) {
        if (this._alertContainerRef) {
            this._alertContainerRef.instance.removeAlert(key);
        }
    };
    /**
     * @private
     * @param {?} overlayRef
     * @return {?}
     */
    AlertMeetingService.prototype._attachAlertContainer = /**
     * @private
     * @param {?} overlayRef
     * @return {?}
     */
    function (overlayRef) {
        var _this = this;
        if (this._alertContainerRef) {
            return this._alertContainerRef.instance;
        }
        /** @type {?} */
        var containerPortal = new ComponentPortal(AlertMeetingContainerComponent);
        this._alertContainerRef = overlayRef.attach(containerPortal);
        /** @type {?} */
        var onDestroySub = this._alertContainerRef.instance._onDestroy.subscribe((/**
         * @return {?}
         */
        function () {
            onDestroySub.unsubscribe();
            _this._dispose();
        }));
        return this._alertContainerRef.instance;
    };
    /**
     * @return {?}
     */
    AlertMeetingService.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        if (this._alertContainerRef) {
            this._alertContainerRef.instance.destroy();
        }
    };
    /**
     * @private
     * @return {?}
     */
    AlertMeetingService.prototype._dispose = /**
     * @private
     * @return {?}
     */
    function () {
        this._alertContainerRef = null;
        this._overlayRef.dispose();
        this._overlayRef = null;
    };
    AlertMeetingService.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    AlertMeetingService.ctorParameters = function () { return [
        { type: Overlay }
    ]; };
    return AlertMeetingService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var AlertMeetingModule = /** @class */ (function () {
    function AlertMeetingModule() {
    }
    AlertMeetingModule.decorators = [
        { type: NgModule, args: [{
                    imports: [
                        OverlayModule,
                        PortalModule,
                        CommonModule,
                        AvatarModule,
                        ButtonModule,
                        CompositeAvatarModule,
                        IconModule,
                    ],
                    declarations: [AlertMeetingComponent, AlertMeetingContainerComponent],
                    exports: [AlertMeetingContainerComponent],
                    entryComponents: [AlertMeetingContainerComponent],
                    providers: [AlertMeetingService]
                },] }
    ];
    return AlertMeetingModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// Used internally by the alert service.
var AlertComponent = /** @class */ (function () {
    function AlertComponent() {
        this.dismissed = new EventEmitter();
    }
    Object.defineProperty(AlertComponent.prototype, "className", {
        get: /**
         * @return {?}
         */
        function () {
            return "cui-alert cui-alert--" + this.type;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    AlertComponent.prototype.dismiss = /**
     * @return {?}
     */
    function () {
        this.dismissed.emit(this.key);
    };
    AlertComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-alert',
                    template: "\n    <div class=\"cui-alert__icon\"></div>\n    <div class=\"cui-alert__content\">\n      <div class=\"cui-alert__title\">{{ title }}</div>\n      <div class=\"cui-alert__message\">{{ message }}</div>\n    </div>\n    <div class=\"cui-alert__button\" *ngIf=\"closable\">\n      <button\n        cui-button\n        #closeButton\n        [attr.aria-label]=\"ariaLabel\"\n        [circle]=\"true\"\n        size=\"44\"\n        (click)=\"dismiss()\"\n      >\n        <cui-icon name=\"cancel_16\"></cui-icon>\n      </button>\n    </div>\n  "
                }] }
    ];
    /** @nocollapse */
    AlertComponent.ctorParameters = function () { return []; };
    AlertComponent.propDecorators = {
        key: [{ type: Input }],
        type: [{ type: Input }],
        ariaLabel: [{ type: Input }],
        closable: [{ type: Input }],
        message: [{ type: Input }],
        title: [{ type: Input }],
        className: [{ type: HostBinding, args: ['class',] }],
        dismissed: [{ type: Output }]
    };
    return AlertComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Configuration used when opening an alert.
 */
var  /**
 * Configuration used when opening an alert.
 */
AlertConfig = /** @class */ (function () {
    function AlertConfig() {
        this.ariaLabel = 'Close';
        /**
         * To show/hide Close button of the Alert | true
         */
        this.closable = true;
        /**
         * Optional Alert message | ''
         */
        this.message = '';
        /**
         * Handler invoked when the user presses on the Alert's close button | null
         */
        this.onHide = null;
        /**
         * Display newest alert messages first | true
         */
        this.orderNewest = true;
        /**
         * Define alert's position with css class name: 'bottom-right'
         */
        this.position = 'bottom-right';
        /**
         * Optional Alert title | ''
         */
        this.title = '';
        /**
         * Sets the type of the Alert | 'info'
         */
        this.type = 'info';
    }
    return AlertConfig;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Used internally by the alert service.
 */
var AlertContainerComponent = /** @class */ (function () {
    function AlertContainerComponent(alertConfig) {
        this.alertConfig = alertConfig;
        this.role = 'alert';
        this._onDestroy = new Subject();
        this.alertList = [];
    }
    Object.defineProperty(AlertContainerComponent.prototype, "className", {
        get: /**
         * @return {?}
         */
        function () {
            return "cui-alert__container cui-alert__container--" + this.alertConfig.position;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @param {?} alert
     * @return {?}
     */
    AlertContainerComponent.prototype.addAlert = /**
     * @param {?} alert
     * @return {?}
     */
    function (alert) {
        alert.key = uniqueId('alert_');
        this.alertList = this.alertConfig.orderNewest
            ? __spread([alert], this.alertList) : __spread(this.alertList, [alert]);
        return alert.key;
    };
    /**
     * @param {?} key
     * @return {?}
     */
    AlertContainerComponent.prototype.removeAlert = /**
     * @param {?} key
     * @return {?}
     */
    function (key) {
        /** @type {?} */
        var alert = find(this.alertList, { key: key });
        if (alert && alert.onHide) {
            alert.onHide();
        }
        this.alertList = reject(this.alertList, { key: key });
        if (!this.alertList.length) {
            this.destroy();
        }
    };
    /**
     * @return {?}
     */
    AlertContainerComponent.prototype.destroy = /**
     * @return {?}
     */
    function () {
        this._onDestroy.next();
        this._onDestroy.complete();
    };
    /**
     * @return {?}
     */
    AlertContainerComponent.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.destroy();
    };
    AlertContainerComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-alert-container',
                    template: "\n    <ng-container *ngFor=\"let alert of alertList\">\n      <cui-alert\n        [key]=\"alert.key\"\n        [ariaLabel]=\"alert.ariaLabel\"\n        [type]=\"alert.type\"\n        [title]=\"alert.title\"\n        [message]=\"alert.message\"\n        [closable]=\"alert.closable\"\n        (dismissed)=\"removeAlert($event)\"\n      ></cui-alert>\n    </ng-container>\n  "
                }] }
    ];
    /** @nocollapse */
    AlertContainerComponent.ctorParameters = function () { return [
        { type: AlertConfig }
    ]; };
    AlertContainerComponent.propDecorators = {
        role: [{ type: HostBinding, args: ['attr.role',] }],
        className: [{ type: HostBinding, args: ['class',] }]
    };
    return AlertContainerComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var AlertService = /** @class */ (function () {
    function AlertService(_overlay, _injector) {
        this._overlay = _overlay;
        this._injector = _injector;
        this._overlayRef = null;
        this._alertContainerRef = null;
    }
    /**
     * @param {?} title
     * @param {?} message
     * @param {?=} config
     * @return {?}
     */
    AlertService.prototype.info = /**
     * @param {?} title
     * @param {?} message
     * @param {?=} config
     * @return {?}
     */
    function (title, message, config) {
        return this._open('info', title, message, config);
    };
    /**
     * @param {?} title
     * @param {?} message
     * @param {?=} config
     * @return {?}
     */
    AlertService.prototype.success = /**
     * @param {?} title
     * @param {?} message
     * @param {?=} config
     * @return {?}
     */
    function (title, message, config) {
        return this._open('success', title, message, config);
    };
    /**
     * @param {?} title
     * @param {?} message
     * @param {?=} config
     * @return {?}
     */
    AlertService.prototype.warning = /**
     * @param {?} title
     * @param {?} message
     * @param {?=} config
     * @return {?}
     */
    function (title, message, config) {
        return this._open('warning', title, message, config);
    };
    /**
     * @param {?} title
     * @param {?} message
     * @param {?=} config
     * @return {?}
     */
    AlertService.prototype.error = /**
     * @param {?} title
     * @param {?} message
     * @param {?=} config
     * @return {?}
     */
    function (title, message, config) {
        return this._open('error', title, message, config);
    };
    /**
     * @param {?} key
     * @return {?}
     */
    AlertService.prototype.hide = /**
     * @param {?} key
     * @return {?}
     */
    function (key) {
        if (this._alertContainerRef) {
            this._alertContainerRef.instance.removeAlert(key);
        }
    };
    /**
     * @private
     * @param {?} type
     * @param {?} title
     * @param {?} message
     * @param {?=} config
     * @return {?}
     */
    AlertService.prototype._open = /**
     * @private
     * @param {?} type
     * @param {?} title
     * @param {?} message
     * @param {?=} config
     * @return {?}
     */
    function (type, title, message, config) {
        /** @type {?} */
        var _config = __assign({}, new AlertConfig(), config);
        _config.type = type;
        _config.title = title;
        _config.message = message;
        if (!this._overlayRef) {
            this._overlayRef = this._overlay.create();
        }
        /** @type {?} */
        var container = this._attachAlertContainer(this._overlayRef, _config);
        return container.addAlert(_config);
    };
    /**
     * @private
     * @param {?} overlayRef
     * @param {?} config
     * @return {?}
     */
    AlertService.prototype._attachAlertContainer = /**
     * @private
     * @param {?} overlayRef
     * @param {?} config
     * @return {?}
     */
    function (overlayRef, config) {
        var _this = this;
        if (this._alertContainerRef) {
            return this._alertContainerRef.instance;
        }
        /** @type {?} */
        var injector = new PortalInjector(this._injector, new WeakMap([[AlertConfig, config]]));
        /** @type {?} */
        var containerPortal = new ComponentPortal(AlertContainerComponent, null, injector);
        this._alertContainerRef = overlayRef.attach(containerPortal);
        /** @type {?} */
        var onDestroySub = this._alertContainerRef.instance._onDestroy.subscribe((/**
         * @return {?}
         */
        function () {
            onDestroySub.unsubscribe();
            _this._dispose();
        }));
        return this._alertContainerRef.instance;
    };
    /**
     * @return {?}
     */
    AlertService.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        if (this._alertContainerRef) {
            this._alertContainerRef.instance.destroy();
        }
    };
    /**
     * @private
     * @return {?}
     */
    AlertService.prototype._dispose = /**
     * @private
     * @return {?}
     */
    function () {
        this._alertContainerRef = null;
        this._overlayRef.dispose();
        this._overlayRef = null;
    };
    AlertService.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    AlertService.ctorParameters = function () { return [
        { type: Overlay },
        { type: Injector }
    ]; };
    return AlertService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var AlertModule = /** @class */ (function () {
    function AlertModule() {
    }
    AlertModule.decorators = [
        { type: NgModule, args: [{
                    imports: [
                        OverlayModule,
                        PortalModule,
                        CommonModule,
                        ButtonModule,
                        IconModule,
                    ],
                    declarations: [AlertComponent, AlertContainerComponent],
                    exports: [AlertContainerComponent],
                    entryComponents: [AlertContainerComponent],
                    providers: [AlertService],
                },] }
    ];
    return AlertModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var BadgeComponent = /** @class */ (function () {
    function BadgeComponent(elementRef) {
        this.elementRef = elementRef;
        /**
         * \@option Sets the optional rounded class | false
         */
        this.rounded = false;
        this._color = null;
    }
    Object.defineProperty(BadgeComponent.prototype, "color", {
        /** @option Sets optional button color | null */
        set: /**
         * \@option Sets optional button color | null
         * @param {?} color
         * @return {?}
         */
        function (color) {
            if (this._color) {
                this.elementRef.nativeElement.classList.remove("cui-badge--" + this._color);
            }
            this.elementRef.nativeElement.classList.add("cui-badge--" + color);
            this._color = color;
        },
        enumerable: true,
        configurable: true
    });
    BadgeComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-badge',
                    template: "\n    <ng-content></ng-content>\n  ",
                    host: {
                        class: 'cui-badge',
                        '[class.cui-badge--round]': 'rounded',
                    }
                }] }
    ];
    /** @nocollapse */
    BadgeComponent.ctorParameters = function () { return [
        { type: ElementRef }
    ]; };
    BadgeComponent.propDecorators = {
        rounded: [{ type: Input }],
        color: [{ type: Input }]
    };
    return BadgeComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var BadgeModule = /** @class */ (function () {
    function BadgeModule() {
    }
    BadgeModule.decorators = [
        { type: NgModule, args: [{
                    imports: [CommonModule],
                    declarations: [BadgeComponent],
                    exports: [BadgeComponent],
                },] }
    ];
    return BadgeModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// tslint:disable:no-use-before-declare
/** @type {?} */
var CUSTOM_CHECKBOX_CONTROL_VALUE_ACCESSOR = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef((/**
     * @return {?}
     */
    function () { return CheckboxComponent; })),
    multi: true,
};
// tslint:enable:no-use-before-declare
var CheckboxComponent = /** @class */ (function () {
    function CheckboxComponent(cdr) {
        this.cdr = cdr;
        /**
         * \@option String value that corresponds with Checkbox  | ''
         */
        this.value = '';
        /**
         * \@option index of the checkbox in tab order
         */
        this.tabIndex = 0;
        /**
         * \@option Optional css class string | ''
         */
        this.class = '';
        /**
         * \@option Sets the disabled attribute of the Input | false
         */
        this.disabled = false;
        /**
         * \@option Optional indeterminate capabilities of checkbox | false
         */
        this.indeterminate = false;
        /**
         * \@option Input label text | ''
         */
        this.label = '';
        /**
         * \@option Set the level of nested checkboxes | 0
         */
        this.nestedLevel = 0;
        /**
         * \@option Optional required setting for Checkbox input | false
         */
        this.required = false;
        /**
         * \@option Unique HTML ID. Used for tying label to HTML input | ''
         */
        this.htmlId = '';
        /**
         * \@option sets value of the Checkbox input element | false
         */
        this.selectedItem = false;
        /**
         * \@option Sets the attribute name to the Checkbox input element | ''
         */
        this.name = '';
        /**
         * \@prop optional emitter to invoke an checkStatusChange handler when checkbox toggles
         */
        this.checkStatusChange = new EventEmitter();
        this.checked = false;
        this.onListChange = (/**
         * @return {?}
         */
        function () { });
        this.onListTouched = (/**
         * @return {?}
         */
        function () { });
    }
    Object.defineProperty(CheckboxComponent.prototype, "checkStatus", {
        get: /**
         * @return {?}
         */
        function () {
            return this.checked;
        },
        set: /**
         * @param {?} status
         * @return {?}
         */
        function (status) {
            if (status !== undefined && !this.disabled) {
                this.checked = status;
                this.checkStatusChange.emit(this.checked);
                this.updateList();
            }
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @param {?} list
     * @return {?}
     */
    CheckboxComponent.prototype.writeValue = /**
     * @param {?} list
     * @return {?}
     */
    function (list) {
        this.list = list;
        this.checked = this.isChecked();
        this.cdr.markForCheck();
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    CheckboxComponent.prototype.registerOnChange = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this.onListChange = fn;
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    CheckboxComponent.prototype.registerOnTouched = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this.onListTouched = fn;
    };
    /**
     * @param {?} e
     * @return {?}
     */
    CheckboxComponent.prototype.onClick = /**
     * @param {?} e
     * @return {?}
     */
    function (e) {
        event.preventDefault();
        if (this.checkStatusChange.observers.length > 0) {
            return;
        }
        if (this.disabled) {
            return;
        }
        this.checked = !this.checked;
        this.updateList();
    };
    /**
     * @return {?}
     */
    CheckboxComponent.prototype.updateList = /**
     * @return {?}
     */
    function () {
        if (this.indeterminate) {
            return;
        }
        if (this.checked) {
            this.addCheck();
        }
        else {
            this.uncheck();
        }
        this.onListChange(this.list);
        if (this.formControl) {
            this.formControl.setValue(this.list);
        }
        this.checkStatusChange.emit(this.checked);
    };
    /**
     * @param {?} event
     * @return {?}
     */
    CheckboxComponent.prototype.handleChange = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        this.checked = event.target.checked;
        this.updateList();
    };
    /**
     * @return {?}
     */
    CheckboxComponent.prototype.isChecked = /**
     * @return {?}
     */
    function () {
        return this.list && this.list.includes(this.value);
    };
    /**
     * @return {?}
     */
    CheckboxComponent.prototype.uncheck = /**
     * @return {?}
     */
    function () {
        var _this = this;
        this.list = this.list.filter((/**
         * @param {?} check
         * @return {?}
         */
        function (check) { return check !== _this.value; }));
    };
    /**
     * @return {?}
     */
    CheckboxComponent.prototype.addCheck = /**
     * @return {?}
     */
    function () {
        if (this.list) {
            this.list = __spread(this.list, [this.value]);
        }
        else {
            this.list = [this.value];
        }
    };
    Object.defineProperty(CheckboxComponent.prototype, "wrapperClasses", {
        get: /**
         * @return {?}
         */
        function () {
            var _a;
            return _a = {},
                _a['cui-input--nested-' + this.nestedLevel] = this.nestedLevel,
                _a[this.class] = this.class,
                _a;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CheckboxComponent.prototype, "inputClasses", {
        get: /**
         * @return {?}
         */
        function () {
            var _a;
            return _a = {},
                _a['indeterminate'] = this.indeterminate,
                _a;
        },
        enumerable: true,
        configurable: true
    });
    CheckboxComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-checkbox',
                    template: "\n    <div class=\"cui-input-group cui-checkbox\" [ngClass]=\"wrapperClasses\">\n      <input\n        class=\"cui-input cui-checkbox__input\"\n        [ngClass]=\"inputClasses\"\n        type=\"checkbox\"\n        (change)=\"handleChange($event)\"\n        [value]=\"value\"\n        [checked]=\"checked\"\n        [name]=\"name\"\n        [attr.id]=\"htmlId\"\n        [attr.tabindex]=\"tabIndex\"\n        [disabled]=\"disabled\"\n        [required]=\"required\"\n      />\n\n      <label\n        *ngIf=\"label\"\n        class=\"cui-checkbox__label\"\n        [attr.for]=\"htmlId\"\n        (click)=\"onClick($event)\"\n      >\n        <span>{{ label }}</span>\n      </label>\n\n      <ng-content></ng-content>\n    </div>\n  ",
                    providers: [CUSTOM_CHECKBOX_CONTROL_VALUE_ACCESSOR]
                }] }
    ];
    /** @nocollapse */
    CheckboxComponent.ctorParameters = function () { return [
        { type: ChangeDetectorRef }
    ]; };
    CheckboxComponent.propDecorators = {
        value: [{ type: Input }],
        tabIndex: [{ type: Input }],
        formControl: [{ type: Input }],
        class: [{ type: Input }],
        disabled: [{ type: Input }],
        indeterminate: [{ type: Input }],
        label: [{ type: Input }],
        nestedLevel: [{ type: Input }],
        required: [{ type: Input }],
        htmlId: [{ type: Input }],
        selectedItem: [{ type: Input }],
        name: [{ type: Input }],
        checkStatus: [{ type: Input }],
        checkStatusChange: [{ type: Output }]
    };
    return CheckboxComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var LabelComponent = /** @class */ (function () {
    function LabelComponent() {
    }
    /**
     * @return {?}
     */
    LabelComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () { };
    Object.defineProperty(LabelComponent.prototype, "labelClasses", {
        get: /**
         * @return {?}
         */
        function () {
            var _a;
            return _a = {},
                _a['cui-label--' + this.theme] = this.theme,
                _a[this.className] = this.className,
                _a;
        },
        enumerable: true,
        configurable: true
    });
    LabelComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-label',
                    template: "<label [ngClass]=\"labelClasses\" [attr.for]=\"htmlFor\">\n  <span>{{ label }}</span>\n</label>\n"
                }] }
    ];
    /** @nocollapse */
    LabelComponent.ctorParameters = function () { return []; };
    LabelComponent.propDecorators = {
        className: [{ type: Input }],
        theme: [{ type: Input }],
        label: [{ type: Input }],
        htmlFor: [{ type: Input }]
    };
    return LabelComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var LabelModule = /** @class */ (function () {
    function LabelModule() {
    }
    LabelModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [LabelComponent],
                    imports: [CommonModule],
                    exports: [LabelComponent],
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                },] }
    ];
    return LabelModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var InputHelperComponent = /** @class */ (function () {
    function InputHelperComponent() {
    }
    /**
     * @return {?}
     */
    InputHelperComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () { };
    Object.defineProperty(InputHelperComponent.prototype, "inputHelperClasses", {
        get: /**
         * @return {?}
         */
        function () {
            var _a;
            return _a = {},
                _a[this.className] = this.className,
                _a;
        },
        enumerable: true,
        configurable: true
    });
    InputHelperComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-input-helper',
                    template: "\n    <p [ngClass]=\"inputHelperClasses\" class=\"cui-input__help-text\">\n      {{ message }}\n    </p>\n  "
                }] }
    ];
    /** @nocollapse */
    InputHelperComponent.ctorParameters = function () { return []; };
    InputHelperComponent.propDecorators = {
        message: [{ type: Input }],
        className: [{ type: Input }]
    };
    return InputHelperComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var InputHelperModule = /** @class */ (function () {
    function InputHelperModule() {
    }
    InputHelperModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [InputHelperComponent],
                    imports: [CommonModule],
                    exports: [InputHelperComponent],
                },] }
    ];
    return InputHelperModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var CheckboxModule = /** @class */ (function () {
    function CheckboxModule() {
    }
    CheckboxModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [CheckboxComponent],
                    imports: [
                        CommonModule,
                        FormsModule,
                        ReactiveFormsModule,
                        BrowserModule,
                        LabelModule,
                        InputHelperModule,
                    ],
                    exports: [CheckboxComponent],
                    schemas: [CUSTOM_ELEMENTS_SCHEMA]
                },] }
    ];
    return CheckboxModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var cb = (/**
 * @return {?}
 */
function () { });
var InputComponent = /** @class */ (function () {
    function InputComponent(control) {
        this.control = control;
        this.innerValue = '';
        this.onTouchedCallback = cb;
        this.onChangeCallback = cb;
        /**
         * \@option Optional css class string | ''
         */
        this.class = '';
        /**
         * \@option optional button to clear input text
         */
        this.clear = false;
        /**
         * \@option Sets the disabled attribute of the Input | false
         */
        this.disabled = false;
        /**
         * \@option Unique HTML ID used for tying label to HTML input for automated testing
         */
        this.htmlId = '';
        /**
         * \@option Input css class name string
         */
        this.inputClass = '';
        /**
         * \@option Help Text to appear under the input | ''
         */
        this.inputHelpText = '';
        /**
         * \@option Overall input group size | ''
         */
        this.inputSize = '';
        /**
         * \@option Input label text | ''
         */
        this.label = '';
        /**
         * \@option Placeholder text to display when Input is empty | ''
         */
        this.placeholder = '';
        /**
         * \@option Determines if Input can be edited | false
         */
        this.readOnly = false;
        /**
         * \@option Secondary Input label | ''
         */
        this.secondaryLabel = '';
        /**
         * \@option Input type | 'text'
         */
        this.type = 'text';
        /**
         * \@option Sets the attribute name to the input element | ''
         */
        this.name = '';
        /**
         * \@option Optional error messages object with angular validators | {}
         */
        this.errorObj = {};
        /**
         * \@option function when clicked outside of input
         */
        this.handleBlur = new EventEmitter();
        /**
         * \@option function when input is focused
         */
        this.handleFocus = new EventEmitter();
        /**
         * \@option function when key down on input
         */
        this.handleKeyDown = new EventEmitter();
        /**
         * \@option function when mouse down on input
         */
        this.handleMouseDown = new EventEmitter();
        this.determineErrorType = (/**
         * @param {?} array
         * @return {?}
         */
        function (array) {
            return array.reduce((/**
             * @param {?} agg
             * @param {?} e
             * @return {?}
             */
            function (agg, e) {
                return agg === 'error' ? agg : e.type || '';
            }), '');
        });
        this.filterErrorsByType = (/**
         * @param {?} array
         * @param {?} value
         * @return {?}
         */
        function (array, value) {
            return array.reduce((/**
             * @param {?} agg
             * @param {?} e
             * @return {?}
             */
            function (agg, e) { return (e.type === value ? agg.concat(e.error) : agg); }), []);
        });
        if (this.control) {
            this.control.valueAccessor = this;
        }
    }
    Object.defineProperty(InputComponent.prototype, "value", {
        get: /**
         * @return {?}
         */
        function () {
            return this.innerValue;
        },
        set: /**
         * @param {?} v
         * @return {?}
         */
        function (v) {
            if (v !== this.innerValue) {
                this.innerValue = v;
                this.onChangeCallback(v);
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InputComponent.prototype, "hasRequiredError", {
        get: /**
         * @return {?}
         */
        function () {
            return this.control.hasError('required');
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InputComponent.prototype, "errorValid", {
        get: /**
         * @return {?}
         */
        function () {
            // if (this.control.pristine) {
            //   return '';
            // }
            for (var error in this.errorObj) {
                if (this.control.errors[error]) {
                    return this.errorObj[error];
                }
            }
            return '';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InputComponent.prototype, "wrapperClasses", {
        get: /**
         * @return {?}
         */
        function () {
            var _a;
            return _a = {},
                _a[this.inputSize + ' columns'] = this.inputSize,
                _a['read-only'] = this.readOnly,
                _a.disabled = this.disabled,
                _a.error = Object.keys(this.errorObj).length > 0 &&
                    this.control.invalid &&
                    this.control.dirty,
                _a['cui-input-group--' + this.theme] = this.theme,
                _a[this.errorType] = this.errorType,
                _a[this.class] = this.class,
                _a;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InputComponent.prototype, "inputClasses", {
        get: /**
         * @return {?}
         */
        function () {
            var _a;
            return _a = {},
                _a[this.inputClass] = this.inputClass,
                _a['read-only'] = this.readOnly,
                _a.disabled = this.disabled,
                _a.dirty = this.value,
                _a;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @param {?} event
     * @return {?}
     */
    InputComponent.prototype.onBlur = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        this.handleBlur.emit();
        event.stopPropagation();
    };
    /**
     * @param {?} event
     * @return {?}
     */
    InputComponent.prototype.onFocus = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        if (this.disabled) {
            event.stopPropagation();
            return;
        }
        this.handleFocus.emit();
    };
    /**
     * @param {?} event
     * @return {?}
     */
    InputComponent.prototype.onKeyDown = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        this.handleKeyDown.emit();
    };
    /**
     * @param {?} event
     * @return {?}
     */
    InputComponent.prototype.onMouseDown = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        if (this.disabled) {
            event.stopPropagation();
            return;
        }
        this.handleMouseDown.emit();
    };
    /**
     * @param {?} value
     * @return {?}
     */
    InputComponent.prototype.writeValue = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        if (value !== this.innerValue) {
            this.innerValue = value;
        }
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    InputComponent.prototype.registerOnChange = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this.onChangeCallback = fn;
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    InputComponent.prototype.registerOnTouched = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this.onTouchedCallback = fn;
    };
    /**
     * @param {?} changes
     * @return {?}
     */
    InputComponent.prototype.ngOnChanges = /**
     * @param {?} changes
     * @return {?}
     */
    function (changes) {
        if (changes.errorArr) {
            /** @type {?} */
            var changedArr = changes.errorArr.currentValue;
            if (changedArr.length > 0) {
                this.errorType = this.determineErrorType(changedArr);
                this.errors =
                    this.errorType && this.filterErrorsByType(changedArr, this.errorType);
            }
        }
    };
    /**
     * @return {?}
     */
    InputComponent.prototype.handleClear = /**
     * @return {?}
     */
    function () {
        this.value = '';
    };
    InputComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-input',
                    template: "\n    <div class=\"cui-input-group\" [ngClass]=\"wrapperClasses\">\n      <cui-label\n        *ngIf=\"label\"\n        [label]=\"label\"\n        className=\"cui-label\"\n        htmlFor=\"htmlId\"\n      ></cui-label>\n\n      <ng-container *ngIf=\"clear; else normal\">\n        <div class=\"cui-input__icon-container\">\n          <input\n            *ngIf=\"!secondaryLabel\"\n            class=\"cui-input\"\n            [ngClass]=\"inputClasses\"\n            [(ngModel)]=\"value\"\n            [placeholder]=\"placeholder\"\n            [readonly]=\"readOnly\"\n            [attr.disabled]=\"disabled ? '' : null\"\n            [type]=\"type\"\n            (blur)=\"onBlur($event)\"\n            (focus)=\"onFocus($event)\"\n            (keydown)=\"onKeyDown($event)\"\n            (mousedown)=\"onMouseDown($event)\"\n            [name]=\"name\"\n            [attr.id]=\"htmlId\"\n          />\n\n          <button\n            *ngIf=\"clear && value !== ''\"\n            class=\"cui-button cui-button--36 cui-button--icon\"\n            type=\"button\"\n            aria-label=\"clear input\"\n            (click)=\"handleClear()\"\n          >\n            <span class=\"cui-button__children\" style=\"opacity: 1;\">\n              <i\n                class=\"cui-icon icon icon-clear-active_16 cui-input__icon-clear\"\n              ></i>\n            </span>\n          </button>\n        </div>\n      </ng-container>\n\n      <ng-template #normal>\n        <input\n          class=\"cui-input\"\n          [ngClass]=\"inputClasses\"\n          [(ngModel)]=\"value\"\n          [type]=\"type\"\n          [placeholder]=\"placeholder\"\n          [readonly]=\"readOnly\"\n          [attr.disabled]=\"disabled ? '' : null\"\n          (blur)=\"onBlur($event)\"\n          (focus)=\"onFocus($event)\"\n          (keydown)=\"onKeyDown($event)\"\n          (mousedown)=\"onMouseDown($event)\"\n          [name]=\"name\"\n          [attr.id]=\"htmlId\"\n        />\n      </ng-template>\n\n      <!-- Secondary Label -->\n      <div class=\"cui-label__secondary-label-container\" *ngIf=\"secondaryLabel\">\n        <input\n          class=\"cui-input\"\n          [ngClass]=\"inputClasses\"\n          [(ngModel)]=\"value\"\n          [type]=\"type\"\n          [placeholder]=\"placeholder\"\n          [readonly]=\"readOnly\"\n          [attr.disabled]=\"disabled ? '' : null\"\n          (blur)=\"onBlur($event)\"\n          (focus)=\"onFocus($event)\"\n          (keydown)=\"onKeyDown($event)\"\n          (mousedown)=\"onMouseDown($event)\"\n          [name]=\"name\"\n          [attr.id]=\"htmlId\"\n        />\n\n        <label class=\"cui-label__secondary-label\" [attr.for]=\"htmlId\">\n          <span>{{ secondaryLabel }}</span>\n        </label>\n      </div>\n\n      <!-- Error Input -->\n      <ng-container *ngIf=\"errors\">\n        <cui-input-error *ngFor=\"let error of errors\" [error]=\"error\">\n        </cui-input-error>\n      </ng-container>\n\n      <!-- Angular Control Error Validation Message  -->\n      <div class=\"cui-input__messages\" role=\"alert\" *ngIf=\"control.invalid\">\n        <div class=\"message\">{{ errorValid }}</div>\n      </div>\n\n      <!-- Helper Text -->\n      <cui-input-helper *ngIf=\"inputHelpText\" [message]=\"inputHelpText\">\n      </cui-input-helper>\n    </div>\n  "
                }] }
    ];
    /** @nocollapse */
    InputComponent.ctorParameters = function () { return [
        { type: NgControl, decorators: [{ type: Optional }, { type: Self }] }
    ]; };
    InputComponent.propDecorators = {
        class: [{ type: Input }],
        clear: [{ type: Input }],
        disabled: [{ type: Input }],
        htmlId: [{ type: Input }],
        errorArr: [{ type: Input }],
        inputClass: [{ type: Input }],
        inputHelpText: [{ type: Input }],
        inputSize: [{ type: Input }],
        label: [{ type: Input }],
        placeholder: [{ type: Input }],
        readOnly: [{ type: Input }],
        secondaryLabel: [{ type: Input }],
        theme: [{ type: Input }],
        type: [{ type: Input }],
        name: [{ type: Input }],
        errorObj: [{ type: Input }],
        handleBlur: [{ type: Output }],
        handleFocus: [{ type: Output }],
        handleKeyDown: [{ type: Output }],
        handleMouseDown: [{ type: Output }]
    };
    return InputComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var InputErrorComponent = /** @class */ (function () {
    function InputErrorComponent() {
    }
    /**
     * @return {?}
     */
    InputErrorComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () { };
    InputErrorComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-input-error',
                    template: "\n    <div class=\"cui-input__messages\" role=\"alert\">\n      <div class=\"message\">{{ error }}</div>\n    </div>\n  "
                }] }
    ];
    /** @nocollapse */
    InputErrorComponent.ctorParameters = function () { return []; };
    InputErrorComponent.propDecorators = {
        error: [{ type: Input }]
    };
    return InputErrorComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var InputErrorModule = /** @class */ (function () {
    function InputErrorModule() {
    }
    InputErrorModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [InputErrorComponent],
                    imports: [CommonModule],
                    exports: [InputErrorComponent],
                },] }
    ];
    return InputErrorModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var InputModule = /** @class */ (function () {
    function InputModule() {
    }
    InputModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [InputComponent],
                    imports: [
                        CommonModule,
                        BrowserModule,
                        FormsModule,
                        ReactiveFormsModule,
                        LabelModule,
                        InputErrorModule,
                        InputHelperModule,
                        IconModule,
                    ],
                    exports: [InputComponent],
                },] }
    ];
    return InputModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var cb$1 = (/**
 * @return {?}
 */
function () { });
var EditableTextfieldComponent = /** @class */ (function () {
    function EditableTextfieldComponent(control, elementRef) {
        var _this = this;
        this.control = control;
        this.elementRef = elementRef;
        this._alignment = null;
        /**
         * \@option Optional css class name for non-edit text div | null
         */
        this.buttonClass = '';
        /**
         * \@option Optional css class string on div wrapping input | ''
         */
        this.wrapperClass = '';
        /**
         * \@option Sets the disabled attribute of the Input | false
         */
        this.disabled = false;
        /**
         * \@option Unique HTML ID used for tying label to HTML input for automated testing
         */
        this.htmlId = '';
        /**
         * \@option Input css class name string
         */
        this.inputClass = '';
        /**
         * \@option Overall input group size | ''
         */
        this.inputSize = '';
        /**
         * \@option Placeholder text to display when Input is empty | ''
         */
        this.placeholder = '';
        /**
         * \@option Determines if Input can be edited | false
         */
        this.readOnly = false;
        /**
         * \@option Input type | 'text'
         */
        this.type = 'text';
        /**
         * \@option Sets the attribute name to the input element | ''
         */
        this.name = '';
        /**
         * \@option Optional function after done editing
         */
        this.handleDoneEditing = new EventEmitter();
        this.innerValue = '';
        this.isEditing = false;
        this.onTouchedCallback = cb$1;
        this.onChangeCallback = cb$1;
        this.handleClick = (/**
         * @return {?}
         */
        function () {
            if (_this.disabled) {
                return;
            }
            else {
                _this.isEditing = true;
                setTimeout((/**
                 * @return {?}
                 */
                function () { return _this.inputRef.nativeElement.focus(); }), 100);
            }
        });
        if (this.control) {
            this.control.valueAccessor = this;
        }
    }
    Object.defineProperty(EditableTextfieldComponent.prototype, "alignment", {
        /** @option Sets optional button alignment | null */
        set: /**
         * \@option Sets optional button alignment | null
         * @param {?} alignment
         * @return {?}
         */
        function (alignment) {
            if (this._alignment) {
                this.elementRef.nativeElement.classList.remove("cui-editable-textfield--" + this._alignment);
            }
            this.elementRef.nativeElement.classList.add("cui-editable-textfield--" + alignment);
            this._alignment = alignment;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EditableTextfieldComponent.prototype, "value", {
        get: /**
         * @return {?}
         */
        function () {
            return this.innerValue;
        },
        set: /**
         * @param {?} v
         * @return {?}
         */
        function (v) {
            if (v !== this.innerValue) {
                this.innerValue = v;
                this.onChangeCallback(v);
            }
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @param {?} value
     * @return {?}
     */
    EditableTextfieldComponent.prototype.writeValue = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        if (value !== this.innerValue) {
            this.innerValue = value;
        }
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    EditableTextfieldComponent.prototype.registerOnChange = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this.onChangeCallback = fn;
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    EditableTextfieldComponent.prototype.registerOnTouched = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this.onTouchedCallback = fn;
    };
    /**
     * @param {?} e
     * @return {?}
     */
    EditableTextfieldComponent.prototype.onKeyDown = /**
     * @param {?} e
     * @return {?}
     */
    function (e) {
        if (e.keyCode === 27) {
            this.isEditing = false;
        }
        else if (e.keyCode === 13) {
            this.handleEnter(e, e.target.value);
        }
    };
    /**
     * @param {?} e
     * @param {?} value
     * @return {?}
     */
    EditableTextfieldComponent.prototype.handleEnter = /**
     * @param {?} e
     * @param {?} value
     * @return {?}
     */
    function (e, value) {
        this.isEditing = false;
    };
    // keydown on non edit button
    // keydown on non edit button
    /**
     * @param {?} event
     * @return {?}
     */
    EditableTextfieldComponent.prototype.handleKey = 
    // keydown on non edit button
    /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        var _this = this;
        if (!this.disabled) {
            if ((event.keyCode >= 65 && event.keyCode <= 90) ||
                (event.keyCode >= 48 && event.keyCode <= 57) ||
                event.keyCode === 32) {
                this.isEditing = true;
                setTimeout((/**
                 * @return {?}
                 */
                function () { return _this.inputRef.nativeElement.focus(); }), 100);
                this.value = this.value.concat(event.key);
            }
        }
    };
    /**
     * @param {?} e
     * @return {?}
     */
    EditableTextfieldComponent.prototype.onBlur = /**
     * @param {?} e
     * @return {?}
     */
    function (e) {
        this.isEditing = false;
        this.handleDoneEditing.emit(this.value);
        e.stopPropagation();
    };
    /**
     * @param {?} event
     * @return {?}
     */
    EditableTextfieldComponent.prototype.onFocus = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        if (this.disabled) {
            event.stopPropagation();
            return;
        }
        this.isEditing = true;
    };
    Object.defineProperty(EditableTextfieldComponent.prototype, "textWrapperClasses", {
        get: /**
         * @return {?}
         */
        function () {
            var _a;
            return _a = {},
                _a[this.inputSize + ' columns'] = this.inputSize,
                _a[this.wrapperClass] = this.wrapperClass,
                _a;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EditableTextfieldComponent.prototype, "buttonClasses", {
        get: /**
         * @return {?}
         */
        function () {
            var _a;
            return _a = {},
                _a[this.buttonClass] = this.buttonClass,
                _a;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EditableTextfieldComponent.prototype, "inputClasses", {
        get: /**
         * @return {?}
         */
        function () {
            var _a;
            return _a = {},
                _a[this.inputClass] = this.inputClass,
                _a['read-only'] = this.readOnly,
                _a.disabled = this.disabled,
                _a.dirty = this.value,
                _a;
        },
        enumerable: true,
        configurable: true
    });
    EditableTextfieldComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-editable-textfield',
                    template: "\n    <div\n      *ngIf=\"isEditing\"\n      class=\"cui-input-group cui-editable-textfield__editing\"\n      [ngClass]=\"textWrapperClasses\"\n    >\n      <input\n        #inputRef\n        class=\"cui-input\"\n        [ngClass]=\"inputClasses\"\n        [attr.id]=\"htmlId\"\n        [(ngModel)]=\"value\"\n        [placeholder]=\"placeholder\"\n        [readonly]=\"readOnly\"\n        [attr.disabled]=\"disabled ? '' : null\"\n        [type]=\"type\"\n        [name]=\"name\"\n        (blur)=\"onBlur($event)\"\n        (focus)=\"onFocus($event)\"\n        (keydown)=\"onKeyDown($event)\"\n      />\n    </div>\n\n    <!-- Not in Edit Mode -->\n    <div\n      [hidden]=\"isEditing\"\n      class=\"cui-editable-textfield__button\"\n      [ngClass]=\"buttonClasses\"\n      role=\"button\"\n      tabindex=\"0\"\n      (click)=\"handleClick()\"\n      (keydown)=\"handleKey($event)\"\n    >\n      {{ value }}\n    </div>\n  ",
                    host: {
                        class: 'cui-editable-textfield',
                    }
                }] }
    ];
    /** @nocollapse */
    EditableTextfieldComponent.ctorParameters = function () { return [
        { type: NgControl, decorators: [{ type: Optional }, { type: Self }] },
        { type: ElementRef }
    ]; };
    EditableTextfieldComponent.propDecorators = {
        alignment: [{ type: Input }],
        buttonClass: [{ type: Input }],
        wrapperClass: [{ type: Input }],
        disabled: [{ type: Input }],
        htmlId: [{ type: Input }],
        inputClass: [{ type: Input }],
        inputSize: [{ type: Input }],
        placeholder: [{ type: Input }],
        readOnly: [{ type: Input }],
        type: [{ type: Input }],
        name: [{ type: Input }],
        handleDoneEditing: [{ type: Output }],
        inputRef: [{ type: ViewChild, args: ['inputRef',] }]
    };
    return EditableTextfieldComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var EditableTextfieldModule = /** @class */ (function () {
    function EditableTextfieldModule() {
    }
    EditableTextfieldModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [EditableTextfieldComponent],
                    imports: [CommonModule, InputModule, FormsModule, ReactiveFormsModule],
                    exports: [EditableTextfieldComponent],
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                },] }
    ];
    return EditableTextfieldModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var OptionSelectionChange = /** @class */ (function () {
    function OptionSelectionChange(source, isUserInput) {
        if (isUserInput === void 0) { isUserInput = false; }
        this.source = source;
        this.isUserInput = isUserInput;
    }
    return OptionSelectionChange;
}());
var ListItemComponent = /** @class */ (function () {
    function ListItemComponent(el) {
        var _this = this;
        this.el = el;
        this.checkedValues = [''];
        this.isSelectOption = this._hasHostAttributes('cui-select-option');
        /**
         * \@prop Active prop to determine styles | false
         */
        this.active = false;
        /**
         * \@prop selected prop to determine styles | false
         */
        this.selected = false;
        /**
         * \@prop Disabled attribute for ListItem to determine styles | false
         */
        this.disabled = false;
        /**
         * \@prop Sets ListItem to a focus state | false
         */
        this.focus = false;
        /**
         * \@prop Sets ListItem id | null
         */
        this.id = uniqueId$1('cui-list-item-');
        /**
         * \@prop Determines if is part of a multi select | false
         */
        this.isMulti = false;
        /**
         * \@prop Determines if ListItem is clickable | false
         */
        this.isReadOnly = false;
        /**
         * \@prop ListItem text and form value | ''
         */
        this.label = null;
        /**
         * \@prop external link associated input | ''
         */
        this.link = '';
        /**
         * \@prop ListItem ref name | 'navLink'
         */
        this.role = null;
        /**
         * \@prop Controls whether to show separator or not | false
         */
        this.separator = false;
        /**
         * \@prop ListItem Title | ''
         */
        this.title = '';
        /**
         * \@prop ListItem size | ''
         */
        this._type = null;
        this.selectionChange = new EventEmitter();
        this.handleClick = (/**
         * @param {?} event
         * @return {?}
         */
        function (event) {
            if (_this.isReadOnly) {
                event.stopImmediatePropagation();
            }
            else {
                _this._selectViaInteraction();
            }
        });
        this.handleKeydown = (/**
         * @param {?} event
         * @return {?}
         */
        function (event) {
            if ((event.keyCode === ENTER || event.keyCode === SPACE) && !hasModifierKey(event)) {
                _this._selectViaInteraction();
                event.preventDefault();
            }
        });
        this._isTypeOptionValid = (/**
         * @return {?}
         */
        function () { return (['', 'small', 'large', 'xlarge', 'space', 'header', 36, 52, 60].includes(_this._type)); });
    }
    Object.defineProperty(ListItemComponent.prototype, "myRole", {
        get: /**
         * @return {?}
         */
        function () {
            return this.role || this.isSelectOption ? 'option' : 'listItem';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListItemComponent.prototype, "type", {
        set: /**
         * @param {?} type
         * @return {?}
         */
        function (type) {
            if (this._type) {
                this.el.nativeElement.classList.remove("cui-list-item--" + this._type);
            }
            this.el.nativeElement.classList.add("cui-list-item--" + type);
            this._type = type;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListItemComponent.prototype, "myHref", {
        get: /**
         * @return {?}
         */
        function () {
            return (this.link && this._getHostElement().localName === 'a') ? this.link : null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListItemComponent.prototype, "theTitle", {
        get: /**
         * @return {?}
         */
        function () {
            return this.title || this.label;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    ListItemComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        if (this._type && !this._isTypeOptionValid()) {
            throw new Error("cui-list-item: ListItem type option must be one of the following:\n        small, large, xlarge, space, header, 36, 52, 60");
        }
    };
    /**
     * @return {?}
     */
    ListItemComponent.prototype.ngAfterViewInit = /**
     * @return {?}
     */
    function () {
        if (!this.label) {
            throw new Error('cui-list-item: Label property is required');
        }
    };
    /**
     * @return {?}
     */
    ListItemComponent.prototype.select = /**
     * @return {?}
     */
    function () {
        if (!this.selected) {
            this.selected = true;
            this._emitSelectionChangeEvent();
        }
    };
    /** Deselects the option. */
    /**
     * Deselects the option.
     * @return {?}
     */
    ListItemComponent.prototype.deselect = /**
     * Deselects the option.
     * @return {?}
     */
    function () {
        if (this.selected) {
            this.selected = false;
            this._emitSelectionChangeEvent();
        }
    };
    // Highlightable Interface methods
    // Highlightable Interface methods
    /**
     * @return {?}
     */
    ListItemComponent.prototype.setActiveStyles = 
    // Highlightable Interface methods
    /**
     * @return {?}
     */
    function () {
        this.focus = true;
    };
    /**
     * @return {?}
     */
    ListItemComponent.prototype.setInactiveStyles = /**
     * @return {?}
     */
    function () {
        this.focus = false;
    };
    /**
     * @return {?}
     */
    ListItemComponent.prototype.getLabel = /**
     * @return {?}
     */
    function () {
        return this.label;
    };
    /**
     * @return {?}
     */
    ListItemComponent.prototype._selectViaInteraction = /**
     * @return {?}
     */
    function () {
        if (!this.disabled) {
            this.selected = !this.selected;
            this.checkStatus = this.selected;
            this._emitSelectionChangeEvent(true);
        }
    };
    /**
     * @param {?=} isUserInput
     * @return {?}
     */
    ListItemComponent.prototype._emitSelectionChangeEvent = /**
     * @param {?=} isUserInput
     * @return {?}
     */
    function (isUserInput) {
        if (isUserInput === void 0) { isUserInput = false; }
        return this.selectionChange.emit(new OptionSelectionChange(this, isUserInput));
    };
    /**
     * @return {?}
     */
    ListItemComponent.prototype._getHostElement = /**
     * @return {?}
     */
    function () {
        return this.el.nativeElement;
    };
    /** Gets whether the button has one of the given attributes. */
    /**
     * Gets whether the button has one of the given attributes.
     * @param {...?} attributes
     * @return {?}
     */
    ListItemComponent.prototype._hasHostAttributes = /**
     * Gets whether the button has one of the given attributes.
     * @param {...?} attributes
     * @return {?}
     */
    function () {
        var _this = this;
        var attributes = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            attributes[_i] = arguments[_i];
        }
        return attributes.some((/**
         * @param {?} attribute
         * @return {?}
         */
        function (attribute) { return _this._getHostElement().hasAttribute(attribute); }));
    };
    ListItemComponent.decorators = [
        { type: Component, args: [{
                    selector: 'a[cui-list-item], div[cui-list-item], div[cui-select-option]',
                    exportAs: 'cuiListItem',
                    template: "\n      <ng-container *ngIf=\"isSelectOption\">\n\n        <ng-container *ngIf=\"!isMulti\">\n          <cui-list-item-section key=\"child-0\" position='center'>\n              <ng-container *ngIf=\"label; else content\">{{label}}</ng-container>\n              <ng-template #content>\n                  <ng-content></ng-content>\n              </ng-template>\n          </cui-list-item-section>\n          <cui-list-item-section key=\"child-1\" position='right'>\n              <cui-icon *ngIf=\"selected\" name=\"check_20\" color=\"blue\"></cui-icon>\n          </cui-list-item-section>\n        </ng-container>\n\n        <ng-container *ngIf=\"isMulti\">\n          <cui-checkbox\n            name=\"{{id}}-checkbox\"\n            value=\"{{label}}\"\n            label=\"{{label}}\"\n            [(checkStatus)]=\"checkStatus\"\n            htmlId=\"{{id}}-checkbox\">\n          </cui-checkbox>\n        </ng-container>\n\n      </ng-container>\n\n      <ng-container *ngIf=\"!isSelectOption\">\n        <div *ngIf=\"label; else content\">{{label}}</div>\n        <ng-template #content>\n          <ng-content></ng-content>\n        </ng-template>\n      </ng-container>\n  ",
                    host: {
                        'class': 'cui-list-item',
                        '[class.cui-list-item--read-only]': 'isReadOnly',
                        '[class.cui-list-item--separator]': 'separator',
                        '[class.cui-select-option]': 'isSelectOption',
                        '[class.active]': 'active',
                        '[class.focus]': 'focus',
                        '[class.disabled]': 'disabled',
                    }
                }] }
    ];
    /** @nocollapse */
    ListItemComponent.ctorParameters = function () { return [
        { type: ElementRef }
    ]; };
    ListItemComponent.propDecorators = {
        active: [{ type: Input }],
        selected: [{ type: Input }],
        disabled: [{ type: Input }],
        focus: [{ type: Input }],
        id: [{ type: HostBinding, args: ['attr.id',] }, { type: Input }],
        isMulti: [{ type: Input }],
        isReadOnly: [{ type: Input }],
        label: [{ type: Input }],
        link: [{ type: Input }],
        role: [{ type: Input }],
        myRole: [{ type: HostBinding, args: ['attr.role',] }],
        separator: [{ type: Input }],
        title: [{ type: Input }],
        type: [{ type: Input }],
        selectionChange: [{ type: Output }],
        myHref: [{ type: HostBinding, args: ['attr.href',] }],
        theTitle: [{ type: HostBinding, args: ['attr.title',] }],
        handleClick: [{ type: HostListener, args: ['click', ['$event'],] }],
        handleKeydown: [{ type: HostListener, args: ['keydown', ['$event'],] }]
    };
    return ListItemComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ListItemSectionComponent = /** @class */ (function () {
    function ListItemSectionComponent() {
        var _this = this;
        /**
         * \@option position Determine the ListItemSection's position | 'center'
         */
        this.position = 'center';
        /**
         * \@option class Optional css class name | ''
         */
        this.class = '';
        this.isValidPosition = (/**
         * @return {?}
         */
        function () {
            return ['left', 'center', 'right', 'center-align'].includes(_this.position);
        });
    }
    Object.defineProperty(ListItemSectionComponent.prototype, "className", {
        get: /**
         * @return {?}
         */
        function () {
            return ("cui-list-item__" + this.position +
                ("" + ((this.class && " " + this.class) || '')));
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    ListItemSectionComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        if (this.position && !this.isValidPosition()) {
            throw new Error("cui-list-item-section: ListItemSection position option\n        must be one of the following: left, center, right, center-align");
        }
    };
    ListItemSectionComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-list-item-section',
                    template: "\n    <ng-content></ng-content>\n  "
                }] }
    ];
    /** @nocollapse */
    ListItemSectionComponent.ctorParameters = function () { return []; };
    ListItemSectionComponent.propDecorators = {
        position: [{ type: Input }],
        class: [{ type: Input }],
        className: [{ type: HostBinding, args: ['class',] }]
    };
    return ListItemSectionComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ListItemSectionModule = /** @class */ (function () {
    function ListItemSectionModule() {
    }
    ListItemSectionModule.decorators = [
        { type: NgModule, args: [{
                    imports: [CommonModule],
                    declarations: [ListItemSectionComponent],
                    exports: [ListItemSectionComponent],
                    schemas: [CUSTOM_ELEMENTS_SCHEMA]
                },] }
    ];
    return ListItemSectionModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ListItemModule = /** @class */ (function () {
    function ListItemModule() {
    }
    ListItemModule.decorators = [
        { type: NgModule, args: [{
                    imports: [
                        CheckboxModule,
                        CommonModule,
                        IconModule,
                        ListItemSectionModule
                    ],
                    declarations: [ListItemComponent],
                    exports: [ListItemComponent],
                    schemas: [CUSTOM_ELEMENTS_SCHEMA]
                },] }
    ];
    return ListItemModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ListComponent = /** @class */ (function () {
    function ListComponent(cd, _ngZone, el) {
        var _this = this;
        this.cd = cd;
        this._ngZone = _ngZone;
        this.el = el;
        /**
         * \@option Optional ID value of List | null
         */
        this.id = uniqueId$1('cui-list-');
        /**
         * \@option itemRole prop to set child roles | 'listItem'
         */
        this.itemRole = 'listItem';
        /**
         * \@option Callback function invoked by user selecting an interactive item within list | null
         */
        this.select = new EventEmitter();
        /**
         * \@option Sets the ARIA role for the Nav, in the context of a TabContainer | 'list'
         */
        this.role = 'list';
        /**
         * \@option Sets the orientation of the List | 'vertical'
         */
        this._tabType = 'vertical';
        /**
         * \@option Type sets List size | null
         */
        this.type = null;
        /**
         * \@option Wrap prop type to wrap items to next row
         */
        this.wrap = false;
        /**
         * Combined stream of all of the child options' change events.
         */
        this.optionSelectionChanges = (/** @type {?} */ (defer((/**
         * @return {?}
         */
        function () {
            if (_this.listItems) {
                return merge.apply(void 0, __spread(_this.listItems.map((/**
                 * @param {?} option
                 * @return {?}
                 */
                function (option) { return option.selectionChange; }))));
            }
            return _this._ngZone.onStable.asObservable()
                .pipe(take(1), switchMap((/**
             * @return {?}
             */
            function () { return _this.optionSelectionChanges; })));
        }))));
        this._isTabTypeOptionValid = (/**
         * @return {?}
         */
        function () { return (['vertical', 'horizontal'].includes(_this._tabType)); });
        this._isTypeOptionValid = (/**
         * @return {?}
         */
        function () { return (['small', 'large', 'space', 'xlarge'].includes(_this.type)); });
        this.optionSelectionChanges.subscribe((/**
         * @param {?} event
         * @return {?}
         */
        function (event) {
            _this._onSelect(event.source);
        }));
    }
    Object.defineProperty(ListComponent.prototype, "tabType", {
        set: /**
         * @param {?} tabType
         * @return {?}
         */
        function (tabType) {
            if (this._tabType) {
                this.el.nativeElement.classList.remove("cui-list--" + this._tabType);
            }
            this.el.nativeElement.classList.add("cui-list--" + tabType);
            this._tabType = tabType;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    ListComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        if (this._tabType && !this._isTabTypeOptionValid()) {
            throw new Error("cui-list: List tabType option must be one of the following:\n        vertical, horizontal");
        }
        if (this.type && !this._isTypeOptionValid()) {
            throw new Error("cui-list: List type option must be one of the following:\n        small, large, space, xlarge");
        }
    };
    /**
     * @return {?}
     */
    ListComponent.prototype.ngAfterContentInit = /**
     * @return {?}
     */
    function () {
        var _this = this;
        // set up listItems children with props
        this.listItems.forEach((/**
         * @param {?} listItem
         * @return {?}
         */
        function (listItem) {
            if (_this.type) {
                listItem.type = _this.type;
            }
            listItem.role = _this.itemRole;
            listItem.id = uniqueId$1(_this.id + "__list-item-");
        }));
        this.cd.detectChanges();
    };
    /** Invoked when an option is clicked. */
    /**
     * Invoked when an option is clicked.
     * @private
     * @param {?} option
     * @return {?}
     */
    ListComponent.prototype._onSelect = /**
     * Invoked when an option is clicked.
     * @private
     * @param {?} option
     * @return {?}
     */
    function (option) {
        if (this.select) {
            return this.select.emit(option);
        }
    };
    ListComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-list',
                    template: "\n    <ng-content></ng-content>\n  ",
                    host: {
                        'class': 'cui-list',
                        '[class.cui-list--wrap]': 'wrap'
                    }
                }] }
    ];
    /** @nocollapse */
    ListComponent.ctorParameters = function () { return [
        { type: ChangeDetectorRef },
        { type: NgZone },
        { type: ElementRef }
    ]; };
    ListComponent.propDecorators = {
        id: [{ type: HostBinding, args: ['id',] }, { type: Input }],
        itemRole: [{ type: Input }],
        select: [{ type: Output }],
        role: [{ type: HostBinding, args: ['attr.role',] }, { type: Input }],
        tabType: [{ type: Input }],
        type: [{ type: Input }],
        wrap: [{ type: Input }],
        listItems: [{ type: ContentChildren, args: [ListItemComponent,] }]
    };
    return ListComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ListModule = /** @class */ (function () {
    function ListModule() {
    }
    ListModule.decorators = [
        { type: NgModule, args: [{
                    imports: [CommonModule],
                    declarations: [ListComponent],
                    exports: [ListComponent],
                    schemas: [CUSTOM_ELEMENTS_SCHEMA]
                },] }
    ];
    return ListModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ListSeparatorComponent = /** @class */ (function () {
    function ListSeparatorComponent() {
    }
    Object.defineProperty(ListSeparatorComponent.prototype, "_backgroundColor", {
        get: /**
         * @return {?}
         */
        function () {
            return this.backgroundColor;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListSeparatorComponent.prototype, "_lineColor", {
        get: /**
         * @return {?}
         */
        function () {
            return this.lineColor;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListSeparatorComponent.prototype, "_margin", {
        get: /**
         * @return {?}
         */
        function () {
            return this.margin;
        },
        enumerable: true,
        configurable: true
    });
    ListSeparatorComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-list-separator',
                    template: "\n    <span class=\"cui-list-separator__container\">\n      <ng-content *ngIf=\"!text; else textTemplate\"></ng-content>\n    </span>\n\n    <ng-template #textTemplate>\n      <span\n        class=\"cui-list-separator__text\"\n        [ngStyle]=\"{ color: textColor, padding: textPadding }\"\n        >{{ text }}</span\n      >\n    </ng-template>\n  ",
                    host: {
                        class: 'cui-list-separator',
                    }
                }] }
    ];
    ListSeparatorComponent.propDecorators = {
        backgroundColor: [{ type: Input }],
        lineColor: [{ type: Input }],
        margin: [{ type: Input }],
        text: [{ type: Input }],
        textColor: [{ type: Input }],
        textPadding: [{ type: Input }],
        _backgroundColor: [{ type: HostBinding, args: ['style.background-color',] }],
        _lineColor: [{ type: HostBinding, args: ['style.color',] }],
        _margin: [{ type: HostBinding, args: ['style.margin',] }]
    };
    return ListSeparatorComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ListSeparatorModule = /** @class */ (function () {
    function ListSeparatorModule() {
    }
    ListSeparatorModule.decorators = [
        { type: NgModule, args: [{
                    imports: [CommonModule],
                    declarations: [ListSeparatorComponent],
                    exports: [ListSeparatorComponent],
                },] }
    ];
    return ListSeparatorModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ModalBodyComponent = /** @class */ (function () {
    function ModalBodyComponent() {
        /**
         * \@option css class names | '' | ''
         */
        this.class = '';
    }
    Object.defineProperty(ModalBodyComponent.prototype, "className", {
        get: /**
         * @return {?}
         */
        function () {
            return 'cui-modal__body' + ("" + ((this.class && " " + this.class) || '')) + "";
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    ModalBodyComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () { };
    ModalBodyComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-modal-body',
                    template: "\n    <ng-content></ng-content>\n  "
                }] }
    ];
    /** @nocollapse */
    ModalBodyComponent.ctorParameters = function () { return []; };
    ModalBodyComponent.propDecorators = {
        class: [{ type: Input }],
        className: [{ type: HostBinding, args: ['class',] }]
    };
    return ModalBodyComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ModalBodyModule = /** @class */ (function () {
    function ModalBodyModule() {
    }
    ModalBodyModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [ModalBodyComponent],
                    imports: [CommonModule],
                    exports: [ModalBodyComponent],
                },] }
    ];
    return ModalBodyModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ModalFooterComponent = /** @class */ (function () {
    function ModalFooterComponent() {
        /**
         * \@option css class names | '' | ''
         */
        this.class = '';
    }
    Object.defineProperty(ModalFooterComponent.prototype, "className", {
        get: /**
         * @return {?}
         */
        function () {
            return ('cui-modal__footer' + ("" + ((this.class && " " + this.class) || '')) + "");
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    ModalFooterComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () { };
    ModalFooterComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-modal-footer',
                    template: "\n    <ng-content></ng-content>\n  "
                }] }
    ];
    /** @nocollapse */
    ModalFooterComponent.ctorParameters = function () { return []; };
    ModalFooterComponent.propDecorators = {
        class: [{ type: Input }],
        className: [{ type: HostBinding, args: ['class',] }]
    };
    return ModalFooterComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ModalFooterModule = /** @class */ (function () {
    function ModalFooterModule() {
    }
    ModalFooterModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [ModalFooterComponent],
                    imports: [CommonModule],
                    exports: [ModalFooterComponent],
                },] }
    ];
    return ModalFooterModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ModalService = /** @class */ (function () {
    function ModalService() {
        this.isModalOpened = new Subject();
        this.isModalOpened$ = this.isModalOpened.asObservable();
    }
    /**
     * @param {?} isOpen
     * @return {?}
     */
    ModalService.prototype.setModalStatus = /**
     * @param {?} isOpen
     * @return {?}
     */
    function (isOpen) {
        this.isModalOpened.next(isOpen);
    };
    ModalService.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    ModalService.ctorParameters = function () { return []; };
    return ModalService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ModalHeaderComponent = /** @class */ (function () {
    function ModalHeaderComponent(modalService) {
        var _this = this;
        this.modalService = modalService;
        /**
         * \@option Optional css class string | ''
         */
        this.class = '';
        /**
         * ModalHeader label text | ''
         */
        this.headerLabel = '';
        /**
         * Modal message | ''
         */
        this.message = '';
        /**
         * show/hide close button | true
         */
        this.showCloseButton = true;
        this.hideNative = true;
        this.closeModal = (/**
         * @return {?}
         */
        function () {
            _this.modalService.setModalStatus(false);
        });
    }
    Object.defineProperty(ModalHeaderComponent.prototype, "className", {
        get: /**
         * @return {?}
         */
        function () {
            return ('cui-modal__header' + ("" + ((this.class && " " + this.class) || '')) + "");
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    ModalHeaderComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        this.hideNative = this.headerLabel !== '';
    };
    /**
     * @return {?}
     */
    ModalHeaderComponent.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        // prevent memory leak when component destroyed
        if (this.subscription) {
            this.subscription.unsubscribe();
        }
    };
    ModalHeaderComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-modal-header',
                    template: "\n    <ng-content></ng-content>\n    <span *ngIf=\"hideNative\" class=\"cui-modal__title\">{{ headerLabel }}</span>\n    <span *ngIf=\"hideNative && message\" class=\"cui-modal__message\">{{\n      message\n    }}</span>\n    <button\n      *ngIf=\"showCloseButton\"\n      cui-button\n      aria-label=\"Close Modal\"\n      (click)=\"closeModal()\"\n      class=\"cui-close cui-modal__close\"\n    ></button>\n  ",
                    providers: []
                }] }
    ];
    /** @nocollapse */
    ModalHeaderComponent.ctorParameters = function () { return [
        { type: ModalService }
    ]; };
    ModalHeaderComponent.propDecorators = {
        class: [{ type: Input }],
        headerLabel: [{ type: Input }],
        message: [{ type: Input }],
        showCloseButton: [{ type: Input }],
        className: [{ type: HostBinding, args: ['class',] }]
    };
    return ModalHeaderComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ModalHeaderModule = /** @class */ (function () {
    function ModalHeaderModule() {
    }
    ModalHeaderModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [ModalHeaderComponent],
                    imports: [CommonModule],
                    exports: [ModalHeaderComponent],
                },] }
    ];
    return ModalHeaderModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ModalComponent = /** @class */ (function () {
    function ModalComponent(overlay, viewContainerRef, modalService) {
        var _this = this;
        this.overlay = overlay;
        this.viewContainerRef = viewContainerRef;
        this.modalService = modalService;
        /**
         * \@option Determines the visibility and ability to edit the backdrop of the Modal | true
         */
        this.backdrop = true;
        /**
         * \@option To enable/disable clicking on underlay to exit modal | false
         */
        this.backdropClickExit = false;
        /**
         * \@option Show/hide modal
         */
        this.show = false; //
        //
        /**
         * \@option Size of the modal | 'default'
         */
        this.sizeType = 'default';
        /**
         * \@option Callback function invoked when user clicks on cross button or esc key
         */
        this.whenHide = new EventEmitter();
        this.showModal = (/**
         * @return {?}
         */
        function () {
            _this.modalService.setModalStatus(true);
        });
        this.closeModal = (/**
         * @return {?}
         */
        function () {
            _this.modalService.setModalStatus(false);
        });
        this.showContent = (/**
         * @return {?}
         */
        function () {
            if (_this.overlayRef && !_this.overlayRef.hasAttached()) {
                _this.overlayRef.attach(_this.tp);
            }
        });
        this.dismissContent = (/**
         * @return {?}
         */
        function () {
            if (_this.overlayRef && _this.overlayRef.hasAttached()) {
                _this.overlayRef.detach();
                _this.overlayRef.backdropElement.remove();
                _this.whenHide.emit();
            }
        });
    }
    Object.defineProperty(ModalComponent.prototype, "classes", {
        get: /**
         * @return {?}
         */
        function () {
            var _a;
            return _a = {
                    'cui-modal': true
                },
                _a['cui-modal--' + this.sizeType] = true,
                _a.overlayerFix = this.sizeType === 'full',
                _a[this.class] = this.class,
                _a;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    ModalComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        var _this = this;
        /** @type {?} */
        var strategy = this.overlay
            .position()
            .global()
            .centerHorizontally()
            .centerVertically();
        /** @type {?} */
        var config = new OverlayConfig({
            hasBackdrop: true,
            backdropClass: this.backdrop ? 'cui-modal__backdrop' : '',
            positionStrategy: strategy,
        });
        this.overlayRef = this.overlay.create(config);
        this.tp = new TemplatePortal(this.modalContentTemplate, this.viewContainerRef);
        if (this.backdropClickExit) {
            this.overlayRef.backdropClick().subscribe((/**
             * @return {?}
             */
            function () {
                _this.dismissContent();
            }));
        }
        this.modalService.isModalOpened$.subscribe((/**
         * @param {?} isOpen
         * @return {?}
         */
        function (isOpen) {
            if (isOpen) {
                _this.showContent();
            }
            else {
                _this.dismissContent();
            }
        }));
        this.modalService.setModalStatus(this.show);
    };
    ModalComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-modal',
                    template: "\n    <ng-template #overlayGlobalTemplate>\n      <div [ngClass]=\"classes\">\n        <div class=\"cui-modal__content\">\n          <div class=\"cui-modal__flex-container\">\n            <ng-content></ng-content>\n          </div>\n        </div>\n      </div>\n    </ng-template>\n  ",
                    providers: [ModalService],
                    styles: ['.overlayerFix{ top:0; }']
                }] }
    ];
    /** @nocollapse */
    ModalComponent.ctorParameters = function () { return [
        { type: Overlay },
        { type: ViewContainerRef },
        { type: ModalService }
    ]; };
    ModalComponent.propDecorators = {
        backdrop: [{ type: Input }],
        backdropClickExit: [{ type: Input }],
        class: [{ type: Input }],
        show: [{ type: Input }],
        sizeType: [{ type: Input }],
        whenHide: [{ type: Output }],
        modalContentTemplate: [{ type: ViewChild, args: ['overlayGlobalTemplate',] }]
    };
    return ModalComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ModalModule = /** @class */ (function () {
    function ModalModule() {
    }
    ModalModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [ModalComponent],
                    entryComponents: [ModalComponent],
                    imports: [CommonModule, OverlayModule],
                    exports: [ModalComponent],
                },] }
    ];
    return ModalModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// tslint:disable:no-use-before-declare
/** @type {?} */
var CUSTOM_RADIO_VALUE_ACCESSOR = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef((/**
     * @return {?}
     */
    function () { return RadioComponent; })),
    multi: true,
};
// tslint:enable:no-use-before-declare
var RadioComponent = /** @class */ (function () {
    function RadioComponent(cdr) {
        this.cdr = cdr;
        /**
         * \@option Optional CSS class name | ''
         */
        this.class = '';
        /**
         * \@option Sets the attribute disabled to the Radio | false
         */
        this.disabled = false;
        /**
         * \@option Unique HTML ID used for tying label to HTML input for automated testing
         */
        this.htmlId = '';
        /**
         * \@option Radio label text | ''
         */
        this.label = '';
        /**
         * \@option Radio name for group | ''
         */
        this.name = '';
        /**
         * \@option String value that corresponds with Radio button | ''
         */
        this.value = '';
        /**
         * \@option Set the level of nested radio | 0
         */
        this.nestedLevel = 0;
        /**
         * \@option Callback function invoked when user clicks the Radio button | null
         */
        this.click = new EventEmitter();
        this.onChangeCallback = (/**
         * @return {?}
         */
        function () { });
        this.onTouchedCallback = (/**
         * @return {?}
         */
        function () { });
    }
    Object.defineProperty(RadioComponent.prototype, "wrapperClasses", {
        get: /**
         * @return {?}
         */
        function () {
            var _a;
            return _a = {},
                _a['cui-input--nested-' + this.nestedLevel] = this.nestedLevel,
                _a[this.class] = this.class,
                _a;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @param {?} e
     * @return {?}
     */
    RadioComponent.prototype.onToggle = /**
     * @param {?} e
     * @return {?}
     */
    function (e) {
        if (!this.disabled) {
            this.radioViewChild.nativeElement.checked = true;
            this.checked = true;
            this.onChangeCallback(this.value);
            this.click.emit();
        }
    };
    /**
     * @param {?} value
     * @return {?}
     */
    RadioComponent.prototype.writeValue = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        this.checked = value === this.value;
        if (this.radioViewChild.nativeElement) {
            this.radioViewChild.nativeElement.checked = this.checked;
        }
        this.cdr.markForCheck();
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    RadioComponent.prototype.registerOnTouched = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this.onTouchedCallback = fn;
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    RadioComponent.prototype.registerOnChange = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this.onChangeCallback = fn;
    };
    RadioComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-radio',
                    template: "\n    <div class=\"cui-radio-group\">\n      <div class=\"cui-input-group cui-radio\" [ngClass]=\"wrapperClasses\">\n        <input\n          class=\"cui-input cui-radio__input\"\n          type=\"radio\"\n          #radioInput\n          (change)=\"onToggle($event)\"\n          [attr.id]=\"htmlId\"\n          [attr.name]=\"name\"\n          [attr.tabindex]=\"tabIndex\"\n          [attr.value]=\"value\"\n          [checked]=\"checked\"\n          [disabled]=\"disabled\"\n        />\n\n        <label\n          class=\"cui-radio__label\"\n          (click)=\"onToggle($event)\"\n          [attr.for]=\"htmlId\"\n        >\n          <span>{{ label }}</span>\n        </label>\n      </div>\n\n      <ng-content></ng-content>\n    </div>\n  ",
                    providers: [CUSTOM_RADIO_VALUE_ACCESSOR]
                }] }
    ];
    /** @nocollapse */
    RadioComponent.ctorParameters = function () { return [
        { type: ChangeDetectorRef }
    ]; };
    RadioComponent.propDecorators = {
        class: [{ type: Input }],
        disabled: [{ type: Input }],
        htmlId: [{ type: Input }],
        label: [{ type: Input }],
        name: [{ type: Input }],
        tabIndex: [{ type: Input }],
        value: [{ type: Input }],
        nestedLevel: [{ type: Input }],
        click: [{ type: Output }],
        radioViewChild: [{ type: ViewChild, args: ['radioInput',] }]
    };
    return RadioComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var RadioModule = /** @class */ (function () {
    function RadioModule() {
    }
    RadioModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [RadioComponent],
                    imports: [CommonModule, FormsModule, ReactiveFormsModule, BrowserModule],
                    exports: [RadioComponent],
                },] }
    ];
    return RadioModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var cb$2 = (/**
 * @return {?}
 */
function () { });
var SearchInputComponent = /** @class */ (function () {
    function SearchInputComponent(control) {
        this.control = control;
        /**
         * \@option Optional css class string | ''
         */
        this.class = '';
        /**
         * \@option optional button to clear input text
         */
        this.clear = false;
        /**
         * \@option Sets the disabled attribute of the Input | false
         */
        this.disabled = false;
        /**
         * \@option Unique HTML ID used for tying label to HTML input for automated testing
         */
        this.htmlId = '';
        /**
         * \@option Input css class name string
         */
        this.inputClass = '';
        /**
         * \@option Overall input group size | ''
         */
        this.inputSize = '';
        /**
         * \@option Input label text | ''
         */
        this.label = '';
        /**
         * \@option Placeholder text to display when Input is empty | ''
         */
        this.placeholder = '';
        /**
         * \@option Determines if Input can be edited | false
         */
        this.readOnly = false;
        /**
         * \@option Input type | 'text'
         */
        this.type = 'text';
        /**
         * \@option Sets the attribute name to the input element | ''
         */
        this.name = '';
        /**
         * \@option function when clicked outside of input
         */
        this.handleBlur = new EventEmitter();
        /**
         * \@option function when input is focused
         */
        this.handleFocus = new EventEmitter();
        /**
         * \@option function when key down on input
         */
        this.handleKeyDown = new EventEmitter();
        /**
         * \@option function when mouse down on input
         */
        this.handleMouseDown = new EventEmitter();
        this.innerValue = '';
        this.onTouchedCallback = cb$2;
        this.onChangeCallback = cb$2;
        if (this.control) {
            this.control.valueAccessor = this;
        }
    }
    Object.defineProperty(SearchInputComponent.prototype, "value", {
        get: /**
         * @return {?}
         */
        function () {
            return this.innerValue;
        },
        set: /**
         * @param {?} v
         * @return {?}
         */
        function (v) {
            if (v !== this.innerValue) {
                this.innerValue = v;
                this.onChangeCallback(v);
            }
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @param {?} value
     * @return {?}
     */
    SearchInputComponent.prototype.writeValue = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        if (value !== this.innerValue) {
            this.innerValue = value;
        }
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    SearchInputComponent.prototype.registerOnChange = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this.onChangeCallback = fn;
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    SearchInputComponent.prototype.registerOnTouched = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this.onTouchedCallback = fn;
    };
    /**
     * @param {?} event
     * @return {?}
     */
    SearchInputComponent.prototype.onKeyDown = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        this.handleKeyDown.emit();
    };
    /**
     * @param {?} event
     * @return {?}
     */
    SearchInputComponent.prototype.onMouseDown = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        if (this.disabled) {
            event.stopPropagation();
            return;
        }
        this.handleMouseDown.emit();
    };
    /**
     * @param {?} event
     * @return {?}
     */
    SearchInputComponent.prototype.onBlur = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        this.handleBlur.emit();
        event.stopPropagation();
    };
    /**
     * @param {?} event
     * @return {?}
     */
    SearchInputComponent.prototype.onFocus = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        if (this.disabled) {
            event.stopPropagation();
            return;
        }
        this.handleFocus.emit();
    };
    Object.defineProperty(SearchInputComponent.prototype, "wrapperClasses", {
        get: /**
         * @return {?}
         */
        function () {
            var _a;
            return _a = {},
                _a[this.inputSize + ' columns'] = this.inputSize,
                _a['cui-search-input--pill'] = this.type === 'pill',
                _a['read-only'] = this.readOnly,
                _a.disabled = this.disabled,
                _a[this.class] = this.class,
                _a;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SearchInputComponent.prototype, "inputClasses", {
        get: /**
         * @return {?}
         */
        function () {
            var _a;
            return _a = {},
                _a[this.inputClass] = this.inputClass,
                _a['read-only'] = this.readOnly,
                _a.disabled = this.disabled,
                _a.dirty = this.value,
                _a;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SearchInputComponent.prototype, "iconClass", {
        get: /**
         * @return {?}
         */
        function () {
            return {
                'icon-search_16 ': this.type === 'pill',
                'icon-search_20': this.type !== 'pill',
            };
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    SearchInputComponent.prototype.handleClear = /**
     * @return {?}
     */
    function () {
        this.value = '';
    };
    SearchInputComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-search-input',
                    template: "\n    <div class=\"cui-input-group cui-search-input\" [ngClass]=\"wrapperClasses\">\n      <cui-label\n        *ngIf=\"label\"\n        [label]=\"label\"\n        className=\"cui-label\"\n        htmlFor=\"htmlId\"\n      ></cui-label>\n\n      <div class=\"cui-input__icon-container\">\n        <input\n          class=\"cui-input\"\n          [ngClass]=\"inputClasses\"\n          [attr.id]=\"htmlId\"\n          [(ngModel)]=\"value\"\n          [placeholder]=\"placeholder\"\n          [readonly]=\"readOnly\"\n          [attr.disabled]=\"disabled ? '' : null\"\n          [type]=\"type\"\n          [name]=\"name\"\n          (blur)=\"onBlur($event)\"\n          (focus)=\"onFocus($event)\"\n          (keydown)=\"onKeyDown($event)\"\n          (mousedown)=\"onMouseDown($event)\"\n        />\n\n        <i\n          class=\"cui-icon icon cui-search-input__icon\"\n          [ngClass]=\"iconClass\"\n        ></i>\n\n        <button\n          *ngIf=\"clear && value !== ''\"\n          class=\"cui-button cui-button--36 cui-button--icon\"\n          (click)=\"handleClear()\"\n          type=\"button\"\n          aria-label=\"clear input\"\n        >\n          <span class=\"cui-button__children\" style=\"opacity: 1;\">\n            <i\n              class=\"cui-icon icon icon-clear-active_16 cui-input__icon-clear\"\n            ></i>\n          </span>\n        </button>\n      </div>\n    </div>\n  "
                }] }
    ];
    /** @nocollapse */
    SearchInputComponent.ctorParameters = function () { return [
        { type: NgControl, decorators: [{ type: Optional }, { type: Self }] }
    ]; };
    SearchInputComponent.propDecorators = {
        class: [{ type: Input }],
        clear: [{ type: Input }],
        disabled: [{ type: Input }],
        htmlId: [{ type: Input }],
        inputClass: [{ type: Input }],
        inputSize: [{ type: Input }],
        label: [{ type: Input }],
        placeholder: [{ type: Input }],
        readOnly: [{ type: Input }],
        theme: [{ type: Input }],
        type: [{ type: Input }],
        name: [{ type: Input }],
        handleBlur: [{ type: Output }],
        handleFocus: [{ type: Output }],
        handleKeyDown: [{ type: Output }],
        handleMouseDown: [{ type: Output }]
    };
    return SearchInputComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var SearchInputModule = /** @class */ (function () {
    function SearchInputModule() {
    }
    SearchInputModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [SearchInputComponent],
                    imports: [
                        FormsModule,
                        ReactiveFormsModule,
                        CommonModule,
                        IconModule,
                        InputModule,
                        LabelModule,
                    ],
                    exports: [SearchInputComponent],
                },] }
    ];
    return SearchInputModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// tslint:disable:no-use-before-declare
/** @type {?} */
var CUI_SLIDER_VALUE_ACCESSOR = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef((/**
     * @return {?}
     */
    function () { return SliderComponent; })),
    multi: true,
};
var SliderComponent = /** @class */ (function () {
    function SliderComponent(changeDetectorRef) {
        this.changeDetectorRef = changeDetectorRef;
        /**
         * \@option Determines if minimum pointer can cross over maximum pointer | false
         */
        this.canCross = false;
        /**
         * \@option Set the attribute disabled to Slider | false
         */
        this.disabled = false;
        /**
         * \@option Set the initial minimum value | 0
         */
        this.min = 0;
        /**
         * \@option Set visual step measurement | 1
         */
        this.step = 1;
        /**
         * \@option Set increment of x-axis labels | 0
         */
        this.tick = 0;
        /**
         * Event emitted when change a pointer position
         */
        this.change = new EventEmitter();
        this._value = 0;
    }
    Object.defineProperty(SliderComponent.prototype, "value", {
        /** @option Set either maximum pointer value or a combination of high and low pointer values | 0 */
        get: /**
         * \@option Set either maximum pointer value or a combination of high and low pointer values | 0
         * @return {?}
         */
        function () {
            return this._value;
        },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) {
            if (this._value !== value) {
                this._value = value;
                this.sliderLow =
                    (isObject(this.value) && ((/** @type {?} */ (this.value))).low) || this.min;
                this.sliderHigh =
                    (isObject(this.value) && ((/** @type {?} */ (this.value))).high) ||
                        ((/** @type {?} */ (this.value)));
                this.returnCurrentValues();
                this.changeDetectorRef.markForCheck();
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SliderComponent.prototype, "sliderLow", {
        get: /**
         * @return {?}
         */
        function () {
            return this._sliderLow;
        },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) {
            this._sliderLow = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SliderComponent.prototype, "sliderHigh", {
        get: /**
         * @return {?}
         */
        function () {
            return this._sliderHigh;
        },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) {
            this._sliderHigh = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SliderComponent.prototype, "sliderLowPosition", {
        get: /**
         * @return {?}
         */
        function () {
            return ((this.sliderLow - this.min) / (this.max - this.min)) * 100;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SliderComponent.prototype, "sliderHighPosition", {
        get: /**
         * @return {?}
         */
        function () {
            return ((this.sliderHigh - this.min) / (this.max - this.min)) * 100;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @private
     * @param {?} _
     * @return {?}
     */
    SliderComponent.prototype._change = /**
     * @private
     * @param {?} _
     * @return {?}
     */
    function (_) { };
    /**
     * @return {?}
     */
    SliderComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        if (!this.max) {
            throw new Error('Initial maximum(max) value is required');
        }
        this.isObject = isObject;
        this.scale = [this.min, this.max];
        this.getScale(this.min, this.max, this.tick);
        this.getSelectionWidth();
    };
    /**
     * @return {?}
     */
    SliderComponent.prototype.ngAfterViewInit = /**
     * @return {?}
     */
    function () {
        this.setScalePos(this.scale, this.min, this.max);
    };
    /**
     * @param {?} value
     * @return {?}
     */
    SliderComponent.prototype.writeValue = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        if (value !== null) {
            this.value = value;
        }
    };
    /**
     * @param {?} disabled
     * @return {?}
     */
    SliderComponent.prototype.setDisabledState = /**
     * @param {?} disabled
     * @return {?}
     */
    function (disabled) {
        this.disabled = disabled;
        this.changeDetectorRef.markForCheck();
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    SliderComponent.prototype.registerOnChange = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this._change = fn;
    };
    /**
     * @param {?} _
     * @return {?}
     */
    SliderComponent.prototype.registerOnTouched = /**
     * @param {?} _
     * @return {?}
     */
    function (_) { };
    /**
     * @param {?} scale
     * @param {?} min
     * @param {?} max
     * @return {?}
     */
    SliderComponent.prototype.setScalePos = /**
     * @param {?} scale
     * @param {?} min
     * @param {?} max
     * @return {?}
     */
    function (scale, min, max) {
        this.ticksContainer.nativeElement
            .querySelectorAll('span')
            .forEach((/**
         * @param {?} child
         * @param {?} idx
         * @return {?}
         */
        function (child, idx) {
            /** @type {?} */
            var leftValue = ((scale[idx] - min) / max) * 100 + "%";
            child.style.left = "calc(" + leftValue + " - " + child.offsetWidth / 2 + "px)";
        }));
    };
    /**
     * @param {?=} low
     * @param {?=} high
     * @param {?=} tick
     * @return {?}
     */
    SliderComponent.prototype.getScale = /**
     * @param {?=} low
     * @param {?=} high
     * @param {?=} tick
     * @return {?}
     */
    function (low, high, tick) {
        if (low === void 0) { low = 0; }
        if (this.tick > 0) {
            /** @type {?} */
            var value = high;
            /** @type {?} */
            var ticksArray = [high];
            while (value > 0 && value - tick >= low) {
                value -= tick;
                ticksArray.unshift(Math.abs(Math.round(value / tick) * tick));
            }
            this.scale = ticksArray;
        }
    };
    /**
     * @return {?}
     */
    SliderComponent.prototype.getSliderWidth = /**
     * @return {?}
     */
    function () {
        return this.sliderBar.nativeElement.getBoundingClientRect().width;
    };
    /**
     * @return {?}
     */
    SliderComponent.prototype.getStepWidth = /**
     * @return {?}
     */
    function () {
        /** @type {?} */
        var maxValue = this.max - this.min;
        /** @type {?} */
        var maxWidth = this.getSliderWidth();
        return (this.step / maxValue) * maxWidth;
    };
    /**
     * @param {?} position
     * @return {?}
     */
    SliderComponent.prototype.getSteps = /**
     * @param {?} position
     * @return {?}
     */
    function (position) {
        if (position.isKeyBoard) {
            return 1;
        }
        /** @type {?} */
        var diff = position.direction === 1
            ? position.to - position.from
            : position.from - position.to;
        if (diff < 0) {
            return 0;
        }
        /** @type {?} */
        var steps = diff / this.getStepWidth();
        return steps - Math.floor(steps) >= 0.5
            ? Math.ceil(steps)
            : Math.floor(steps);
    };
    /**
     * @param {?} pointerKey
     * @param {?} direction
     * @return {?}
     */
    SliderComponent.prototype.getLimit = /**
     * @param {?} pointerKey
     * @param {?} direction
     * @return {?}
     */
    function (pointerKey, direction) {
        if (pointerKey === 'sliderLow') {
            return this.canCross
                ? direction === 1
                    ? this.max
                    : this.min
                : direction === 1
                    ? this.sliderHigh
                    : this.min;
        }
        else if (pointerKey === 'sliderHigh') {
            return this.canCross
                ? direction === 1
                    ? this.max
                    : this.min
                : direction === 1
                    ? this.max
                    : this.sliderLow;
        }
    };
    /**
     * @return {?}
     */
    SliderComponent.prototype.returnCurrentValues = /**
     * @return {?}
     */
    function () {
        this.getSelectionWidth();
        /** @type {?} */
        var value = isObject(this.value)
            ? {
                low: Math.min(this.sliderHigh, this.sliderLow),
                high: Math.max(this.sliderHigh, this.sliderLow),
            }
            : this.sliderHigh;
        this._change(value);
        this.change.emit(value);
    };
    /**
     * @param {?} key
     * @param {?} pixelMove
     * @param {?} limit
     * @return {?}
     */
    SliderComponent.prototype.moveForward = /**
     * @param {?} key
     * @param {?} pixelMove
     * @param {?} limit
     * @return {?}
     */
    function (key, pixelMove, limit) {
        /** @type {?} */
        var newPosition = this[key] + pixelMove <= limit ? this[key] + pixelMove : limit;
        this[key] = newPosition;
        this.returnCurrentValues();
    };
    /**
     * @param {?} key
     * @param {?} pixelMove
     * @param {?} limit
     * @return {?}
     */
    SliderComponent.prototype.moveBack = /**
     * @param {?} key
     * @param {?} pixelMove
     * @param {?} limit
     * @return {?}
     */
    function (key, pixelMove, limit) {
        /** @type {?} */
        var newPosition = this[key] - pixelMove >= limit ? this[key] - pixelMove : limit;
        this[key] = newPosition;
        this.returnCurrentValues();
    };
    /**
     * @param {?} key
     * @param {?} position
     * @return {?}
     */
    SliderComponent.prototype.onSliderMove = /**
     * @param {?} key
     * @param {?} position
     * @return {?}
     */
    function (key, position) {
        if (this.disabled) {
            return;
        }
        /** @type {?} */
        var limit = this.getLimit(key, position.direction);
        /** @type {?} */
        var pixelMove = this.getSteps(position) * this.step;
        position.direction === 1
            ? this.moveForward(key, pixelMove, limit)
            : this.moveBack(key, pixelMove, limit);
    };
    /**
     * @return {?}
     */
    SliderComponent.prototype.getSelectionWidth = /**
     * @return {?}
     */
    function () {
        /** @type {?} */
        var baseValue = Number.isInteger(this.sliderLow)
            ? this.sliderLow
            : this.min;
        this.selectionLeft =
            ((Math.min(baseValue, this.sliderHigh) - this.min) / this.max) * 100;
        this.selectionWidth =
            (Math.abs(this.sliderHigh - baseValue) / this.max) * 100;
    };
    SliderComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-slider',
                    template: "\n    <span #sliderBar class=\"cui-slider__bar\"></span>\n    <span\n      class=\"cui-slider__selection\"\n      [ngStyle]=\"{ 'left.%': selectionLeft, 'width.%': selectionWidth }\"\n    ></span>\n    <cui-slider-pointer\n      *ngIf=\"isObject(value)\"\n      [position]=\"sliderLowPosition\"\n      (move)=\"onSliderMove('sliderLow', $event)\"\n    ></cui-slider-pointer>\n    <cui-slider-pointer\n      [position]=\"sliderHighPosition\"\n      (move)=\"onSliderMove('sliderHigh', $event)\"\n    ></cui-slider-pointer>\n    <div #ticksContainer>\n      <span *ngFor=\"let tickValue of scale\" class=\"cui-slider__hashlabel\">{{\n        (translateFn && translateFn(tickValue)) || tickValue\n      }}</span>\n    </div>\n  ",
                    host: {
                        class: 'cui-slider',
                        '[class.cui-slider--disabled]': 'disabled',
                    },
                    providers: [CUI_SLIDER_VALUE_ACCESSOR]
                }] }
    ];
    /** @nocollapse */
    SliderComponent.ctorParameters = function () { return [
        { type: ChangeDetectorRef }
    ]; };
    SliderComponent.propDecorators = {
        canCross: [{ type: Input }],
        disabled: [{ type: Input }],
        max: [{ type: Input }],
        min: [{ type: Input }],
        step: [{ type: Input }],
        tick: [{ type: Input }],
        translateFn: [{ type: Input }],
        value: [{ type: Input }],
        change: [{ type: Output }],
        sliderBar: [{ type: ViewChild, args: ['sliderBar',] }],
        ticksContainer: [{ type: ViewChild, args: ['ticksContainer',] }]
    };
    return SliderComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var SliderPointerComponent = /** @class */ (function () {
    function SliderPointerComponent(elementRef, _document) {
        var _this = this;
        this.elementRef = elementRef;
        /**
         * \@option Set Slider Pointer's position | 0
         */
        this.position = 0;
        /**
         * Event emitted when user moves the Slider Pointer
         */
        this.move = new EventEmitter();
        this.onMousemove = (/**
         * @param {?} event
         * @return {?}
         */
        function (event) {
            /** @type {?} */
            var xPos = event.clientX;
            /** @type {?} */
            var direction = _this.getDirections(xPos);
            _this.move.emit({
                from: _this.elementRef.nativeElement.getBoundingClientRect().left,
                to: xPos,
                direction: direction,
                isKeyBoard: false,
            });
        });
        this.onMouseup = (/**
         * @param {?} _
         * @return {?}
         */
        function (_) {
            _this._document.removeEventListener('mousemove', _this.onMousemove);
            _this._document.removeEventListener('mouseup', _this.onMouseup);
        });
        this._document = _document;
    }
    Object.defineProperty(SliderPointerComponent.prototype, "left", {
        get: /**
         * @return {?}
         */
        function () {
            return this.position;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @param {?} currentPos
     * @return {?}
     */
    SliderPointerComponent.prototype.getDirections = /**
     * @param {?} currentPos
     * @return {?}
     */
    function (currentPos) {
        if (currentPos > this._previousPosition) {
            return 1;
        }
        return -1;
    };
    /**
     * @param {?} event
     * @return {?}
     */
    SliderPointerComponent.prototype.onKeydown = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        switch (event.code) {
            case 'ArrowUp':
            case 'ArrowRight':
                this.move.emit({
                    direction: 1,
                    isKeyBoard: true,
                });
                event.preventDefault();
                break;
            case 'ArrowDown':
            case 'ArrowLeft':
                this.move.emit({
                    direction: -1,
                    isKeyBoard: true,
                });
                event.preventDefault();
                break;
        }
    };
    /**
     * @param {?} event
     * @return {?}
     */
    SliderPointerComponent.prototype.onMousedown = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        this._document.addEventListener('mousemove', this.onMousemove);
        this._document.addEventListener('mouseup', this.onMouseup);
        this._previousPosition = event.clientX;
    };
    /**
     * @param {?} event
     * @return {?}
     */
    SliderPointerComponent.prototype.onTouchstart = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        this._previousPosition = event.touches[0].clientX;
    };
    /**
     * @param {?} event
     * @return {?}
     */
    SliderPointerComponent.prototype.onTouchmove = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        /** @type {?} */
        var xPos = event.touches[0].clientX;
        /** @type {?} */
        var direction = this.getDirections(xPos);
        this.move.emit({
            from: this.elementRef.nativeElement.getBoundingClientRect().left,
            to: xPos,
            direction: direction,
            isKeyBoard: false,
        });
    };
    SliderPointerComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-slider-pointer',
                    template: "",
                    host: {
                        class: 'cui-slider__pointer',
                        role: 'button',
                        tabindex: '0',
                        '(keydown)': 'onKeydown($event)',
                        '(mousedown)': 'onMousedown($event)',
                        '(touchstart)': 'onTouchstart($event)',
                        '(touchmove)': 'onTouchmove($event)',
                    }
                }] }
    ];
    /** @nocollapse */
    SliderPointerComponent.ctorParameters = function () { return [
        { type: ElementRef },
        { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] }] }
    ]; };
    SliderPointerComponent.propDecorators = {
        position: [{ type: Input }],
        move: [{ type: Output }],
        left: [{ type: HostBinding, args: ['style.left.%',] }]
    };
    return SliderPointerComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var SliderModule = /** @class */ (function () {
    function SliderModule() {
    }
    SliderModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [SliderComponent, SliderPointerComponent],
                    imports: [CommonModule],
                    exports: [SliderComponent],
                },] }
    ];
    return SliderModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Change event object that is emitted when the select value has changed.
 */
var  /**
 * Change event object that is emitted when the select value has changed.
 */
SelectChange = /** @class */ (function () {
    function SelectChange(source, value) {
        this.source = source;
        this.value = value;
    }
    return SelectChange;
}());
var SelectComponent = /** @class */ (function () {
    function SelectComponent(_ngZone) {
        var _this = this;
        this._ngZone = _ngZone;
        this._destroy = new Subject();
        this.overlayOpen = false;
        this.anchorWidth = null;
        /**
         * \@prop Optional CSS button class name | ''
         */
        this.buttonClass = '';
        /**
         * \@prop Set the default selected option | ''
         */
        this.defaultValue = null;
        /**
         * \@prop Disable the Select Component | false
         */
        this.disabled = false;
        /**
         * \@prop Set ID for Select Component | null
         */
        this.id = uniqueId('cui-select-');
        /**
         * \@prop Optional prop to know if user is able to select multiple options | false
         */
        this.isMulti = false;
        this.select = new EventEmitter();
        /**
         * Event emitted when the selected value has been changed by the user.
         */
        this.selectionChange = new EventEmitter();
        /**
         * facilitates the two-way binding for the `ngModel` input.
         */
        this.ngModelChange = new EventEmitter();
        /**
         * Combined stream of all of the child options' change events.
         */
        this.optionSelectionChanges = (/** @type {?} */ (defer((/**
         * @return {?}
         */
        function () {
            if (_this.selectOptions) {
                return merge.apply(void 0, __spread(_this.selectOptions.map((/**
                 * @param {?} option
                 * @return {?}
                 */
                function (option) { return option.selectionChange; }))));
            }
            return _this._ngZone.onStable
                .asObservable()
                .pipe(take(1), switchMap((/**
             * @return {?}
             */
            function () { return _this.optionSelectionChanges; })));
        }))));
        /**
         * `View -> model callback called when value changes`
         */
        this._onChange = (/**
         * @return {?}
         */
        function () { });
        /**
         * `View -> model callback called when select has been touched`
         */
        this._onTouched = (/**
         * @return {?}
         */
        function () { });
        /**
         * Comparison function to specify which option is displayed. Defaults to object equality.
         */
        this._compareWith = (/**
         * @param {?} o1
         * @param {?} o2
         * @return {?}
         */
        function (o1, o2) { return o1 === o2; });
        /**
         * Handles keyboard events when the selected is open.
         */
        this._handleOpenKeydown = (/**
         * @param {?} event
         * @return {?}
         */
        function (event) {
            if (event.defaultPrevented || !_this.overlayOpen) {
                return;
            }
            /** @type {?} */
            var keyCode = event.keyCode;
            /** @type {?} */
            var isArrowKey = keyCode === DOWN_ARROW || keyCode === UP_ARROW;
            /** @type {?} */
            var manager = _this._keyManager;
            if (keyCode === HOME || keyCode === END) {
                event.preventDefault();
                keyCode === HOME ? manager.setFirstItemActive() : manager.setLastItemActive();
            }
            else if (isArrowKey && event.altKey) {
                // Close the select on ALT + arrow key to match the native <select>
                event.preventDefault();
                _this.close();
            }
            else if ((keyCode === ENTER || keyCode === SPACE) && manager.activeItem &&
                !hasModifierKey(event)) {
                if (!_this.isMulti) {
                    _this.close();
                }
                event.preventDefault();
                manager.activeItem._selectViaInteraction();
            }
            else if (_this.isMulti && keyCode === A && event.ctrlKey) {
                event.preventDefault();
                /** @type {?} */
                var hasDeselectedOptions_1 = _this.selectOptions.some((/**
                 * @param {?} opt
                 * @return {?}
                 */
                function (opt) { return !opt.disabled && !opt.selected; }));
                _this.selectOptions.forEach((/**
                 * @param {?} option
                 * @return {?}
                 */
                function (option) {
                    if (!option.disabled) {
                        hasDeselectedOptions_1 ? option.select() : option.deselect();
                    }
                }));
            }
            else {
                /** @type {?} */
                var previouslyFocusedIndex = manager.activeItemIndex;
                manager.onKeydown(event);
                if (_this.isMulti && isArrowKey && event.shiftKey && manager.activeItem &&
                    manager.activeItemIndex !== previouslyFocusedIndex) {
                    manager.activeItem._selectViaInteraction();
                }
            }
        });
        /**
         * Toggles the overlay panel open or closed.
         */
        this.toggle = (/**
         * @return {?}
         */
        function () {
            _this.overlayOpen ? _this.close() : _this.open();
            _this.focus();
        });
        this.open = (/**
         * @return {?}
         */
        function () {
            if (_this.disabled || !_this.selectOptions || !_this.selectOptions.length || _this.overlayOpen) {
                return;
            }
            _this.overlayOpen = true;
        });
        this.close = (/**
         * @return {?}
         */
        function () {
            if (_this.overlayOpen) {
                _this.overlayOpen = false;
            }
        });
        /**
         * Focuses the origin button of the select component.
         */
        this.focus = (/**
         * @return {?}
         */
        function () {
            _this.originButton.el.nativeElement.focus();
        });
        this.currentValue = (/**
         * @return {?}
         */
        function () {
            if (!_this._selectionModel) {
                return;
            }
            /** @type {?} */
            var selected = _this._selectionModel.selected;
            if (!_this.isMulti && selected.length) {
                return selected[0].label;
            }
            if (selected.length === 1) {
                return selected.length + " Item Selected";
            }
            else if (selected.length) {
                return selected.length + " Items Selected";
            }
        });
        /**
         * Sets the select's ngModelValue. Part of the ControlValueAccessor interface
         */
        this.writeValue = (/**
         * @param {?} value
         * @return {?}
         */
        function (value) {
            if (_this.selectOptions) {
                _this._setSelectionByValue(value);
            }
        });
        this.registerOnChange = (/**
         * @param {?} fn
         * @return {?}
         */
        function (fn) {
            _this._onChange = fn;
        });
        this.registerOnTouched = (/**
         * @param {?} fn
         * @return {?}
         */
        function (fn) {
            _this._onTouched = fn;
        });
        this.setDisabledState = (/**
         * @param {?} isDisabled
         * @return {?}
         */
        function (isDisabled) {
            _this.disabled = isDisabled;
        });
        /**
         * Sets the selected option based on a value.
         */
        this._setSelectionByValue = (/**
         * @param {?} value
         * @return {?}
         */
        function (value) {
            if (_this.isMulti && value) {
                if (!Array.isArray(value)) {
                    throw Error('Value must be an array in multi-selection mode.');
                }
                _this._selectionModel.clear();
                value.forEach((/**
                 * @param {?} currentValue
                 * @return {?}
                 */
                function (currentValue) { return _this._selectValue(currentValue); }));
            }
            else {
                _this._selectionModel.clear();
                /** @type {?} */
                var matchedOption = _this._selectValue(value);
                if (matchedOption) {
                    _this._keyManager.setActiveItem(matchedOption);
                }
            }
        });
        this._selectValue = (/**
         * @param {?} value
         * @return {?}
         */
        function (value) {
            /** @type {?} */
            var matchedOption = _this.selectOptions.find((/**
             * @param {?} option
             * @return {?}
             */
            function (option) {
                try {
                    return option.label != null && _this._compareWith(option.label, value);
                }
                catch (error) {
                    console.warn(error);
                }
                return false;
            }));
            if (matchedOption) {
                _this._selectionModel.select(matchedOption);
            }
            return matchedOption;
        });
        this._propagateChanges = (/**
         * @param {?=} fallbackValue
         * @return {?}
         */
        function (fallbackValue) {
            /** @type {?} */
            var valueToEmit = null;
            if (_this.isMulti) {
                valueToEmit = ((/** @type {?} */ (_this.selected))).map((/**
                 * @param {?} option
                 * @return {?}
                 */
                function (option) { return option.label; }));
            }
            else {
                valueToEmit = _this.selected ? ((/** @type {?} */ (_this.selected))).label : fallbackValue;
            }
            _this._ngModelValue = valueToEmit;
            _this.ngModelChange.emit(valueToEmit);
            _this._onChange(valueToEmit);
            _this.selectionChange.emit(new SelectChange(_this, valueToEmit));
        });
        /**
         * Sets up a key manager to listen to keyboard events on the overlay panel.
         */
        this._initKeyManager = (/**
         * @return {?}
         */
        function () {
            _this._keyManager = new ActiveDescendantKeyManager(_this.selectOptions)
                .withWrap()
                .withTypeAhead();
            _this._keyManager.tabOut.pipe(takeUntil(_this._destroy)).subscribe((/**
             * @return {?}
             */
            function () {
                // Restore focus to the trigger before closing
                _this.focus();
                _this.close();
            }));
            _this._keyManager.change.pipe(takeUntil(_this._destroy)).subscribe((/**
             * @return {?}
             */
            function () {
                if (!_this.overlayOpen && !_this.isMulti && _this._keyManager.activeItem) {
                    _this._keyManager.activeItem._selectViaInteraction();
                }
            }));
        });
        /**
         * Drops current option subscriptions and IDs and resets from scratch.
         */
        this._resetOptions = (/**
         * @return {?}
         */
        function () {
            /** @type {?} */
            var changedOrDestroyed = merge(_this.selectOptions.changes, _this._destroy);
            _this.optionSelectionChanges.pipe(takeUntil(changedOrDestroyed)).subscribe((/**
             * @param {?} event
             * @return {?}
             */
            function (event) {
                _this._onSelect(event.source, event.isUserInput);
                if (event.isUserInput && !_this.isMulti && _this.overlayOpen) {
                    _this.close();
                    _this.focus();
                }
            }));
        });
        /**
         * Invoked when an option is clicked.
         */
        this._onSelect = (/**
         * @param {?} option
         * @param {?} isUserInput
         * @return {?}
         */
        function (option, isUserInput) {
            /** @type {?} */
            var wasSelected = _this._selectionModel.isSelected(option);
            if (option.label == null && !_this.isMulti) {
                option.deselect();
                _this._selectionModel.clear();
                _this._propagateChanges(option.label);
            }
            else {
                option.selected ? _this._selectionModel.select(option) : _this._selectionModel.deselect(option);
                if (isUserInput) {
                    _this._keyManager.setActiveItem(option);
                }
                if (_this.isMulti) {
                    if (isUserInput) {
                        // if clicked with mouse, focus on origin button
                        _this.focus();
                    }
                }
            }
            if (wasSelected !== _this._selectionModel.isSelected(option)) {
                _this._propagateChanges();
            }
        });
        this._setAnchorWidth = (/**
         * @param {?} elementAnchor
         * @return {?}
         */
        function (elementAnchor) {
            /** @type {?} */
            var anchor = elementAnchor && elementAnchor.getBoundingClientRect();
            _this.anchorWidth = anchor.width;
        });
        this.optionSelectionChanges.subscribe((/**
         * @param {?} event
         * @return {?}
         */
        function (event) {
            _this._onSelect(event.source, event.isUserInput);
        }));
    }
    Object.defineProperty(SelectComponent.prototype, "ngModel", {
        /** @option Callback function invoked by user selecting an interactive option within list | null */
        /** ngModel of Select Control */
        get: /** @option Callback function invoked by user selecting an interactive option within list | null */
        /**
         * ngModel of Select Control
         * @return {?}
         */
        function () { return this._ngModelValue; },
        set: /**
         * @param {?} newValue
         * @return {?}
         */
        function (newValue) {
            if (newValue !== this._ngModelValue) {
                this.writeValue(newValue);
                this._ngModelValue = newValue;
            }
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    SelectComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        this._selectionModel = new SelectionModel(this.isMulti);
    };
    /**
     * @return {?}
     */
    SelectComponent.prototype.ngAfterContentInit = /**
     * @return {?}
     */
    function () {
        var _this = this;
        this._initKeyManager();
        this._selectionModel.changed.pipe(takeUntil(this._destroy)).subscribe((/**
         * @param {?} event
         * @return {?}
         */
        function (event) {
            event.added.forEach((/**
             * @param {?} option
             * @return {?}
             */
            function (option) { return option.select(); }));
            event.removed.forEach((/**
             * @param {?} option
             * @return {?}
             */
            function (option) { return option.deselect(); }));
        }));
        this.selectOptions.changes.pipe(startWith(null), takeUntil(this._destroy)).subscribe((/**
         * @return {?}
         */
        function () {
            _this._resetOptions();
        }));
        // set up children with props
        this.selectOptions.forEach((/**
         * @param {?} option
         * @return {?}
         */
        function (option) {
            option.isMulti = _this.isMulti;
            _this._resetOptions();
        }));
    };
    /**
     * @return {?}
     */
    SelectComponent.prototype.ngAfterContentChecked = /**
     * @return {?}
     */
    function () {
        this._setAnchorWidth(this.originButton.el.nativeElement);
    };
    Object.defineProperty(SelectComponent.prototype, "buttonClasses", {
        get: /**
         * @return {?}
         */
        function () {
            return 'cui-button--input' +
                ("" + ((this.buttonClass && " " + this.buttonClass) || '')) +
                ("" + ((this.disabled && " disabled") || ''));
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SelectComponent.prototype, "selected", {
        /** The currently selected option. */
        get: /**
         * The currently selected option.
         * @return {?}
         */
        function () {
            return this.isMulti ? this._selectionModel.selected : this._selectionModel.selected[0];
        },
        enumerable: true,
        configurable: true
    });
    SelectComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-select',
                    template: "\n    <button cui-button\n        cdkOverlayOrigin\n        #trigger=\"cdkOverlayOrigin\"\n        [attr.name]='id'\n        aria-label=\"select button\"\n        [id]='id'\n        (click)='toggle()'\n        [class]=\"buttonClasses\"\n        type=\"button\"\n        >\n          <div class='cui-select__label' id=\"{{id}}__label\">\n              {{currentValue() || defaultValue}}\n              <cui-icon name=\"arrow-down_16\"></cui-icon>\n          </div>\n    </button>\n\n    <ng-template\n      cdkConnectedOverlay\n      [cdkConnectedOverlayOrigin]=\"trigger\"\n      [cdkConnectedOverlayOpen]=\"overlayOpen\"\n      [cdkConnectedOverlayWidth]=\"this.anchorWidth\"\n      [cdkConnectedOverlayOffsetY]=\"6\"\n      [cdkConnectedOverlayPanelClass]=\"'cui-event-overlay__children'\"\n      [cdkConnectedOverlayHasBackdrop]=\"true\"\n      [cdkConnectedOverlayBackdropClass]=\"'cdk-overlay-transparent-backdrop'\"\n      (backdropClick)=\"close()\"\n      (detach)=\"close()\">\n      <cui-list\n        #panel\n        role='listbox'>\n        <ng-content></ng-content>\n      </cui-list>\n    </ng-template>\n  ",
                    host: {
                        class: 'cui-input-group cui-select',
                        '[class.disabled]': 'disabled'
                    }
                }] }
    ];
    /** @nocollapse */
    SelectComponent.ctorParameters = function () { return [
        { type: NgZone }
    ]; };
    SelectComponent.propDecorators = {
        buttonClass: [{ type: Input }],
        defaultValue: [{ type: Input }],
        disabled: [{ type: HostBinding, args: ['attr.disabled',] }, { type: Input }],
        id: [{ type: Input }],
        isMulti: [{ type: Input }],
        ngModel: [{ type: Input }],
        select: [{ type: Output }],
        selectionChange: [{ type: Output }],
        ngModelChange: [{ type: Output }],
        originButton: [{ type: ViewChild, args: [ButtonComponent,] }],
        panel: [{ type: ViewChild, args: ['panel',] }],
        selectOptions: [{ type: ContentChildren, args: [ListItemComponent,] }],
        _handleOpenKeydown: [{ type: HostListener, args: ['keydown', ['$event'],] }]
    };
    return SelectComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var SelectModule = /** @class */ (function () {
    function SelectModule() {
    }
    SelectModule.decorators = [
        { type: NgModule, args: [{
                    imports: [
                        CommonModule,
                        ButtonModule,
                        IconModule,
                        PortalModule,
                        ListModule,
                        ListItemModule,
                        OverlayModule,
                        A11yModule,
                    ],
                    declarations: [
                        SelectComponent,
                    ],
                    exports: [SelectComponent],
                    schemas: [CUSTOM_ELEMENTS_SCHEMA]
                },] }
    ];
    return SelectModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var SpinnerComponent = /** @class */ (function () {
    function SpinnerComponent() {
        var _this = this;
        this.color = 'black';
        this.size = 36;
        this.showCheck = false;
        this.showPercentage = false;
        this.checkPercentage = (/**
         * @param {?} percent
         * @return {?}
         */
        function (percent) {
            if (!isNaN(percent)) {
                _this.isPercentage = true;
                _this.percentage = Math.round(percent);
            }
            if (Number(_this.size) !== 36 && _this.showPercentage) {
                console.warn('Spinner: Percentage will not be shown for sizes smaller than 36');
            }
        });
        this.sizeCheck = (/**
         * @param {?} size
         * @return {?}
         */
        function (size) {
            /** @type {?} */
            var sizes = [16, 20, 28, 36];
            if (sizes.includes(Number(size))) {
                return true;
            }
            console.warn('Only sizes 16, 20, 28, 36 are acceptable for spinner');
            return false;
        });
    }
    /**
     * @return {?}
     */
    SpinnerComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        this.checkPercentage(this.percentage);
    };
    Object.defineProperty(SpinnerComponent.prototype, "defaultClasses", {
        get: /**
         * @return {?}
         */
        function () {
            var _a;
            return _a = {},
                _a['cui-spinner--' + this.size] = this.sizeCheck(this.size),
                _a['cui-spinner--36'] = Number(this.size) === 36,
                _a['cui-spinner--' + this.color] = this.color,
                _a[this.className] = this.className,
                _a;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SpinnerComponent.prototype, "percentageClasses", {
        get: /**
         * @return {?}
         */
        function () {
            var _a;
            return _a = {},
                _a['cui-spinner-progress--' + this.size] = this.sizeCheck(this.size),
                _a['cui-spinner-progress__percentage-' + this.percentage] = this.percentage,
                _a['cui-spinner-progress--' + this.color] = this.color,
                _a[this.className] = this.className,
                _a;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SpinnerComponent.prototype, "checkMarkClasses", {
        get: /**
         * @return {?}
         */
        function () {
            var _a;
            return _a = {},
                _a['cui-spinner-progress__inset-circle--check'] = this.sizeCheck(this.size) && this.showCheck,
                _a['cui-spinner-progress__inset-circle--check' + this.percentage] = this.showCheck && Number(this.percentage) === 100,
                _a;
        },
        enumerable: true,
        configurable: true
    });
    SpinnerComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-spinner',
                    template: "<div\n  *ngIf=\"isPercentage; else normal\"\n  [ngClass]=\"percentageClasses\"\n  class=\"cui-spinner-progress\"\n>\n  <div class=\"cui-spinner-progress__circle\">\n    <div class=\"cui-spinner-progress__mask cui-spinner-progress__full\">\n      <div class=\"cui-spinner-progress__fill\"></div>\n    </div>\n    <div class=\"cui-spinner-progress__mask cui-spinner-progress__half\">\n      <div class=\"cui-spinner-progress__fill\"></div>\n      <div class=\"cui-spinner-progress__fill cui-spinner-progress__fix\"></div>\n    </div>\n    <div\n      [ngClass]=\"checkMarkClasses\"\n      class=\"cui-spinner-progress__inset-circle\"\n    >\n      <div\n        *ngIf=\"size === 36 && showPercentage && !showCheck\"\n        class=\"cui-spinner-progress__percentage\"\n      >\n        {{ percentage | number: '1.0-0' }}\n      </div>\n    </div>\n  </div>\n</div>\n\n<ng-template #normal>\n  <i [ngClass]=\"defaultClasses\" class=\"cui-spinner\"></i>\n</ng-template>\n"
                }] }
    ];
    /** @nocollapse */
    SpinnerComponent.ctorParameters = function () { return []; };
    SpinnerComponent.propDecorators = {
        className: [{ type: Input }],
        color: [{ type: Input }],
        size: [{ type: Input }],
        percentage: [{ type: Input }],
        showCheck: [{ type: Input }],
        showPercentage: [{ type: Input }]
    };
    return SpinnerComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var SpinnerModule = /** @class */ (function () {
    function SpinnerModule() {
    }
    SpinnerModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [SpinnerComponent],
                    imports: [CommonModule],
                    exports: [SpinnerComponent],
                },] }
    ];
    return SpinnerModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var TabsService = /** @class */ (function () {
    function TabsService() {
        var _this = this;
        this.Pane = 0;
        this.Tabs = 0;
        this.currentTabValue = 0;
        this.currentFocusValue = 0;
        this.TabDic = {};
        this.TabDisables = {};
        this.current = new Subject();
        this.current$ = this.current.asObservable();
        this.focusIndex = new Subject();
        this.focusIndex$ = this.focusIndex.asObservable();
        this.tabSize = new Subject();
        this.tabSize$ = this.tabSize.asObservable();
        this.paneSize = new Subject();
        this.paneSize$ = this.paneSize.asObservable();
        this.registerPane = (/**
         * @return {?}
         */
        function () {
            /** @type {?} */
            var index = _this.Pane + 1;
            _this.paneSize.next(index);
            return index;
        });
        this.registerTab = (/**
         * @return {?}
         */
        function () {
            /** @type {?} */
            var index = _this.Tabs + 1;
            _this.tabSize.next(index);
            return index;
        });
        this.registerDisable = (/**
         * @param {?} index
         * @param {?} disable
         * @return {?}
         */
        function (index, disable) {
            _this.TabDisables[index] = disable;
        });
        this.registerKey = (/**
         * @param {?} key
         * @param {?} value
         * @return {?}
         */
        function (key, value) {
            key = key.toLowerCase();
            if (_this.TabDic[key] === undefined) {
                _this.TabDic[key] = [];
            }
            _this.TabDic[key].push(value);
        });
        this.select = (/**
         * @param {?} index
         * @param {?=} ifFocusSelect
         * @return {?}
         */
        function (index, ifFocusSelect) {
            if (_this.TabDisables[index] === false || ifFocusSelect) {
                _this.setFocus(index);
                _this.current.next(index);
            }
        });
        this.selectFocus = (/**
         * @param {?} index
         * @return {?}
         */
        function (index) {
            if (_this.currentFocusValue !== _this.currentTabValue) {
                _this.current.next(_this.currentFocusValue);
            }
        });
        this.setFocus = (/**
         * @param {?} index
         * @return {?}
         */
        function (index) {
            _this.focusIndex.next(index);
        });
        this.setFocusPre = (/**
         * @return {?}
         */
        function () {
            _this.setFocus(Math.max(0, _this.currentFocusValue - 1));
        });
        this.setFocusNext = (/**
         * @return {?}
         */
        function () {
            _this.setFocus(Math.min(_this.Tabs, _this.currentFocusValue + 1));
        });
        this.setFocusLast = (/**
         * @return {?}
         */
        function () {
            _this.setFocus(_this.Tabs - 1);
        });
        this.setFocusByFirstCharacter = (/**
         * @param {?} key
         * @return {?}
         */
        function (key) {
            key = key.toLowerCase();
            /** @type {?} */
            var arr = _this.TabDic[key];
            /** @type {?} */
            var index;
            /** @type {?} */
            var length;
            /** @type {?} */
            var tabIndex = _this.currentFocusValue;
            if (arr !== undefined) {
                length = arr.length;
                index = arr.indexOf(_this.currentFocusValue);
                if (index !== -1 && length > 1 && index + 1 < length) {
                    tabIndex = arr[index + 1];
                }
                else {
                    tabIndex = arr[0];
                }
            }
            if (tabIndex !== _this.currentFocusValue) {
                _this.setFocus(tabIndex);
                return true;
            }
            return false;
        });
        this.current$.subscribe((/**
         * @param {?} index
         * @return {?}
         */
        function (index) {
            _this.currentTabValue = index;
        }));
        this.focusIndex$.subscribe((/**
         * @param {?} index
         * @return {?}
         */
        function (index) {
            _this.currentFocusValue = index;
        }));
        this.tabSize$.subscribe((/**
         * @param {?} n
         * @return {?}
         */
        function (n) {
            _this.Tabs = n;
        }));
        this.paneSize$.subscribe((/**
         * @param {?} n
         * @return {?}
         */
        function (n) {
            _this.Pane = n;
        }));
    }
    TabsService.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    TabsService.ctorParameters = function () { return []; };
    return TabsService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var TabsComponent = /** @class */ (function () {
    function TabsComponent(tabsService) {
        var _this = this;
        this.tabsService = tabsService;
        /**
         * \@option Optional CSS class name | ''
         */
        this.className = '';
        /**
         * \@option Set the index of the focused Tab | 0
         */
        this.focus = 0;
        /**
         * \@option Determines if the Tabs are in a justified layout | false
         */
        this.justified = false;
        /**
         * \@option Type of Tabs | 'pills'
         */
        this.tabType = 'pills';
        /**
         * \@option Callback function invoked when user selects a tab | null
         */
        this.whenSelect = new EventEmitter();
        this.tabsService.current$.subscribe((/**
         * @param {?} currentIndex
         * @return {?}
         */
        function (currentIndex) {
            _this.whenSelect.emit(currentIndex);
        }));
    }
    Object.defineProperty(TabsComponent.prototype, "classes", {
        get: /**
         * @return {?}
         */
        function () {
            return ('cui-tab' +
                ("" + ((this.tabType && " cui-tab--" + this.tabType) || '')) +
                ("" + ((this.justified && ' cui-tab--justified') || '')) +
                ("" + ((this.className && " " + this.className) || '')) +
                "");
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    TabsComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        this.tabsService.select(this.focus, true);
    };
    TabsComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-tabs',
                    template: "\n    <ng-content></ng-content>\n  ",
                    providers: [TabsService]
                }] }
    ];
    /** @nocollapse */
    TabsComponent.ctorParameters = function () { return [
        { type: TabsService }
    ]; };
    TabsComponent.propDecorators = {
        className: [{ type: Input }],
        focus: [{ type: Input }],
        justified: [{ type: Input }],
        tabType: [{ type: Input }],
        whenSelect: [{ type: Output }],
        classes: [{ type: HostBinding, args: ['class',] }]
    };
    return TabsComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var TabsModule = /** @class */ (function () {
    function TabsModule() {
    }
    TabsModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [TabsComponent],
                    imports: [CommonModule],
                    exports: [TabsComponent],
                },] }
    ];
    return TabsModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var TabComponent = /** @class */ (function () {
    function TabComponent(tabsService, el) {
        var _this = this;
        this.tabsService = tabsService;
        this.el = el;
        /**
         * \@option Optional CSS class name
         */
        this.className = '';
        /**
         * \@option Sets the attribute disabled to the Tab | false
         */
        this.disabled = false;
        /**
         * \@option Tab's anchor role type | 'tab'
         */
        this.role = 'tab';
        /**
         * \@option Callback function invoked when user click a tab | null
         */
        this.whenPress = new EventEmitter();
        /**
         * \@option Callback function invoked when user press a key | null
         */
        this.whenKeyDown = new EventEmitter();
        this.attrRole = 'presentation';
        this.tabIndex = tabsService.registerTab();
        this.tabSize = this.tabIndex;
        this.tabsService.current$.subscribe((/**
         * @param {?} index
         * @return {?}
         */
        function (index) {
            _this.ifCurrent = _this.tabIndex === index;
        }));
        this.tabsService.focusIndex$.subscribe((/**
         * @param {?} index
         * @return {?}
         */
        function (index) {
            _this.ifFocus = _this.tabIndex === index;
            if (_this.ifFocus) {
                _this.el.nativeElement.getElementsByTagName('A')[0].focus();
            }
        }));
        this.tabsService.tabSize$.subscribe((/**
         * @param {?} n
         * @return {?}
         */
        function (n) {
            _this.tabSize = n;
        }));
    }
    Object.defineProperty(TabComponent.prototype, "classes", {
        /*
        @HostBinding('attr.tabIndex') get attrTabIndex():number {
          return this.tabIndex;
        };
        @HostBinding('attr.aria-posinset') ariaPosinset = this.tabIndex;
        @HostBinding('attr.aria-setsize') ariaSetsize = this.tabSize;
        @HostBinding('attr.aria-selected') ariaSelected = this.ifCurrent;
        @HostBinding('attr.aria-disabled') ariaDisabled = this.disabled;
        */
        get: /*
          @HostBinding('attr.tabIndex') get attrTabIndex():number {
            return this.tabIndex;
          };
          @HostBinding('attr.aria-posinset') ariaPosinset = this.tabIndex;
          @HostBinding('attr.aria-setsize') ariaSetsize = this.tabSize;
          @HostBinding('attr.aria-selected') ariaSelected = this.ifCurrent;
          @HostBinding('attr.aria-disabled') ariaDisabled = this.disabled;
          */
        /**
         * @return {?}
         */
        function () {
            return ('cui-tab__item' +
                ("" + ((this.className && " " + this.className) || '')) +
                ("" + ((this.ifCurrent && ' active') || '')) +
                ("" + ((this.disabled && ' disabled') || '')) +
                "");
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    TabComponent.prototype.select = /**
     * @return {?}
     */
    function () {
        this.tabsService.select(this.tabIndex);
    };
    /**
     * @return {?}
     */
    TabComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        this.tabsService.registerDisable(this.tabIndex, this.disabled);
        /** @type {?} */
        var innerText = this.el.nativeElement.innerText;
        if (innerText) {
            this.tabsService.registerKey(innerText[0], this.tabIndex);
        }
    };
    TabComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-tab',
                    template: "\n    <a\n      href=\"javascript:void(0)\"\n      (click)=\"whenPress.emit()\"\n      (keydown)=\"whenKeyDown.emit()\"\n      [attr.role]=\"role\"\n      [attr.tabindex]=\"ifFocus ? 0 : -1\"\n      [attr.aria-posinset]=\"tabIndex\"\n      [attr.aria-setsize]=\"tabSize\"\n      [attr.aria-selected]=\"ifCurrent\"\n      [attr.aria-disabled]=\"disabled\"\n    >\n      <ng-content></ng-content>\n    </a>\n  "
                }] }
    ];
    /** @nocollapse */
    TabComponent.ctorParameters = function () { return [
        { type: TabsService },
        { type: ElementRef }
    ]; };
    TabComponent.propDecorators = {
        className: [{ type: Input }],
        disabled: [{ type: Input }],
        role: [{ type: Input }],
        whenPress: [{ type: Output }],
        whenKeyDown: [{ type: Output }],
        attrRole: [{ type: HostBinding, args: ['attr.role',] }],
        classes: [{ type: HostBinding, args: ['class',] }],
        select: [{ type: HostListener, args: ['click',] }]
    };
    return TabComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var TabModule = /** @class */ (function () {
    function TabModule() {
    }
    TabModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [TabComponent],
                    imports: [CommonModule],
                    exports: [TabComponent],
                },] }
    ];
    return TabModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var TabContentComponent = /** @class */ (function () {
    function TabContentComponent() {
    }
    Object.defineProperty(TabContentComponent.prototype, "className", {
        get: /**
         * @return {?}
         */
        function () {
            return 'cui-tab__content';
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    TabContentComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () { };
    TabContentComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-tab-content',
                    template: "\n    <ng-content></ng-content>\n  "
                }] }
    ];
    /** @nocollapse */
    TabContentComponent.ctorParameters = function () { return []; };
    TabContentComponent.propDecorators = {
        className: [{ type: HostBinding, args: ['class',] }]
    };
    return TabContentComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var TabContentModule = /** @class */ (function () {
    function TabContentModule() {
    }
    TabContentModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [TabContentComponent],
                    imports: [CommonModule],
                    exports: [TabContentComponent],
                },] }
    ];
    return TabContentModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var TabHeaderComponent = /** @class */ (function () {
    function TabHeaderComponent() {
        /**
         * \@option Subheader text | ''
         */
        this.subHeading = '';
    }
    /**
     * @return {?}
     */
    TabHeaderComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () { };
    TabHeaderComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-tab-header',
                    template: "\n    {{ heading }}\n    <div *ngIf=\"subHeading\">{{ subHeading }}</div>\n  "
                }] }
    ];
    /** @nocollapse */
    TabHeaderComponent.ctorParameters = function () { return []; };
    TabHeaderComponent.propDecorators = {
        heading: [{ type: Input }],
        subHeading: [{ type: Input }]
    };
    return TabHeaderComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var TabHeaderModule = /** @class */ (function () {
    function TabHeaderModule() {
    }
    TabHeaderModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [TabHeaderComponent],
                    imports: [CommonModule],
                    exports: [TabHeaderComponent],
                },] }
    ];
    return TabHeaderModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var TabListComponent = /** @class */ (function () {
    function TabListComponent(tabsService) {
        var _this = this;
        this.tabsService = tabsService;
        /**
         * \@option Tab's anchor role type | 'tablist'
         */
        this.role = 'tablist';
        this.regIsCharacter = /\S/;
        this.isPrintableCharacter = (/**
         * @param {?} str
         * @return {?}
         */
        function (str) {
            return str.length === 1 && str.match(_this.regIsCharacter);
        });
        this.onPress = (/**
         * @param {?} e
         * @return {?}
         */
        function (e) {
            /** @type {?} */
            var key = e.keyCode;
            /** @type {?} */
            var tgt = e.target;
            /** @type {?} */
            var flag = false;
            /** @type {?} */
            var clickEvent;
            switch (key) {
                case 32:
                case 13:
                    try {
                        clickEvent = new MouseEvent('click', {
                            view: window,
                            bubbles: true,
                            cancelable: true,
                        });
                    }
                    catch (err) {
                        if (document.createEvent) {
                            clickEvent = document.createEvent('MouseEvents');
                            clickEvent.initEvent('click', true, true);
                        }
                    }
                    tgt.dispatchEvent(clickEvent);
                    flag = true;
                    break;
                case 38:
                case 37:
                    _this.tabsService.setFocusPre();
                    flag = true;
                    break;
                case 39:
                case 40:
                    _this.tabsService.setFocusNext();
                    flag = true;
                    break;
                case 33:
                case 36:
                    _this.tabsService.setFocus(0);
                    flag = true;
                    break;
                case 34:
                case 35:
                    _this.tabsService.setFocusLast();
                    flag = true;
                    break;
                default:
                    if (_this.isPrintableCharacter(e.key)) {
                        flag = _this.tabsService.setFocusByFirstCharacter(e.key);
                    }
                    break;
            }
            if (flag) {
                e.stopPropagation();
                e.preventDefault();
            }
        });
    }
    Object.defineProperty(TabListComponent.prototype, "className", {
        get: /**
         * @return {?}
         */
        function () {
            return 'cui-tab__list';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TabListComponent.prototype, "roleName", {
        get: /**
         * @return {?}
         */
        function () {
            return this.role;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @param {?} $event
     * @return {?}
     */
    TabListComponent.prototype.select = /**
     * @param {?} $event
     * @return {?}
     */
    function ($event) {
        this.onPress($event);
    };
    /**
     * @return {?}
     */
    TabListComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () { };
    TabListComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-tab-list',
                    template: "\n    <ng-content></ng-content>\n  "
                }] }
    ];
    /** @nocollapse */
    TabListComponent.ctorParameters = function () { return [
        { type: TabsService }
    ]; };
    TabListComponent.propDecorators = {
        role: [{ type: Input }],
        className: [{ type: HostBinding, args: ['class',] }],
        roleName: [{ type: HostBinding, args: ['attr.role',] }],
        select: [{ type: HostListener, args: ['keyup', ['$event'],] }]
    };
    return TabListComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var TabListModule = /** @class */ (function () {
    function TabListModule() {
    }
    TabListModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [TabListComponent],
                    imports: [CommonModule],
                    exports: [TabListComponent],
                },] }
    ];
    return TabListModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var TabPaneComponent = /** @class */ (function () {
    function TabPaneComponent(tabsService) {
        var _this = this;
        this.tabsService = tabsService;
        this.tabIndex = tabsService.registerPane();
        this.tabsService.current$.subscribe((/**
         * @param {?} currentIndex
         * @return {?}
         */
        function (currentIndex) {
            _this.ifCurrent = _this.tabIndex === currentIndex;
        }));
    }
    Object.defineProperty(TabPaneComponent.prototype, "className", {
        get: /**
         * @return {?}
         */
        function () {
            return 'cui-tab__pane' + ("" + ((this.ifCurrent && ' active') || '')) + "";
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    TabPaneComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () { };
    TabPaneComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-tab-pane',
                    template: "\n    <div class=\"cui-tab__content\">\n      <ng-content></ng-content>\n    </div>\n  "
                }] }
    ];
    /** @nocollapse */
    TabPaneComponent.ctorParameters = function () { return [
        { type: TabsService }
    ]; };
    TabPaneComponent.propDecorators = {
        className: [{ type: HostBinding, args: ['class',] }]
    };
    return TabPaneComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var TabPaneModule = /** @class */ (function () {
    function TabPaneModule() {
    }
    TabPaneModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [TabPaneComponent],
                    imports: [CommonModule],
                    exports: [TabPaneComponent],
                },] }
    ];
    return TabPaneModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// tslint:disable:no-use-before-declare
/** @type {?} */
var CUSTOM_TOGGLE_SWITCH_VALUE_ACCESSOR = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef((/**
     * @return {?}
     */
    function () { return ToggleSwitchComponent; })),
    multi: true,
};
// tslint:enable:no-use-before-declare
var ToggleSwitchComponent = /** @class */ (function () {
    function ToggleSwitchComponent(cdr) {
        this.cdr = cdr;
        /**
         * \@option Optional CSS class name | ''
         */
        this.class = '';
        /**
         * \@option Sets the attribute disabled to the ToggleSwitch | false
         */
        this.disabled = false;
        /**
         * \@option Unique HTML ID used for tying label to HTML input for automated testing
         */
        this.htmlId = '';
        /**
         * \@option ToggleSwitch label text | ''
         */
        this.label = '';
        /**
         * \@option ToggleSwitch name for group | ''
         */
        this.name = '';
        /**
         * \@option String value that corresponds with ToggleSwitch button | ''
         */
        this.value = '';
        /**
         * \@option Callback function invoked when user clicks the ToggleSwitch button | null
         */
        this.change = new EventEmitter();
        // pass in boolean to ngModel for filled.
        this.checked = false;
        this.onSwitchChange = (/**
         * @return {?}
         */
        function () { });
        this.onSwitchTouch = (/**
         * @return {?}
         */
        function () { });
    }
    /**
     * @param {?} checked
     * @return {?}
     */
    ToggleSwitchComponent.prototype.writeValue = /**
     * @param {?} checked
     * @return {?}
     */
    function (checked) {
        this.checked = checked;
        this.cdr.markForCheck();
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    ToggleSwitchComponent.prototype.registerOnChange = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this.onSwitchTouch = fn;
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    ToggleSwitchComponent.prototype.registerOnTouched = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this.onSwitchChange = fn;
    };
    /**
     * @param {?} event
     * @return {?}
     */
    ToggleSwitchComponent.prototype.onSwitch = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        if (!this.disabled) {
            /** @type {?} */
            var isChecked = ((/** @type {?} */ (event.target))).checked;
            this.updateSwitch(event, isChecked);
        }
    };
    /**
     * @param {?} event
     * @param {?} bool
     * @return {?}
     */
    ToggleSwitchComponent.prototype.updateSwitch = /**
     * @param {?} event
     * @param {?} bool
     * @return {?}
     */
    function (event, bool) {
        this.checked = bool;
        this.onSwitchChange(this.checked);
        this.change.emit({ switchEvent: event, checked: this.checked });
    };
    ToggleSwitchComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-toggle-switch',
                    template: "\n    <input\n      class=\"cui-input cui-toggle-switch__input\"\n      type=\"checkbox\"\n      (change)=\"onSwitch($event)\"\n      [attr.id]=\"htmlId\"\n      [attr.name]=\"name\"\n      [attr.tabindex]=\"tabIndex\"\n      [checked]=\"checked\"\n      [disabled]=\"disabled\"\n    />\n    <label class=\"cui-toggle-switch__label\" [attr.for]=\"htmlId\">\n      <span class=\"cui-toggle-switch__label__container\"></span>\n      <span class=\"cui-toggle-switch__label__text\">{{ label }}</span>\n    </label>\n  ",
                    host: {
                        class: 'cui-input-group cui-toggle-switch',
                    },
                    providers: [CUSTOM_TOGGLE_SWITCH_VALUE_ACCESSOR]
                }] }
    ];
    /** @nocollapse */
    ToggleSwitchComponent.ctorParameters = function () { return [
        { type: ChangeDetectorRef }
    ]; };
    ToggleSwitchComponent.propDecorators = {
        class: [{ type: Input }],
        disabled: [{ type: Input }],
        htmlId: [{ type: Input }],
        label: [{ type: Input }],
        name: [{ type: Input }],
        tabIndex: [{ type: Input }],
        value: [{ type: Input }],
        change: [{ type: Output }]
    };
    return ToggleSwitchComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ToggleSwitchModule = /** @class */ (function () {
    function ToggleSwitchModule() {
    }
    ToggleSwitchModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [ToggleSwitchComponent],
                    imports: [CommonModule, FormsModule, ReactiveFormsModule],
                    exports: [ToggleSwitchComponent],
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                },] }
    ];
    return ToggleSwitchModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var TopbarComponent = /** @class */ (function () {
    function TopbarComponent() {
    }
    TopbarComponent.decorators = [
        { type: Component, args: [{
                    selector: 'header[cui-top-bar]',
                    template: "\n    <div class=\"cui-top-bar__container\">\n      <div class=\"cui-top-bar__brand\">\n        <ng-content select=\"brand\"></ng-content>\n      </div>\n      <ng-content></ng-content>\n    </div>\n  ",
                    host: {
                        'attr.role': 'navigation',
                        class: 'cui-top-bar',
                        '[class.cui-top-bar--fixed]': 'fixed',
                        '[class.cui-top-bar--blue]': 'color === "blue"',
                        '[class.cui-top-bar--dark]': 'color === "dark"',
                        '[class.cui-top-bar--light]': 'color === "light"',
                    }
                }] }
    ];
    TopbarComponent.propDecorators = {
        color: [{ type: Input }],
        fixed: [{ type: Input }]
    };
    return TopbarComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var TopbarBrandComponent = /** @class */ (function () {
    function TopbarBrandComponent() {
        /**
         * \@option Icon class name | 'icon-cisco-logo'
         */
        this.icon = 'icon-cisco-logo';
    }
    TopbarBrandComponent.decorators = [
        { type: Component, args: [{
                    selector: '[cui-top-bar-brand]',
                    exportAs: 'cuiTopbarBrand',
                    template: "\n    <div class=\"cui-brand__logo\">\n      <img *ngIf=\"image\" src=\"{{ image }}\" alt=\"{{ alt }}\" />\n      <i *ngIf=\"!image\" [ngClass]=\"['icon', icon]\"></i>\n    </div>\n    <div class=\"cui-brand__title\">{{ title }}</div>\n  ",
                    host: {
                        class: 'cui-brand',
                    }
                }] }
    ];
    TopbarBrandComponent.propDecorators = {
        alt: [{ type: Input }],
        icon: [{ type: Input }],
        image: [{ type: Input }],
        title: [{ type: Input }]
    };
    return TopbarBrandComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var TopbarMobileComponent = /** @class */ (function () {
    function TopbarMobileComponent() {
        /**
         * \@option Aria Label for close Button | 'Close Menu'
         */
        this.closeMenuAriaLabel = 'Close Menu';
        /**
         * \@option Aria Label for open Button | 'Open Menu
         */
        this.openMenuAriaLabel = 'Open Menu';
        /**
         * \@option Set mobile menu to close on any click | true
         */
        this.shouldCloseOnClick = true;
        this.isMobileOpen = false;
    }
    /**
     * @param {?} e
     * @return {?}
     */
    TopbarMobileComponent.prototype.handleKeydown = /**
     * @param {?} e
     * @return {?}
     */
    function (e) {
        if (e.code === 'Space' || e.code === 'Enter') {
            this.handleClose();
            e.preventDefault();
        }
    };
    /**
     * @return {?}
     */
    TopbarMobileComponent.prototype.handleClose = /**
     * @return {?}
     */
    function () {
        this.isMobileOpen = false;
    };
    /**
     * @return {?}
     */
    TopbarMobileComponent.prototype.handleOpen = /**
     * @return {?}
     */
    function () {
        this.isMobileOpen = true;
    };
    TopbarMobileComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-top-bar-mobile',
                    template: "\n    <cui-icon\n      *ngIf=\"!isMobileOpen\"\n      name=\"list-menu_20\"\n      buttonClassName=\"cui-top-bar__mobile-menu-button\"\n      [attr.aria-pressed]=\"isMobileOpen\"\n      [ariaLabel]=\"openMenuAriaLabel\"\n      (click)=\"handleOpen()\"\n    ></cui-icon>\n    <div\n      [ngClass]=\"[\n        'cui-top-bar__mobile',\n        'cui-tb-mobile',\n        (isMobileOpen && 'open') || ''\n      ]\"\n      (click)=\"shouldCloseOnClick ? handleClose() : null\"\n      (keydown)=\"handleKeydown($event)\"\n      role=\"menu\"\n      tabIndex=\"0\"\n    >\n      <cui-icon\n        name=\"cancel_20\"\n        buttonClassName=\"cui-tb-mobile__close\"\n        [attr.aria-pressed]=\"isMobileOpen\"\n        [ariaLabel]=\"closeMenuAriaLabel\"\n        (click)=\"handleClose()\"\n      ></cui-icon>\n      <span (click)=\"handleClose()\" (keydown)=\"handleKeydown($event)\">\n        <div class=\"cui-top-bar__brand\">\n          <ng-content select=\"brand\"></ng-content>\n        </div>\n      </span>\n      <cui-list-separator backgroundColor=\"transparent\"></cui-list-separator>\n      <nav\n        class=\"cui-tb-mobile__nav\"\n        (click)=\"!shouldCloseOnClick ? handleClose() : null\"\n      >\n        <ng-content></ng-content>\n      </nav>\n    </div>\n    <div\n      [ngClass]=\"['cui-tb-mobile__mask', (isMobileOpen && 'open') || '']\"\n      (click)=\"handleClose()\"\n      role=\"none\"\n    ></div>\n  "
                }] }
    ];
    TopbarMobileComponent.propDecorators = {
        closeMenuAriaLabel: [{ type: Input }],
        openMenuAriaLabel: [{ type: Input }],
        shouldCloseOnClick: [{ type: Input }],
        handleKeydown: [{ type: HostListener, args: ['document:keydown', ['$event'],] }]
    };
    return TopbarMobileComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var TopbarNavComponent = /** @class */ (function () {
    function TopbarNavComponent() {
    }
    TopbarNavComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-top-bar-nav',
                    template: "\n    <cui-list tabType=\"horizontal\">\n      <ng-content></ng-content>\n    </cui-list>\n  ",
                    host: {
                        class: 'cui-top-bar__nav',
                    }
                }] }
    ];
    return TopbarNavComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var TopbarRightComponent = /** @class */ (function () {
    function TopbarRightComponent() {
    }
    TopbarRightComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-top-bar-right',
                    template: "\n    <ng-content></ng-content>\n  ",
                    host: {
                        class: 'cui-top-bar__right',
                    }
                }] }
    ];
    return TopbarRightComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var TopbarModule = /** @class */ (function () {
    function TopbarModule() {
    }
    TopbarModule.decorators = [
        { type: NgModule, args: [{
                    imports: [
                        CommonModule,
                        IconModule,
                        ListModule,
                        ListItemModule,
                        ListSeparatorModule,
                    ],
                    declarations: [
                        TopbarComponent,
                        TopbarBrandComponent,
                        TopbarMobileComponent,
                        TopbarNavComponent,
                        TopbarRightComponent,
                    ],
                    exports: [
                        TopbarComponent,
                        TopbarBrandComponent,
                        TopbarMobileComponent,
                        TopbarNavComponent,
                        TopbarRightComponent,
                    ],
                },] }
    ];
    return TopbarModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var AccordionService = /** @class */ (function () {
    function AccordionService() {
        this.focus = new BehaviorSubject(0);
        this.focus$ = this.focus.asObservable();
    }
    /**
     * @param {?} idx
     * @return {?}
     */
    AccordionService.prototype.changeFocus = /**
     * @param {?} idx
     * @return {?}
     */
    function (idx) {
        this.focus.next(idx);
    };
    AccordionService.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    AccordionService.ctorParameters = function () { return []; };
    return AccordionService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var AccordionTabComponent = /** @class */ (function () {
    function AccordionTabComponent(accordion, el, accordionService, cdr) {
        this.accordion = accordion;
        this.el = el;
        this.accordionService = accordionService;
        this.cdr = cdr;
        /**
         * \@prop Set Accordion tab header string | false
         */
        this.header = '';
        /**
         * \@prop Set accordion tab to be expanded | false
         */
        this.isExpanded = false;
        /**
         * \@prop Set the attribute disabled to the accordion tab | false
         */
        this.disabled = false;
        /**
         * \@prop Optional accordion tab css class string | ''
         */
        this.tabClass = '';
        /**
         * \@prop Optional underline under Accordion menu item | false
         */
        this.showSeparator = false;
        /**
         * \@prop Optional accordion header css class string | ''
         */
        this.headerClass = '';
        /**
         * \@prop Handler to be called when the user taps the Accordion Tab | null
         */
        this.contentClick = new EventEmitter();
        /**
         * \@prop Handler to be called when the user presses a key | null
         */
        this.handleKeyDown = new EventEmitter();
    }
    /**
     * @return {?}
     */
    AccordionTabComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        var _this = this;
        if (this.height && this.height !== 56) {
            throw new Error("cui-list: Accordion Header height option must be either the default or 56");
        }
        this.accordionService.focus$.subscribe((/**
         * @param {?} focus
         * @return {?}
         */
        function (focus) { return _this.focus = focus; }));
    };
    /**
     * @return {?}
     */
    AccordionTabComponent.prototype.ngAfterViewInit = /**
     * @return {?}
     */
    function () {
        this.index = this.findTabIndex();
        this.cdr.detectChanges();
    };
    /**
     * @param {?} event
     * @return {?}
     */
    AccordionTabComponent.prototype.handleContentClick = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        this.contentClick.emit({ originalEvent: event, tab: this.accordion.tabs[this.index] });
        event.stopPropagation();
    };
    /**
     * @param {?} event
     * @return {?}
     */
    AccordionTabComponent.prototype.toggle = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        if (this.disabled) {
            return false;
        }
        this.index = this.findTabIndex();
        this.accordionService.changeFocus(this.index);
        if (this.isExpanded) {
            this.isExpanded = false;
        }
        else {
            if (!this.accordion.multipleVisible) {
                for (var i = 0; i < this.accordion.tabs.length; i++) {
                    this.accordion.tabs[i].isExpanded = false;
                }
            }
            this.isExpanded = true;
        }
        event.preventDefault();
    };
    /**
     * @return {?}
     */
    AccordionTabComponent.prototype.findTabIndex = /**
     * @return {?}
     */
    function () {
        var _this = this;
        /** @type {?} */
        var index = -1;
        this.accordion.tabs.map((/**
         * @param {?} tab
         * @param {?} idx
         * @return {?}
         */
        function (tab, idx) {
            if (tab === _this) {
                index = idx;
            }
        }));
        return index;
    };
    /**
     * @param {?} currentIndex
     * @param {?} change
     * @param {?} lastIdx
     * @return {?}
     */
    AccordionTabComponent.prototype.getNewIndex = /**
     * @param {?} currentIndex
     * @param {?} change
     * @param {?} lastIdx
     * @return {?}
     */
    function (currentIndex, change, lastIdx) {
        /** @type {?} */
        var getPossibleIndex = (/**
         * @return {?}
         */
        function () {
            if (currentIndex + change < 0) {
                return lastIdx;
            }
            else if (currentIndex + change > lastIdx) {
                return 0;
            }
            return currentIndex + change;
        });
        /** @type {?} */
        var possibleIndex = getPossibleIndex();
        if (this.accordion.tabs[possibleIndex].disabled) {
            return this.getNewIndex(possibleIndex, change, lastIdx);
        }
        return possibleIndex;
    };
    /**
     * @param {?} event
     * @return {?}
     */
    AccordionTabComponent.prototype.onKeyDown = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        /** @type {?} */
        var newIndex;
        this.handleKeyDown.emit(event.code);
        switch (event.code) {
            case 'Space':
            case 'Enter':
                this.toggle(event);
                break;
            case 'ArrowUp':
            case 'ArrowLeft':
                newIndex = this.getNewIndex(this.index, -1, this.accordion.tabs.length - 1);
                this.accordionService.changeFocus(newIndex);
                if (this.accordion.tabs[this.focus]) {
                    this.accordion.tabs[this.focus].headerRef.nativeElement.focus();
                }
                break;
            case 'Tab':
            case 'ArrowRight':
            case 'ArrowDown':
                newIndex = this.getNewIndex(this.index, 1, this.accordion.tabs.length - 1);
                this.accordionService.changeFocus(newIndex);
                if (this.accordion.tabs[this.focus]) {
                    this.accordion.tabs[this.focus].headerRef.nativeElement.focus();
                }
                break;
            case 'PageUp':
            case 'Home':
                this.accordionService.changeFocus(0);
                this.accordion.tabs[0].headerRef.nativeElement.focus();
                break;
            case 'PageDown':
            case 'End':
                this.accordionService.changeFocus(this.accordion.tabs.length - 1);
                this.accordion.tabs[this.accordion.tabs.length - 1].headerRef.nativeElement.focus();
                break;
            default:
                break;
        }
        event.preventDefault();
    };
    Object.defineProperty(AccordionTabComponent.prototype, "tabWrapperClasses", {
        get: /**
         * @return {?}
         */
        function () {
            var _a;
            return _a = {},
                _a['cui-accordion__group--disabled'] = this.disabled,
                _a['cui-accordion__group--active'] = this.isExpanded,
                _a[this.tabClass] = this.tabClass,
                _a;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AccordionTabComponent.prototype, "headerClasses", {
        get: /**
         * @return {?}
         */
        function () {
            var _a;
            return _a = {},
                _a['cui-accordion__header--' + this.height] = this.height === 56,
                _a['cui-accordion__header--separator'] = this.showSeparator,
                _a[this.headerClass] = this.headerClass,
                _a;
        },
        enumerable: true,
        configurable: true
    });
    AccordionTabComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-accordion-tab',
                    template: "\n    <div\n      [attr.aria-expanded]=\"isExpanded\"\n      class=\"cui-accordion__group\"\n      [ngClass]=\"tabWrapperClasses\"\n    >\n      <div\n        #headerRef\n        role=\"button\"\n        class=\"cui-accordion__header\"\n        [ngClass]=\"headerClasses\"\n        (click) = \"toggle($event)\"\n        (keydown)=\"onKeyDown($event)\"\n        [attr.tabindex]=\"!disabled ? 0 : -1\"\n      >\n        <div>{{header}}</div>\n        <span class=\"cui-accordion__header-icon\"></span>\n      </div>\n\n      <div class=\"cui-accordion__content\">\n        <div\n          class=\"cui-list-item\"\n          id=\"cui-list-item-12\"\n          role=\"listItem\"\n          (click) = \"handleContentClick($event)\"\n        >\n          <ng-content></ng-content>\n        </div>\n      </div>\n    </div>\n  "
                }] }
    ];
    /** @nocollapse */
    AccordionTabComponent.ctorParameters = function () { return [
        { type: AccordionComponent, decorators: [{ type: Inject, args: [forwardRef((/**
                         * @return {?}
                         */
                        function () { return AccordionComponent; })),] }] },
        { type: ElementRef },
        { type: AccordionService },
        { type: ChangeDetectorRef }
    ]; };
    AccordionTabComponent.propDecorators = {
        header: [{ type: Input }],
        isExpanded: [{ type: Input }],
        disabled: [{ type: Input }],
        tabClass: [{ type: Input }],
        height: [{ type: Input }],
        showSeparator: [{ type: Input }],
        headerClass: [{ type: Input }],
        headerRef: [{ type: ViewChild, args: ['headerRef',] }],
        contentRef: [{ type: ViewChild, args: ['contentRef',] }],
        contentClick: [{ type: Output }],
        handleKeyDown: [{ type: Output }]
    };
    return AccordionTabComponent;
}());
// Accordion Below
var AccordionComponent = /** @class */ (function () {
    function AccordionComponent() {
        /**
         * \@prop Set to allow expansion of multiple AccordionGroups | false
         */
        this.multipleVisible = false;
        /**
         * \@prop Optional css class string | ''
         */
        this.class = '';
        /**
         * \@prop Specifies if AccordionHeader automatically gets focus when page loads | false
         */
        this.focus = false;
        this.tabs = [];
    }
    /**
     * @return {?}
     */
    AccordionComponent.prototype.ngAfterContentInit = /**
     * @return {?}
     */
    function () {
        this.tabs = this.tabList.toArray();
    };
    /**
     * @return {?}
     */
    AccordionComponent.prototype.ngAfterViewInit = /**
     * @return {?}
     */
    function () {
        if (this.focus) {
            this.tabs[0].headerRef.nativeElement.focus();
        }
    };
    Object.defineProperty(AccordionComponent.prototype, "wrapperClasses", {
        get: /**
         * @return {?}
         */
        function () {
            var _a;
            return _a = {},
                _a[this.class] = this.class,
                _a;
        },
        enumerable: true,
        configurable: true
    });
    AccordionComponent.decorators = [
        { type: Component, args: [{
                    selector: 'cui-accordion',
                    template: "\n    <div\n      class=\"cui-accordion\"\n      [ngClass]=\"wrapperClasses\"\n    >\n      <ng-content></ng-content>\n    </div>\n  "
                }] }
    ];
    /** @nocollapse */
    AccordionComponent.ctorParameters = function () { return []; };
    AccordionComponent.propDecorators = {
        multipleVisible: [{ type: Input }],
        class: [{ type: Input }],
        focus: [{ type: Input }],
        tabList: [{ type: ContentChildren, args: [AccordionTabComponent,] }]
    };
    return AccordionComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var AccordionModule = /** @class */ (function () {
    function AccordionModule() {
    }
    AccordionModule.decorators = [
        { type: NgModule, args: [{
                    imports: [
                        CommonModule
                    ],
                    declarations: [AccordionComponent, AccordionTabComponent],
                    exports: [AccordionComponent, AccordionTabComponent],
                    schemas: [CUSTOM_ELEMENTS_SCHEMA],
                    providers: [AccordionService],
                },] }
    ];
    return AccordionModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { AlertBannerModule, AlertBannerComponent, AlertMeetingModule, AlertMeetingService, AlertMeetingContainerComponent, AlertMeetingConfig, AlertMeetingComponent, AlertModule, AlertService, AlertContainerComponent, AlertConfig, AlertComponent, AvatarModule, AvatarComponent, BadgeModule, BadgeComponent, ButtonModule, ButtonComponent, CheckboxModule, CheckboxComponent, CompositeAvatarModule, CompositeAvatarComponent, EditableTextfieldModule, EditableTextfieldComponent, IconModule, IconComponent, InputErrorModule, InputErrorComponent, InputHelperModule, InputHelperComponent, InputModule, InputComponent, LabelModule, LabelComponent, ListModule, ListComponent, ListItemModule, OptionSelectionChange, ListItemComponent, ListItemSectionModule, ListItemSectionComponent, ListSeparatorModule, ListSeparatorComponent, LoadingModule, LoadingComponent, ModalBodyModule, ModalBodyComponent, ModalFooterModule, ModalFooterComponent, ModalHeaderModule, ModalHeaderComponent, ModalModule, ModalComponent, RadioModule, RadioComponent, SearchInputModule, SearchInputComponent, SliderModule, SliderComponent, SliderPointerComponent, SelectModule, SelectChange, SelectComponent, SpinnerModule, SpinnerComponent, TabsModule, TabsComponent, TabModule, TabComponent, TabContentModule, TabContentComponent, TabHeaderModule, TabHeaderComponent, TabListModule, TabListComponent, TabPaneModule, TabPaneComponent, ToggleSwitchModule, ToggleSwitchComponent, TopbarModule, TopbarComponent, TopbarBrandComponent, TopbarMobileComponent, TopbarNavComponent, TopbarRightComponent, AccordionModule, AccordionTabComponent, AccordionComponent };

//# sourceMappingURL=collab-ui-angular.js.map